/*************************************************
*
* This is the generic CDL library file containing agent definitions
*
* $Id: behaviors.h,v 1.3 1996/03/20 03:28:55 doug Exp $
*
* $Log: behaviors.h,v $
 * Revision 1.3  1996/03/20  03:28:55  doug
 * *** empty log message ***
 *
 * Revision 1.2  1996/03/19  20:26:38  doug
 * *** empty log message ***
 *
 * Revision 1.1  1996/03/15  16:20:12  doug
 * Initial revision
 *
**************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

// Get definitions for ObjectClasses
#define WANT_OBJECT_DEFS
#include "cfgedit_common.h"

#if !defined(linux)
   extern "C" int strcasecmp(const char *s1, const char *s2);
#endif

#include "gt_simulation.h"
#include "cnl.h"
#include "gt_std_types.h"

//************** constants **********************************************

// The ObjectClasses parameter is a bit mask with the following fields:
#define mines 		(1<<0)
#define enemy_robots 	(1<<1)
#define flags 		(1<<2)
#define EOD_areas 	(1<<3)
#define rocks 		(1<<4)
#define trees 		(1<<5)
#define home_base	(1<<6)
#define unknown_objects	(1<<7)

// Signals
#define DANGER		'd'
#define SAFE		's'

//************  actions  ************************************************

// Kills the robot
void Terminate();

// The robot doesn't move
Vector Stop();

// Check the type of the object
Vector ProbeObject();

// move the robot to the closest object of a certain type
Vector MoveTo(ObjectClasses classes);

// move the robot away from selected types objects
Vector MoveFrom(ObjectClasses classes);

// the robot wanders about the environment
Vector Wander(double curious, double cautious);

// Pickup the closest object of the desired type
Vector PickUp(ObjectClasses classes);

// put the object into the EOD container
Vector PutInEOD();

// Change the object to the new type
Vector MarkObjectAs(ObjectClasses new_class);

// Causes the robot to drop a flag at the current location
Vector DropFlag();

// Terminate the nearest enemy robot
Vector TerminateEnemy();


//**************** triggers *********************************************


// Causes an immediate transition
bool FirstTime();

// Never take this transition
bool Never();

// start a wait cycle
void StartWait();

// Causes a transition after the delay expires
bool CheckWait(int delay);

// Are we near one of the marked objects?
bool Near(ObjectClasses classes, double distance);

// Are we away from all objects of these types?
bool AwayFrom(ObjectClasses classes, double distance);

// Is there one of the objects we are looking for?
bool Detect(ObjectClasses classes);

// Causes a transition when an object is not detected
bool UnDetect(ObjectClasses classes);

// Check the last signal sent (remains true until a new one arrives)
bool SigSense(int signal);

