#ifndef LINE_UTILS_HEADER
#define LINE_UTILS_HEADER

typedef unsigned char BOOL;

typedef struct point2d
{
  /* A point in two dimensions */
  double x, y; 
} point2d_t;

typedef struct lineseq2d
{
  /* A line segment in two dimensions*/
  point2d_t endpt[2];
} lineseg2d_t;

typedef struct line2d
{
  /* A line in two dimensions */ 
  /* If the line is vertical, then c is the x-intercept
     and isdef is set to false. Otherwise m is the slope,
     b is the y-intercept, and isdef is set to true.
  */
  double m, b, c;
  BOOL isdef;
} line2d_t;

typedef struct lineRec2d
{
  /* Records a line segment and the corresponding real
     line equation on which it lies. A next pointer is included
     to ease in  maintaining a linked list structure. 
  */
  lineseg2d_t ls;
  line2d_t ln;
  struct lineRec2d *next;
} lineRec2d_t;

typedef struct sim_obstacle
{
  double center[2];
  struct sim_obstacle *next;
} sim_obstacle_t;

extern sim_obstacle_t *newSimObstacle(point2d_t *p);
extern lineRec2d_t *newLineRec(double *pts);
extern void setLineRec(double x1, double y1, double x2, double y2, lineRec2d_t *retval);
extern int intersectTwoLinesegs(lineRec2d_t *r, lineRec2d_t *s, point2d_t *p);
extern void printLineRec(char *txt, lineRec2d_t *l);
extern double cartesianDistance(point2d_t *r, point2d_t *s);
extern sim_obstacle_t *calculateRayLineIntersections(point2d_t *origin, double heading, 
						     lineRec2d_t *lines, int *numObsDetected);
extern sim_obstacle_t *simulateSonarLineDetection(point2d_t *robotCenter, double robotHeading, 
						  lineRec2d_t *lines, int numSonar, double maxSonarRange, int *numObsDetected);
#endif


