#include "config.h"

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <assert.h>

#include "io.h"
#include "DE.h"
#include "comm_group.h"
#include "reorder_interf.h"

extern int errno;
#ifndef PRINTF_DEFINED
extern int printf ARGS((const char *,...));
#endif
#ifndef FPRINTF_DEFINED
extern int fprintf ARGS((FILE*, const char *,...));
#endif
extern int atoi();
extern char *malloc();
extern char *realloc();
extern void free();

/* global variables */
DExchange my_de = NULL;
extern int de_debug_flag;
extern int de_data_debug_flag;
buffer_recordPtr the_next_event_buffer = NULL;

static int
sensor_event_handler(de, dep, format_id, data, record_len, client_data)
DExchange de;
DEPort dep;
int format_id;
void *data;
int record_len;
void *client_data;
{
    buffer_recordPtr ev_buffer;

    ev_buffer = (buffer_recordPtr) malloc(sizeof(buffer_record));
    ev_buffer->rec = DEqueue_data(de, dep, data, format_id, record_len);

    DBG2("processing data of type %ld (orig), format_id [%d]\n",
	 get_type(ev_buffer), format_id);

    /* put to a global location */
    the_next_event_buffer = ev_buffer;
    reorder_process_record();
    return 0;			/* data was consumed, do not forward */
}

char *sensor_names[NUMBER_OF_SENSORS] =
{
    "thread_init",
    "thread_fork",
    "cthread_exit",
    "cthread_begin_join",
    "cthread_end_join",
    "cthread_detach",
    "cthread_yield",
    "cthread_set_name",
    "mutex_begin_lock",
    "mutex_end_lock",
    "mutex_unlock",
    "mutex_alloc",
    "mutex_init",
    "mutex_free",
    "mutex_clear",
    "mutex_set_name",
    "condition_alloc",
    "condition_init",
    "condition_free",
    "condition_clear",
    "condition_begin_wait",
    "condition_end_wait",
    "condition_broadcast",
    "condition_signal",
    "condition_set_name",
    "thread_off_runq",
    "processor_begin_idle",
    "processor_end_idle",
    "program_exit",
    "monitor_exit"};

int format_ids[NUMBER_OF_SENSORS];

static void
handle_incoming_format(de, dep, format_name)
DExchange de;
DEPort dep;
char *format_name;
{
    int i;

    DExchange_register_filter(de, format_name,
			      (DataHandlingFunc) sensor_event_handler, NULL);

    for (i = 0; i < NUMBER_OF_SENSORS; i++) {
	if (strcmp(format_name, sensor_names[i]) == 0) {
	    format_ids[i] = DEget_format_id(de, format_name);
	}
    }
}

int dynamic_input = True;

extern int
CMonInterfInit(iargc, iargv)
int *iargc;
char **iargv;
{
    int i, num_args, newargc = 0;
    int verbose = 0;
    int file_input = False;
    int socket_input = False;

    num_args = *iargc;
    my_de = DExchange_create();
    for (i = 1; i < num_args; i++) {
	if (!strcmp(iargv[i], "-usock_in")) {
	    if (++i + 1 > num_args) {
		fprintf(stderr, "too few arguments to %s", iargv[i - 1]);
		return -1;
	    }
	    if (DExchange_initiate_conn(my_de, iargv[i], -1, True) == NULL) {
		return -1;
	    }
	    socket_input++;
	    iargv[i] = iargv[i - 1] = NULL;
	} else if (!strcmp(iargv[i], "-isock_in")) {
	    if (++i + 2 > num_args) {
		fprintf(stderr, "too few arguments to %s", iargv[i - 1]);
		return -1;
	    }
	    if (DExchange_initiate_conn(my_de, iargv[i], atoi(iargv[i + 1]), True)
		== NULL) {
		return -1;
	    }
	    iargv[i] = iargv[i - 1] = iargv[i + 1] = NULL;
	    socket_input++;
	    i++;
	} else if (!strcmp(iargv[i], "-rfile")) {
	    if (++i + 1 > num_args) {
		fprintf(stderr, "too few arguments to %s", iargv[i - 1]);
		return -1;
	    }
	    DExchange_accept_conn_file(my_de, iargv[i], "read");
	    file_input++;
	    iargv[i] = iargv[i - 1] = NULL;
	} else if (!strcmp(iargv[i], "-wfile")) {
	    if (++i + 1 > num_args) {
		fprintf(stderr, "too few arguments to %s", iargv[i - 1]);
		return -1;
	    }
	    DExchange_accept_conn_file(my_de, iargv[i], "write");
	    iargv[i] = iargv[i - 1] = NULL;
	} else if (!strcmp(iargv[i], "-no_dynamic_link")) {
	    dynamic_input = False;
	    iargv[i] = NULL;
	} else if (!strcmp(iargv[i], "-verbose")) {
	    verbose++;
	    iargv[i] = NULL;
	} else if (!strcmp(iargv[i], "-debug")) {
	    de_data_debug_flag = True;
	    de_debug_flag = True;
	    iargv[i] = NULL;
	} else if (strcmp(iargv[i], "-app_name") == 0) {
	    if (++i + 1 > num_args) {
		fprintf(stderr, "too few arguments to %s", iargv[i - 1]);
		return -1;
	    }
	    comm_group_set_app_name(iargv[i]);
	    iargv[i] = iargv[i - 1] = NULL;
	} else if (strcmp(iargv[i], "-comm_type") == 0) {
	    if (++i + 1 > num_args) {
		fprintf(stderr, "too few arguments to %s", iargv[i - 1]);
		return -1;
	    }
	    comm_group_set_comm_type(iargv[i]);
	    iargv[i] = iargv[i - 1] = NULL;
	} else {
	    return -1;
	}
    }
    /* clean processed options */
    iargv[newargc++] = iargv[0];
    for (i = 1; i < num_args; i++) {
	if (iargv[i] != NULL)
	    iargv[newargc++] = iargv[i];
    }
    *iargc = newargc;
    if (file_input && !socket_input) {
	dynamic_input = False;
    }
    /* open inet connection */
    if (dynamic_input) {
	if (DExchange_open_inet(my_de) == -1)
	    return -1;
	if (group_server_present()) {
	    char *group_id;
	    group_id = setup_comm_group(NULL, "reorder", "reorder", 60*60,
					DExchange_host_name(my_de),
					DExchange_inet_port(my_de));
	    if (verbose) {
		printf("Registering group \"%s\"\n", group_id);
	    }
	} else {
    printf("Reorder at INET socket connection <%s, %d>\n",
		   DExchange_host_name(my_de), DExchange_inet_port(my_de));
	}
    }
    DEinit_block_check();

    for (i = 0; i < NUMBER_OF_SENSORS; i++)
	format_ids[i] = -1;
    DExchange_register_format_handler(my_de, handle_incoming_format);
    return 0;
}

extern int
same_size(p1, p2)
var_data *p1;
var_data *p2;
{
    return (p1->size == p2->size);
}

extern int 
equal(p1, p2)
var_data *p1;
var_data *p2;
{
    int i;
    assert(p1->size == p2->size);
    for (i = 0; i < p2->size; i++) {
	if (p1->data[i] != p2->data[i])
	    return 0;
    }
    return 1;
}


int thread_size = 0;

static
void
set_thread_size(size)
int size;
{
    if (thread_size < size) {
	thread_size = size;
    }
}

static
 thread_T
new_thread_id()
{
    thread_T tmp = (thread_T) malloc(sizeof(thread_T) + thread_size);
    tmp->size = thread_size;
    return tmp;
}
int mutex_size = 0;

static
void
set_mutex_size(size)
int size;
{
    if (mutex_size < size) {
	mutex_size = size;
    }
}

static
 mutex_T
new_mutex_id()
{
    mutex_T tmp = (mutex_T) malloc(sizeof(mutex_T) + mutex_size);
    tmp->size = mutex_size;
    return tmp;
}

int condition_size = 0;

static
void
set_condition_size(size)
int size;
{
    if (condition_size < size) {
	condition_size = size;
    }
}

static
 condition_t
new_condition_id()
{
    condition_t tmp = (condition_t) malloc(sizeof(condition_t) + condition_size);
    tmp->size = condition_size;
    return tmp;
}

static DERecFieldPtr child_thread_field = NULL;

thread_T
get_child_thread(rec)
buffer_recordPtr rec;
{
    thread_T child_id;

    assert(DESavedRec_format_id(rec->rec) == format_ids[DM_THREAD_FORK]);
    if (child_thread_field == NULL) {
	int field_size;
	DEget_field_info(my_de, "thread_fork", "child_thread", &field_size,
			NULL);
	set_thread_size(field_size);
	child_thread_field = DEmake_field_ptr(my_de, "thread_fork",
					     "child_thread", thread_size,
					     "unsigned integer");
    }
    child_id = new_thread_id();
    if (DEData_OK != DEget_data(my_de, (void *) &(child_id->data[0]),
				DESavedRec_get_data_ptr(rec->rec),
				child_thread_field)) {
	/* set the field info to NULL and recurse to try again */
	DEfree_field_info(child_thread_field);
	child_thread_field = NULL;
	return get_child_thread(rec);
    }
    return child_id;
}

static DERecFieldPtr *type_fields = NULL;
static int num_fields = 0;

long
get_type(rec)
buffer_recordPtr rec;
{
    long rec_type;
    int format_id = DESavedRec_format_id(rec->rec);

/*    if (type_fields == NULL) {
	type_fields = (DERecFieldPtr *) malloc(sizeof(DERecFieldPtr));
	type_fields[0] = NULL;
	num_fields = 1;
    }
    if (format_id >= num_fields) {
	int i;
	type_fields = (DERecFieldPtr *) realloc((void *) type_fields,
						 sizeof(DERecFieldPtr) *
						 (format_id + 1));
	for (i = num_fields; i <= format_id; i++) {
	    type_fields[i] = NULL;
	}
	num_fields = format_id;
    }
    if (type_fields[format_id] == NULL) {
	int field_size;
	DEget_field_info(my_de, DEget_format_name(my_de, format_id), "type",
			&field_size, NULL);
	type_fields[format_id] =
	    DEmake_field_ptr(my_de, DEget_format_name(my_de, format_id), "type",
			    sizeof(long), "integer");
    }
    if (DEData_Truncate == DEget_data(my_de, &rec_type,
				      DESavedRec_get_data_ptr(rec->rec),
				      type_fields[format_id])) {

	assert(0);
    }
    return rec_type;*/
    int i;
    for (i = 0; i < NUMBER_OF_SENSORS; i++) {
	if (format_ids[i] == format_id) return i;
    }
    return -1;

}


static DERecFieldPtr begin_lock_mutex = NULL;
static DERecFieldPtr end_lock_mutex = NULL;
static DERecFieldPtr unlock_mutex = NULL;
static DERecFieldPtr alloc_mutex = NULL;
static DERecFieldPtr init_mutex = NULL;
static DERecFieldPtr free_mutex = NULL;
static DERecFieldPtr clear_mutex = NULL;
static DERecFieldPtr name_mutex = NULL;

mutex_T
get_mutex(rec)
buffer_recordPtr rec;
{
    mutex_T mutex;

    if (DESavedRec_format_id(rec->rec) == format_ids[DM_MUTEX_BEGIN_LOCK]) {
	if (begin_lock_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "mutex_begin_lock", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    begin_lock_mutex = DEmake_field_ptr(my_de, "mutex_begin_lock",
					 "mutex", mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    begin_lock_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(begin_lock_mutex);
	    begin_lock_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_MUTEX_END_LOCK]) {
	if (end_lock_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "mutex_end_lock", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    end_lock_mutex = DEmake_field_ptr(my_de, "mutex_end_lock", "mutex",
					     mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    end_lock_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(end_lock_mutex);
	    end_lock_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_MUTEX_UNLOCK]) {
	if (unlock_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "mutex_unlock", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    unlock_mutex = DEmake_field_ptr(my_de, "mutex_unlock", "mutex",
					   mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    unlock_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(unlock_mutex);
	    unlock_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_MUTEX_ALLOC]) {
	if (alloc_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "mutex_alloc", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    alloc_mutex = DEmake_field_ptr(my_de, "mutex_alloc", "mutex",
					  mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    alloc_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(alloc_mutex);
	    alloc_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_MUTEX_INIT]) {
	if (init_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "mutex_init", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    init_mutex = DEmake_field_ptr(my_de, "mutex_init", "mutex",
					 mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    init_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(init_mutex);
	    init_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_MUTEX_FREE]) {
	if (free_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "mutex_free", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    free_mutex = DEmake_field_ptr(my_de, "mutex_free", "mutex",
					 mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    free_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(free_mutex);
	    free_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_MUTEX_CLEAR]) {
	if (clear_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "mutex_clear", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    clear_mutex = DEmake_field_ptr(my_de, "mutex_clear", "mutex",
					  mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    clear_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(clear_mutex);
	    clear_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_MUTEX_SET_NAME]) {
	if (name_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "mutex_set_name", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    name_mutex = DEmake_field_ptr(my_de, "mutex_set_name", "mutex",
					 mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    name_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(name_mutex);
	    name_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_CONDITION_BEGIN_WAIT]) {
	if (name_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "condition_begin_wait", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    name_mutex = DEmake_field_ptr(my_de, "condition_begin_wait", "mutex",
					 mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    name_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(name_mutex);
	    name_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_CONDITION_END_WAIT]) {
	if (name_mutex == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "condition_end_wait", "mutex", &field_size,
			    NULL);
	    set_mutex_size(field_size);
	    name_mutex = DEmake_field_ptr(my_de, "condition_end_wait", "mutex",
					 mutex_size, "unsigned integer");
	}
	mutex = new_mutex_id();
	if (DEData_OK != DEget_data(my_de, &(mutex->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    name_mutex)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(name_mutex);
	    name_mutex = NULL;
	    free(mutex);
	    return get_mutex(rec);
	}
	return mutex;
    } else {
	assert(0);
	/*NOTREACHED*/
	return NULL;
    }
}

static DERecFieldPtr begin_join_to_field = NULL;
static DERecFieldPtr end_join_to_field = NULL;

thread_T
get_join_to(rec)
buffer_recordPtr rec;
{
    thread_T join_to;

    if (DESavedRec_format_id(rec->rec) == format_ids[DM_THREAD_BEGIN_JOIN]) {
	if (begin_join_to_field == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "cthread_begin_join", "join_to", &field_size,
			    NULL);
	    set_thread_size(field_size);
	    begin_join_to_field = DEmake_field_ptr(my_de, "cthread_begin_join",
						  "join_to", thread_size,
						  "unsigned integer");
	}
	join_to = new_thread_id();
	if (DEData_OK != DEget_data(my_de, (void *) &(join_to->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    begin_join_to_field)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(begin_join_to_field);
	    begin_join_to_field = NULL;
	    free(join_to);
	    return get_join_to(rec);
	}
    } else if (DESavedRec_format_id(rec->rec) == format_ids[DM_THREAD_END_JOIN]) {
	if (end_join_to_field == NULL) {
	    int field_size;
	    DEget_field_info(my_de, "cthread_end_join", "join_to", &field_size,
			    NULL);
	    set_thread_size(field_size);
	    end_join_to_field = DEmake_field_ptr(my_de, "cthread_end_join",
						"join_to", thread_size,
						"unsigned integer");
	}
	join_to = new_thread_id();
	if (DEData_OK != DEget_data(my_de, (void *) &(join_to->data[0]),
				    DESavedRec_get_data_ptr(rec->rec),
				    end_join_to_field)) {
	    /* set the field info to NULL and recurse to try again */
	    DEfree_field_info(end_join_to_field);
	    end_join_to_field = NULL;
	    free(join_to);
	    return get_join_to(rec);
	}
    } else {
	assert(0);
    }
    return join_to;
}


static DERecFieldPtr *seq_num_fields = NULL;
static int num_seq_num_fields = 0;

seq_num
get_seq_num(rec)
buffer_recordPtr rec;
{
    seq_num seq;
    int format_id = DESavedRec_format_id(rec->rec);

    if (seq_num_fields == NULL) {
	seq_num_fields = (DERecFieldPtr *) malloc(sizeof(DERecFieldPtr));
	num_seq_num_fields = 1;
    }
    if (format_id >= num_seq_num_fields) {
	int i;
	seq_num_fields = (DERecFieldPtr *) realloc((void *) seq_num_fields,
						 sizeof(DERecFieldPtr) *
						    (format_id + 1));
	for (i = num_seq_num_fields; i <= format_id; i++) {
	    seq_num_fields[i] = NULL;
	}
	num_seq_num_fields = format_id;
    }
    if (seq_num_fields[format_id] == NULL) {
	int field_size;
	DEget_field_info(my_de, DEget_format_name(my_de, format_id),
			"sequence_number", &field_size, NULL);
	seq_num_fields[format_id] =
	    DEmake_field_ptr(my_de, DEget_format_name(my_de, format_id),
			  "sequence_number", sizeof(seq_num), "integer");
    }
    if (DEData_Truncate == DEget_data(my_de, &seq, DESavedRec_get_data_ptr(rec->rec),
				      seq_num_fields[format_id])) {

	/* seq_num didn't fit in long???? */
	assert(0);
    }
    return seq;
}

static DERecFieldPtr *thread_fields = NULL;
static int num_thread_fields = 0;

thread_T
get_thread(rec)
buffer_recordPtr rec;
{
    thread_T thread;
    int format_id = DESavedRec_format_id(rec->rec);

    if (thread_fields == NULL) {
	thread_fields = (DERecFieldPtr *) malloc(sizeof(DERecFieldPtr));
	num_thread_fields = 1;
    }
    if (format_id >= num_thread_fields) {
	int i;
	thread_fields = (DERecFieldPtr *) realloc((void *) thread_fields,
						 sizeof(DERecFieldPtr) *
						   (format_id + 1));
	for (i = num_thread_fields; i <= format_id; i++) {
	    thread_fields[i] = NULL;
	}
	num_thread_fields = format_id + 1;
    }
    if (thread_fields[format_id] == NULL) {
	int field_size;
	DEget_field_info(my_de, DEget_format_name(my_de, format_id), "thread",
			&field_size, NULL);
	set_thread_size(field_size);
	thread_fields[format_id] =
	    DEmake_field_ptr(my_de, DEget_format_name(my_de, format_id),
			    "thread", thread_size, "unsigned integer");
    }
    thread = new_thread_id();
    if (DEData_Truncate == DEget_data(my_de, (void *) &(thread->data[0]),
				      DESavedRec_get_data_ptr(rec->rec),
				      thread_fields[format_id])) {
	/* set the field info to NULL and recurse to try again */
	DEfree_field_info(thread_fields[format_id]);
	thread_fields[format_id] = NULL;
	free(thread);
	return get_thread(rec);
    }
    return thread;
}

static DERecFieldPtr begin_seq_num_field = NULL;

seq_num
get_begin_seq_num(rec)
buffer_recordPtr rec;
{
    seq_num seq;
    assert(DESavedRec_format_id(rec->rec) == format_ids[DM_CONDITION_BROADCAST]);
    if (begin_seq_num_field == NULL) {
	int field_size;
	DEget_field_info(my_de, "condition_broadcast", "begin_sequence_number",
			&field_size, NULL);
	begin_seq_num_field = DEmake_field_ptr(my_de, "condition_broadcast",
					      "begin_sequence_number",
					      sizeof(seq_num), "integer");
    }
    if (DEData_Truncate == DEget_data(my_de, (void *) &seq,
				      DESavedRec_get_data_ptr(rec->rec),
				      begin_seq_num_field)) {
	/* seq numbers how large??? */
	assert(0);
    }
    return seq;
}

static DERecFieldPtr end_seq_num_field = NULL;

seq_num
get_end_seq_num(rec)
buffer_recordPtr rec;
{
    seq_num seq;
    assert(DESavedRec_format_id(rec->rec) == format_ids[DM_CONDITION_BROADCAST]);
    if (end_seq_num_field == NULL) {
	int field_size;
	DEget_field_info(my_de, "condition_broadcast", "end_sequence_number",
			&field_size, NULL);
	end_seq_num_field = DEmake_field_ptr(my_de, "condition_broadcast",
					    "end_sequence_number",
					    sizeof(seq_num), "integer");
    }
    if (DEData_Truncate == DEget_data(my_de, (void *) &seq,
				      DESavedRec_get_data_ptr(rec->rec),
				      end_seq_num_field)) {
	/* seq numbers how large??? */
	assert(0);
    }
    return seq;
}

static DERecFieldPtr *condition_fields = NULL;
static int num_condition_fields = 0;

condition_t
get_condition(rec)
buffer_recordPtr rec;
{
    condition_t condition;
    int format_id = DESavedRec_format_id(rec->rec);

    if (condition_fields == NULL) {
	condition_fields = (DERecFieldPtr *) malloc(sizeof(DERecFieldPtr));
	condition_fields[0] = NULL;
	num_condition_fields = 1;
    }
    if (format_id >= num_condition_fields) {
	int i;
	condition_fields = (DERecFieldPtr *) realloc((void *) condition_fields,
						 sizeof(DERecFieldPtr) *
						      (format_id + 1));
	for (i = num_condition_fields; i <= format_id; i++) {
	    condition_fields[i] = NULL;
	}
	num_condition_fields = format_id;
    }
    if (condition_fields[format_id] == NULL) {
	int field_size;
	DEget_field_info(my_de, DEget_format_name(my_de, format_id),
			"condition", &field_size, NULL);
	set_condition_size(field_size);
	condition_fields[format_id] =
	    DEmake_field_ptr(my_de, DEget_format_name(my_de, format_id),
			    "condition", condition_size, "unsigned integer");
    }
    condition = new_condition_id();
    if (DEData_Truncate == DEget_data(my_de, &(condition->data[0]),
				      DESavedRec_get_data_ptr(rec->rec),
				      condition_fields[format_id])) {
	/* set the field info to NULL and recurse to try again */
	DEfree_field_info(condition_fields[format_id]);
	condition_fields[format_id] = NULL;
	free(condition);
	return get_condition(rec);
    }
    return condition;
}
