/**********************************************************************
 **                                                                  **
 **                             cbrplanner.h                         **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2003 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef CBRPLANNER_H
#define CBRPLANNER_H

/* $Id: cbrplanner.h,v 1.1 2003/04/06 08:43:44 endo Exp $ */

#include <string>
#include <vector>

#include "domain_manager.h"
#include "memory_manager.h"
#include "planner.h"

class CBRPlanner {

protected:
    CBRPlanner **self_;
    DomainManager *domainManager_;
    MemoryManager *memoryManager_;
    Planner *planner_;

    static const int MAINLOOP_USLEEPTIME_;

public:
    CBRPlanner(void);
    CBRPlanner(CBRPlanner **self);
    ~CBRPlanner(void);

    MemoryManager *memoryManager(void);
    void startSubsystems(void);
    void mainLoop(void);
    vector<int> saveNewMissionPlan(
	CBRPlanner_ACDLMissionPlan_t missionplan,
	MemoryManager::Features_t features);
    void updateFSANames(CBRPlanner_FSANames_t fsaNames);
    CBRPlanner_ACDLMissionPlan_t getACDLMissionPlan(
	MemoryManager::Features_t features);
    vector<CBRPlanner_ACDLMissionPlan_t> getACDLMissionPlans(
	MemoryManager::Features_t features, int maxRating);
    void loadCBRLibrary(string cbrlibraryName);
    void loadCBRLibrary(void);
    void saveCBRLibrary(void);
    bool confirmClear(void);
    bool memoryChanged(void);
    void givePositiveFeedback(vector<int> dataIndexList);
    void giveNegativeFeedback(vector<int> dataIndexList);
};

inline MemoryManager *CBRPlanner::memoryManager(void)
{
    return memoryManager_;
};

inline void CBRPlanner::loadCBRLibrary(string cbrlibraryName)
{
    memoryManager_->loadCBRLibrary(cbrlibraryName);
};

inline void CBRPlanner::loadCBRLibrary(void)
{
    memoryManager_->loadCBRLibrary();
};

inline void CBRPlanner::saveCBRLibrary(void)
{
    memoryManager_->saveCBRLibrary();
};

inline bool CBRPlanner::confirmClear(void)
{
    return (memoryManager_->confirmClear());
};

inline bool CBRPlanner::memoryChanged(void)
{
    return (memoryManager_->memoryChanged());
};

inline void CBRPlanner::givePositiveFeedback(vector<int> dataIndexList)
{
    memoryManager_->givePositiveFeedback(dataIndexList);
};

inline void CBRPlanner::giveNegativeFeedback(vector<int> dataIndexList)
{
    memoryManager_->giveNegativeFeedback(dataIndexList);
};

extern CBRPlanner *gCBRPlanner;

#endif
/**********************************************************************
 * $Log: cbrplanner.h,v $
 * Revision 1.1  2003/04/06 08:43:44  endo
 * Initial revision
 *
 *
 **********************************************************************/
