/**********************************************************************
 **                                                                  **
 **                      cbrplanner_protocol.h                       **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  This header file contains constants and structures for the      **
 **  socket communication between CBRServer and CBRClient.           **
 **                                                                  **
 **  Copyright 2003 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef CBRPLANNER_PROTOCOL_H
#define CBRPLANNER_PROTOCOL_H

/* $Id: cbrplanner_protocol.h,v 1.2 2003/04/06 15:26:49 endo Exp $ */

#include <string>
#include <vector>

using std::string;
using std::vector;

#define CBRPLANNER_COMM_MAX_BUFSIZE (4096)

#define CBRPLANNER_COMM_SYNC_BYTE1 (0x01)
#define CBRPLANNER_COMM_SYNC_BYTE2 (0x02)

#define CBRPLANNER_COMM_CMD_CLIENTINIT (0x11)
#define CBRPLANNER_COMM_CMD_CLIENTEXIT (0x12)
#define CBRPLANNER_COMM_CMD_FSANAMES (0x13)
#define CBRPLANNER_COMM_CMD_MISSIONPLAN_SAVE (0x14)
#define CBRPLANNER_COMM_CMD_MISSIONPLAN_RETRIEVE (0x15)
#define CBRPLANNER_COMM_CMD_SELECTED_MISSION (0x16)
#define CBRPLANNER_COMM_CMD_POSITIVE_FEEDBACK (0x17)
#define CBRPLANNER_COMM_CMD_NEGATIVE_FEEDBACK (0x18)

typedef enum CBRPlanner_ClientType_t
{
    CBRPLANNER_CLIENT_CFGEDIT,
    CBRPLANNER_CLIENT_MLAB
};

typedef enum CBRPlanner_MissionPlanType_t
{
    CBRPLANNER_MISSIONPLAN_EMPTY,
    CBRPLANNER_MISSIONPLAN_NEW,
    CBRPLANNER_MISSIONPLAN_OLD,
    CBRPLANNER_MISSIONPLAN_CONSTRUCTED
};

#define CBRPLANNER_CLIENT_NAME_MAX 256

typedef struct CBRPlanner_ClientInfo_t
{
    int type;
    int pid;
    char name[CBRPLANNER_CLIENT_NAME_MAX];
};

typedef struct CBRPlanner_MissionPlan_t
{
    int type;
    int rating;
    vector<int> dataIndexList;
    int dataSize;
    char *data;
};

typedef struct CBRPlanner_Feature_t
{
    string name;
    string value;
    bool nonIndex;
    double weight;
};

#endif
/**********************************************************************
 * $Log: cbrplanner_protocol.h,v $
 * Revision 1.2  2003/04/06 15:26:49  endo
 * gcc 3.1.1
 *
 * Revision 1.1  2003/04/06 08:43:44  endo
 * Initial revision
 *
 *
 **********************************************************************/
