/**********************************************************************
 **                                                                  **
 **                           communicator.h                         **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2003 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef COMMUNICATOR_H
#define COMMUNICATOR_H

/* $Id: communicator.h,v 1.1 2003/04/06 08:43:44 endo Exp $ */

#include <string>
#include <vector>
#include <pthread.h>

#include "cbrplanner_protocol.h"
#include "cbrplanner_domain.h"
#include "domain_manager.h"
#include "memory_manager.h"

class Communicator {
    
    typedef struct PrivateConnectionInfo_t
    {
	int fd;
	CBRPlanner_ClientInfo_t clientInfo;
	pthread_t connectionThread;
	bool connectionThreadIsUp;
    };

    typedef struct PrivateConnectionThreadInput_t
    {
	int fd;
	Communicator *communicatorInstance;
    };


protected:
    Communicator **self_;

    string socketName_;
    pthread_t daemonThread_;
    int socketFd_;
    bool daemonThreadIsUp_;
    vector<PrivateConnectionInfo_t> shPrivateConnectionInfoList_;

    static pthread_mutex_t privateConnectionInfoListMutex_;

    static const int SOCKET_BACKLOG_SIZE_;
    static const int SELECT_WAIT_SEC_;
    static const int MAX_SYNC_ERROR_COUNT_;

    void setupSocket_(void);
    void daemonMainLoop_(void);
    int waitForNewConnection_(int count);
    void startPrivateConnection_(const int fd);
    void privateConnectionMainLoop_(const int fd);
    bool readAndProcessPrivateConnectionData_(const int fd);
    bool readNBytes_(const int fd, unsigned char *buf, const int n);
    bool syncToClient_(const int fd);
    void setConnectionThreadIsUp_(const int fd, const bool isUp);
    void saveConnectionThread_(const int fd, const pthread_t connectionThread);
    pthread_t getConnectionThread_(const int fd);
    void disconnectPrivateConnection_(const int fd);
    void disconnectAllPrivateConnections_(void);
    CBRPlanner_ClientInfo_t readClientInfo_(const int fd);
    void updateClientInfo_(const int fd, const CBRPlanner_ClientInfo_t clientInfo);
    CBRPlanner_ClientInfo_t getClientInfo_(const int fd);
    CBRPlanner_ACDLMissionPlan_t readACDLMissionPlan_(const int fd);
    void writeACDLMissionPlans_(
	const int fd,
	vector<CBRPlanner_ACDLMissionPlan_t> acdlPlan);
    MemoryManager::Features_t readFeatures_(const int fd);
    int readInteger_(const int fd);
    bool readBoolean_(const int fd);
    CBRPlanner_FSANames_t readFSANames_(const int fd);
    vector<int> readDataIndexList_(const int fd);
    void writeDataIndexList_(const int fd, vector<int> dataIndexList);

    static void *startDaemonThread_(void *communicatorInstance);
    static void *startPrivateConnectionThread_(void *input);

public:
    Communicator(void);
    Communicator(Communicator **self);
    ~Communicator(void);
    void startDaemon(void);
    void setSocketName(string name);
};

inline void Communicator::setSocketName(string name)
{
    socketName_ = name;
};

extern Communicator *gCommunicator;

#endif
/**********************************************************************
 * $Log: communicator.h,v $
 * Revision 1.1  2003/04/06 08:43:44  endo
 * Initial revision
 *
 *
 **********************************************************************/
