/**********************************************************************
 **                                                                  **
 **                              debugger.h                          **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2003 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef DEBUGGER_H
#define DEBUGGER_H

/* $Id: debugger.h,v 1.1 2003/04/06 08:43:44 endo Exp $ */

#include <stdio.h>
#include <string>

using std::string;

//-----------------------------------------------------------------------
// Debugger class
//-----------------------------------------------------------------------
class Debugger {

public:
    typedef enum OutputType_t {
	DEBUG_FILE,
	DEBUG_STDOUT
    };

protected:
    Debugger **self_;
    string debugFilename_;
    FILE *debugFile_;
    FILE *out_;
    int outputType_;
    bool debug_;

    static const string DEBUG_FILENAME_EXTENSION_;
    static const string EMPTY_STRING_;
    
    string createDebugFilenamePrefix_(void);
    void setDefaultDebugFilename_(void);
    void openDebugFile_(void);

public:
    Debugger(void);
    Debugger(Debugger **self);
    ~Debugger(void);
    void printfDebug(const char *format, ...);
    void printfDebug(int outputType, const char *format, ...);
    void printfLine(void);
    void setOutputType(int outputType);
    void enableDebug(void);
    void enableDebug(int outputType);
    void disableDebug(void);
};

inline void Debugger::enableDebug(void)
{
    debug_ = true;
};

inline void Debugger::disableDebug(void)
{
    debug_ = false;
};

extern Debugger *gDebugger;

#endif
/**********************************************************************
 * $Log: debugger.h,v $
 * Revision 1.1  2003/04/06 08:43:44  endo
 * Initial revision
 *
 *
 **********************************************************************/
