/**********************************************************************
 **                                                                  **
 **                           domain_manager.h                       **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2003 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef DOMAIN_MANAGER_H
#define DOMAIN_MANAGER_H

/* $Id: domain_manager.h,v 1.1 2003/04/06 08:43:44 endo Exp $ */

#include <vector>
#include <string>

#include "cbrplanner_protocol.h"
#include "cbrplanner_domain.h"
#include "memory_manager.h"

using std::vector;
using std::string;

class DomainManager {

protected:
    DomainManager **self_;

    static const string ACDL_STRING_BIND_ARCH_;
    static const string ACDL_STRING_INSTANCE_INDEX_;
    static const string ACDL_STRING_INSTANCE_NAME_;
    static const string ACDL_STRING_INSTANCE_;
    static const string ACDL_STRING_TASK_INDEX_;
    static const string ACDL_STRING_AGENT_NAME_;
    static const string ACDL_STRING_TRIGGERING_TASK_INDEX_;
    static const string ACDL_STRING_PARM_NAME_;
    static const string ACDL_STRING_PARM_VALUE_;
    static const string ACDL_STRING_IF_;
    static const string ACDL_STRING_START_;
    static const string ACDL_STRING_GOTO_;
    static const string ACDL_STRING_END_;
    static const string ACDL_STRING_MOVEMENT_;
    static const string ACDL_STRING_STARTTASK_INDEX_;
    static const string ACDL_STRING_MAX_VELOCITY_;
    static const string ACDL_STRING_BASE_VELOCITY_;
    static const string ACDL_STRING_START_SUB_MISSION_;

    void addEffect_(
	vector<CBRPlanner_PlanOperator_t> *plan,
	string operatorIndex,
	CBRPlanner_Effect_t effect);
    void addPrecondition_(
	vector<CBRPlanner_PlanOperator_t> *plan,
	string operatorIndex,
	CBRPlanner_Precondition_t precondition);
    vector<CBRPlanner_PlanOperator_t> addOperatorToPlanSequence_(
	vector<CBRPlanner_PlanOperator_t> planSequence,
	CBRPlanner_PlanOperator_t planOperator);
    void dumpPlanSequence_(CBRPlanner_PlanSequence_t planSequence);
    bool hasAgentInMissionPlan_(
	CBRPlanner_ACDLMissionPlan_t acdlPlan,
	string agentName);

public:
    DomainManager(void);
    DomainManager(DomainManager **self);
    ~DomainManager(void);

    void updateFSANames(CBRPlanner_FSANames_t fsaNames);
    CBRPlanner_ACDLMissionPlan_t missionplan2acdl(
	MemoryManager::MissionPlan_t missionplan);
    MemoryManager::MissionPlan_t acdl2missionplan(
	CBRPlanner_ACDLMissionPlan_t acdlPlan);
    bool hasStartSubMision(CBRPlanner_ACDLMissionPlan_t acdlPlan);
};

inline bool DomainManager::hasStartSubMision(
    CBRPlanner_ACDLMissionPlan_t acdlPlan)
{
    return (hasAgentInMissionPlan_(acdlPlan, ACDL_STRING_START_SUB_MISSION_));
};

#endif
/**********************************************************************
 * $Log: domain_manager.h,v $
 * Revision 1.1  2003/04/06 08:43:44  endo
 * Initial revision
 *
 *
 **********************************************************************/
