/**********************************************************************
 **                                                                  **
 **                              planner.h                           **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2002 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef PLANNER_H
#define PLANNER_H

/* $Id: planner.h,v 1.1 2003/04/06 08:43:44 endo Exp $ */

#include <string>

#include "memory_manager.h"

using std::string;

class Planner {

protected:
    Planner **self_;

    static const string EMPTY_STRING_;
    static const string STRING_EMPTY_ARCHITECTURE_;
    static const string STRING_START_STATE_;
    static const string STRING_START_STATE_INDEX_;
    static const string STRING_IMMEDIATE_;
    static const string STRING_START_SUB_MISSION_;
    static const string STRING_START_SUB_MISSION_INDEX_;
    static const string STRING_SUB_MISSION_NAME_;
    static const string STRING_GOAL_LOCATION_;
    static const string STRING_MOVE_TO_LOCATION_GAIN_;
    static const string STRING_WANDER_GAIN_;
    static const string STRING_AVOID_OBSTACLE_GAIN_;
    static const string STRING_AVOID_OBSTACLE_SPHERE_;
    static const string STRING_AVOID_OBSTACLE_SAFETY_MARGIN_;
    static const string STRING_SUB_MISSION_READY_;
    static const string STRING_SUB_MISSION_READY_INDEX_;
    static const string STRING_SUB_MISSION_DEPLOYMENT_;
    static const string STRING_DEPLOY_FIRST_;
    static const string STRING_EXECUTE_IMMEDIATELY_;
    static const string STRING_GOAL_TOLERANCE_;
    static const string FEATURE_NAME_MAX_VELOCITY_;
    static const string FEATURE_NAME_AGGRESSIVENESS_;
    static const string FEATURE_NAME_NUM_ROBOTS_;
    static const string FEATURE_NAME_START_X_;
    static const string FEATURE_NAME_START_Y_;
    static const string FEATURE_NAME_TASK_NAME_;
    static const string FEATURE_NAME_USE_LANDMARK_;
    static const string FEATURE_NAME_LOCALIZATION_;
    static const string FEATURE_NAME_ENVIRONMENT_;
    static const string FEATURE_VALUE_USEMAPS_;
    static const string FEATURE_VALUE_INDOOR_;
    static const double DEFAULT_MOVE_TO_LOCATION_GAIN_;
    static const double DEFAULT_WANDER_GAIN_;
    static const double DEFAULT_INDOOR_AVOID_OBSTACLE_GAIN_;
    static const double DEFAULT_OUTDOOR_AVOID_OBSTACLE_GAIN_;
    static const double DEFAULT_INDOOR_AVOID_OBSTACLE_SPHERE_;
    static const double DEFAULT_OUTDOOR_AVOID_OBSTACLE_SPHERE_;
    static const double DEFAULT_AVOID_OBSTACLE_SAFETY_MARGIN_;
    static const double DEFAULT_GOAL_TOLERANCE_;

    string extractMaxVelocityValue_(
	MemoryManager::Features_t features);
    string extractAggressivenessValue_(
	MemoryManager::Features_t features);
    int extractNumRobots_(MemoryManager::Features_t features);
    vector<MemoryManager::Features_t> extractSubFeatures_(
	MemoryManager::Features_t features);
    MemoryManager::Coordinates_t extractCoordinates_(
	MemoryManager::Features_t features);
    string extractTaskName_(MemoryManager::Features_t features);
    bool extractUseMaps_(MemoryManager::Features_t features);
    bool extractIndoor_(MemoryManager::Features_t features);
    MemoryManager::Features_t filterNonIndexFeatures_(
	MemoryManager::Features_t features);
    MemoryManager::Features_t sortFeatures_(
	MemoryManager::Features_t features);
    MemoryManager::MissionPlan_t addStartSubMissionState_(
	MemoryManager::MissionPlan_t missionPlan,
	string taskName,
	MemoryManager::Coordinates_t coordinates,
	bool useMaps,
	bool indoor,
	int startSubMissionOperatorIndex);
    MemoryManager::MissionPlan_t addStartState_(
	MemoryManager::MissionPlan_t missionPlan);
    CBRPlanner_PlanOperator_t createStartOperator_(void);
    CBRPlanner_PlanOperator_t createStartSubMissionOperator_(
	MemoryManager::MissionPlan_t missionPlan,
	string taskName,
	MemoryManager::Coordinates_t coordinates,
	bool useMaps,
	bool indoor,
	int startSubMissionOperatorIndex);
    vector<MemoryManager::MissionPlan_t> sortMissionPlans_(
	vector<MemoryManager::MissionPlan_t> missionPlans);

    vector<MemoryManager::MissionPlan_t> appendMissionPlans_(
	vector<MemoryManager::MissionPlan_t> bigMissionPlans,
	vector<MemoryManager::MissionPlan_t> missionPlans,
	int trackingNumber);
    MemoryManager::MissionPlan_t adjustNumRobots_(
	MemoryManager::MissionPlan_t missionPlan,
	int numRobots);
    vector<int> mergeDataIndexLists_(
	vector<int> dataIndexListA,
	vector<int> dataIndexListB);
    vector<int> trimRedundantDataIndex_(vector<int> dataIndexList);
    MemoryManager::MissionPlanList_t separateSubMissions_(
	MemoryManager::MissionPlan_t missionPlan,
	MemoryManager::Features_t features,
	vector<MemoryManager::Features_t> *featuresList);
    MemoryManager::Features_t renameTaskInFeatures_(
	MemoryManager::Features_t features,
	string taskName);
    string getTaskNameInFeatures_(MemoryManager::Features_t features);
    string extractTaskNameFromStartSubMission_(
	CBRPlanner_PlanOperator_t startSubMissionOperator);
    string extractValueContent_(string value, bool withoutQuote);

public:
    Planner(void);
    Planner(Planner **self);
    ~Planner(void);
    vector<MemoryManager::MissionPlan_t> getMissionPlans(
	MemoryManager::Features_t features,
	int maxRating);
    vector<int> saveNewMissionPlan(
	MemoryManager::MissionPlan_t missionPlan,
	MemoryManager::Features_t features,
	bool hasStartSubMission);
 };

#endif
/**********************************************************************
 * $Log: planner.h,v $
 * Revision 1.1  2003/04/06 08:43:44  endo
 * Initial revision
 *
 *
 **********************************************************************/
