/**********************************************************************
 **                                                                  **
 **                              windows.h                           **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2003 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef WINDOWS_H
#define WINDOWS_H

/* $Id: windows.h,v 1.2 2003/04/06 15:29:47 endo Exp $ */

#include <curses.h>
#include <panel.h>
#include <string>

using std::string;

//-----------------------------------------------------------------------
// Typedefs
//-----------------------------------------------------------------------
typedef struct WindowSize_t {
    int x;
    int y;
};

typedef enum CursorVisibility_t {
    CURSOR_HIDE,
    CURSOR_NORMAL,
    CURSOR_VERYVISIBLE
};

//-----------------------------------------------------------------------
// Subwindow class
//-----------------------------------------------------------------------
class Subwindow {

protected:
    Subwindow **self_;
    WINDOW *titleWindow_;
    WINDOW *window_;
    PANEL *titlePanel_;
    PANEL *panel_;
    string title_;
    WindowSize_t mainWindowSize_;
    int startPosY_;
    int height_;

    static const int LINE_OFFSET_;
    static const int MOVE_OFFSET_;
    static const int TITLE_HEIGHT_;
    static const int MAX_BUFSIZE_;
    static const int SCAN_USLEEPTIME_;
    static const int KEYNUM_BACKSPACE_;
    static const int KEYNUM_ENTER_;
    static const string EMPTY_STRING_;

    void printWindow_(char *str);
    void redrawWindow_(void);

public:
    Subwindow(void);
    Subwindow(Subwindow **self);
    ~Subwindow(void);
    void createWindow(
	WindowSize_t mainWindowSize,
	int startPosY,
	int height,
	string title);
    void printfWindow(const char *format, ...);
    void printWindow(char *str);
    void clearWindow(void);
    void redrawWindow(void);
    void setTitle(string title);
    string scanfWindow(void);
};

inline void Subwindow::printWindow(char *str)
{
    printWindow_(str);
};

inline void Subwindow::redrawWindow(void)
{
    redrawWindow_();
};

//-----------------------------------------------------------------------
// Windows class
//-----------------------------------------------------------------------
class Windows {

protected:
    Windows **self_;
    WINDOW *mainWindow_;
    WINDOW *backWindow_;
    PANEL *backPanel_;
    WindowSize_t mainWindowSize_;
    Subwindow *plannerWindow_;
    Subwindow *commWindow_;
    Subwindow *keyboardWindow_;

    static const int COMM_WINDOW_START_POS_Y_;
    static const int PLANNER_WINDOW_START_POS_Y_;
    static const int KEYBOARD_WINDOW_HEIGHT_;
    static const int LINE_OFFSET_;
    static const int MOVE_OFFSET_;
    static const int MAX_BUFSIZE_;
    static const string EMPTY_STRING_;
    static const string MAIN_TITLE_;
    static const string COMM_WINDOW_TITLE_;
    static const string PLANNER_WINDOW_TITLE_;

    void initWindows_(void);
    void createMainWindow_(void);
    void createBackWindow_(void);
    void redrawWindows_(void);

public:
    Windows(void);
    Windows(Windows **self);
    ~Windows(void);
    void startWindows(void);
    void printfPlannerWindow(const char *format, ...);
    void printPlannerWindow(char *str);
    void clearPlannerWindow(void);
    void printfCommWindow(const char *format, ...);
    void printCommWindow(char *str);
    void clearCommWindow(void);
    void printfKeyboardWindow(const char *format, ...);
    void printKeyboardWindow(char *str);
    void setKeyboardWindowTitle(string title);
    void clearKeyboardWindow(void);
    string scanfKeyboardWindow(void);
};

inline void Windows::printPlannerWindow(char *str)
{
    plannerWindow_->printWindow(str);
};

inline void Windows::clearPlannerWindow(void)
{
    plannerWindow_->clearWindow();
};

inline void Windows::printCommWindow(char *str)
{
    commWindow_->printWindow(str);
};

inline void Windows::clearCommWindow(void)
{
    commWindow_->clearWindow();
};

inline void Windows::printKeyboardWindow(char *str)
{
    keyboardWindow_->printWindow(str);
};

inline void Windows::setKeyboardWindowTitle(string title)
{
    keyboardWindow_->setTitle(title);
};

inline void Windows::clearKeyboardWindow(void)
{
    keyboardWindow_->clearWindow();
};

inline string Windows::scanfKeyboardWindow(void)
{
    return (keyboardWindow_->scanfWindow());
};

extern Windows *gWindows;

#endif
/**********************************************************************
 * $Log: windows.h,v $
 * Revision 1.2  2003/04/06 15:29:47  endo
 * gcc 3.1.1
 *
 * Revision 1.1  2003/04/06 08:43:44  endo
 * Initial revision
 *
 *
 **********************************************************************/
