/**********************************************************************
 **                                                                  **
 **                          mission_expert.cc                       **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2002 - 2003 Georgia Tech Research Corporation         **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: mission_expert.cc,v 1.3 2003/04/06 08:50:37 endo Exp $ */

//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <signal.h>
#include <memory.h>
#include <malloc.h>
#include <time.h>
#include <unistd.h>
#include <fstream.h>
#include <iostream.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <string>
#include <vector>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/Intrinsic.h>

#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/ArrowBG.h>
#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>   
#include <Xm/CascadeBG.h>   
#include <Xm/DialogS.h>
#include <Xm/DrawingA.h>
#include <Xm/FileSB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/List.h>
#include <Xm/MainW.h>
#include <Xm/MessageB.h>
#include <Xm/MenuShell.h>
#include <Xm/PanedW.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/ScrollBar.h>
#include <Xm/ScrolledW.h>
#include <Xm/Separator.h>
#include <Xm/SeparatoG.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>

#include "mission_expert.h"
#include "configuration.hpp"
#include "symbol.hpp"
#include "globals.h"
#include "popups.h"
#include "write_cdl.h"
#include "cbrclient.h"
#include "toolbar.h"
#include "EventLogging.h"
#include "file_utils.h"
#include "renumber_robots.h"
#include "load_cdl.h"
#include "bitmaps/star.bit"
#include "bitmaps/empty_star.bit"

//-----------------------------------------------------------------------
// Macros
//-----------------------------------------------------------------------
#define MEXP_CREATE_PIXMAP(name,fg,bg,parent) XCreatePixmapFromBitmapData(XtDisplay(parent),\
	RootWindowOfScreen(XtScreen(parent)),\
	(char *)name##_bits, name##_width, name##_height, fg, bg, \
	DefaultDepthOfScreen(XtScreen(parent)))

//-----------------------------------------------------------------------
// Constants
//-----------------------------------------------------------------------
// Window sizes
const int MissionExpert::DEFAULT_FEATUREMENU_WIDTH_ = 350;
const int MissionExpert::DEFAULT_FEATUREMENU_LINEHEIGHT_ = 32;
const int MissionExpert::DEFAULT_STATUSWINDOW_WIDTH_ = 350;
const int MissionExpert::DEFAULT_STATUSWINDOW_LINEHEIGHT_ = 32;
const int MissionExpert::DEFAULT_STATUSWINDOW_TOPTITLE_HEIGHT_ = 45;
const int MissionExpert::DEFAULT_STATUSWINDOW_TEXT_HEIGHT_ = 165;
const int MissionExpert::DEFAULT_STATUSWINDOW_TEXT_NUMROWS_ = 10;

// Strings
const string MissionExpert::EMPTY_STRING_= "";
const string MissionExpert::RCTABLE_DISABLE_STRING_= "DisableMissionExpert";
const string MissionExpert::RCTABLE_DATAFILESDIR_STRING_= "MExpDataFilesDir";
const string MissionExpert::DEFAULT_FEATUREMENU_TITLE_ = "Component Feature";
const string MissionExpert::DEFAULT_FEATUREMENU_TASK_LABEL_ = "Task";
const string MissionExpert::DEFAULT_FEATUREMENU_SAVE_BUTTON_LABEL_ = "Save";
const string MissionExpert::DEFAULT_FEATUREMENU_CANCEL_BUTTON_LABEL_ = "Cancel";
const string MissionExpert::DEFAULT_STATUSWINDOW_TITLE_ = "Loadable Mission Summary       ";
const string MissionExpert::DEFAULT_STATUSWINDOW_RATING_LABEL_ = "Suitability Rating  ";
const string MissionExpert::DEFAULT_STATUSWINDOW_NAVIGATION_LABEL_ = "Choose a Mission    ";
const string MissionExpert::DEFAULT_STATUSWINDOW_PREV_BUTTON_LABEL_ = "<< Previous   ";
const string MissionExpert::DEFAULT_STATUSWINDOW_NEXT_BUTTON_LABEL_ = "  Next >>";
const string MissionExpert::DEFAULT_STATUSWINDOW_LOAD_BUTTON_LABEL_ = "Load";
const string MissionExpert::DEFAULT_STATUSWINDOW_CANCEL_BUTTON_LABEL_ = "Cancel";
const string MissionExpert::ACDL_STRING_AGENT_NAME_ = "AGENT_NAME";
const string MissionExpert::ACDL_STRING_TASK_INDEX_ = "TASK_INDEX";
const string MissionExpert::ACDL_STRING_START_ = "Start";
const string MissionExpert::CDL_STRING_MAXVELOCITY_ = "max_vel";
const string MissionExpert::CDL_STRING_BASEVELOCITY_ = "base_vel";
const string MissionExpert::CDL_STRING_START_SUB_MISSION_ = "StartSubMission";
const string MissionExpert::FEATURE_NAME_NUMROBOTS_ = "NumberOfRobots";
const string MissionExpert::FEATURE_NAME_MAXVELOCITY_ = "MaxVelocity";
const string MissionExpert::FEATURE_NAME_AGGRESSIVENESS_ = "Aggressiveness";

// Others
const int MissionExpert::MAX_SUITABILITY_RATING_ = 5;
const int MissionExpert::EVENT_WAITER_SLEEP_TIME_USEC_ = 1000;
const int MissionExpert::MAX_NOWAIT_EVENTCOUNT_ = 1500;

//-----------------------------------------------------------------------
// Globals
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// Constructor for the MissionExpert class.
//-----------------------------------------------------------------------
MissionExpert::MissionExpert(
    Display *display,
    Widget parent,
    XtAppContext app,
    const symbol_table<rc_chain> &rc_table,
    Widget menuMExpCascade)
{
    rc_chain *dataFilesDirList = NULL;
    char *dataFilesDir = NULL;

    cbrclient_ = NULL;
    MExpDisplay_ = display;
    MExpParent_ = parent;
    MExpAppContext_ = app;

    XtVaGetValues
	(MExpParent_,
	 XmNforeground, &MExpParentFg_,
	 XmNbackground, &MExpParentBg_,
	 NULL);

    createPixmaps_(MExpParentFg_, MExpParentBg_, MExpParent_);
    
    // Use the MissionExpert unless specified not to.
    if (check_bool_rc(rc_table, RCTABLE_DISABLE_STRING_.c_str()) == true)
    {
	disableMExp_ = true;

	// Disabled the "Expert" button in the menu, too.
	XtSetSensitive(menuMExpCascade, false);
    }
    else
    {
	disableMExp_ = false;
    }

    dataFilesDirectory_ = EMPTY_STRING_;
    keepDataFiles_ = false;

    // Get the directory for saving output data files from the .cfgeditrc file.
    dataFilesDirList = (rc_chain *)rc_table.get(RCTABLE_DATAFILESDIR_STRING_.c_str());

    if (dataFilesDirList != NULL)
    {
	dataFilesDirList->first(&dataFilesDir);

	if (dataFilesDir != NULL)
	{
	    // Directory specified.
	    dataFilesDirectory_ = dataFilesDir;
	    keepDataFiles_ = true;
	}
    }

    numTaskTypes_ = 0;
    featuresLoaded_ = false;
    overlayFileName_ = EMPTY_STRING_;
    globalFeaturesMenuOption_w_ = NULL;
    featureMenuOption_w_ = NULL;
    featureMenuTask_w_ = NULL;
    featureMenuIsUp_ = false;

    statusWindows_.textPos = 0;
    statusWindows_.isUp = false;
    statusWindows_.currentMissionId = -1;
    statusWindows_.selectedMissionId = -1;
    statusWindows_.starBox = new Widget[MAX_SUITABILITY_RATING_];

    defaultTaskWeight_ = 1.0;

    initialize();
}

//-----------------------------------------------------------------------
// Destructor for the MissionExpert class.
//-----------------------------------------------------------------------
MissionExpert::~MissionExpert(void)
{
    if (!keepDataFiles_)
    {
	deleteCreatedDataFiles_();
    }

    if (cbrclient_ != NULL)
    {
	delete cbrclient_;
	cbrclient_ = NULL;
    }

    delete [] statusWindows_.starBox;
    statusWindows_.starBox = NULL;
}

//-----------------------------------------------------------------------
// This function initializes the class variables.
//-----------------------------------------------------------------------
void MissionExpert::initialize(void)
{
    if (disableMExp_)
    {
	return;
    }

    if (!keepDataFiles_)
    {
	deleteCreatedDataFiles_();
    }

    if (overlayFileName_ != EMPTY_STRING_)
    {
	overlayFileName_ = EMPTY_STRING_;
    }

    if (globalFeaturesMenuOption_w_ != NULL)
    {
	delete [] globalFeaturesMenuOption_w_;
	globalFeaturesMenuOption_w_ = NULL;
    }

    if (featureMenuOption_w_ != NULL)
    {
	delete [] featureMenuOption_w_;
	featureMenuOption_w_ = NULL;
    }

    if (featureMenuTask_w_ != NULL)
    {
	delete [] featureMenuTask_w_;
	featureMenuTask_w_ = NULL;
    }

    generalYesNoWindowDone_ = false;
    dataFilesNamed_ = false;
    CBRClientInstantiated_ = false;
    shouldSaveMissionToCBRLibrary_ = false;
    missionSavedToCBRLibrary_ = false;
    missionLoadedFromCBRLibrary_ = false;
    currentCBRLibraryDataIndexList_.clear();
}

//-----------------------------------------------------------------------
// This function asks users if they want to use the Mission Expert.
//-----------------------------------------------------------------------
bool MissionExpert::askUseMissionExpert(void)
{
    Widget w;
    int eventCount;

    if (disableMExp_)
    {
	return false;
    }

    gEventLogging->start("Do you want to use Mission Expert?");

    w = createGeneralYesNoWindow_("Do you want to use Mission Expert?");
    popupGeneralYesNoWindow_(w);

    
    // Wait for the answer.
    while(!generalYesNoWindowDone_)
    {
	// Run X
	if (XtAppPending(MExpAppContext_))
	{
	    XtAppProcessEvent(MExpAppContext_, XtIMAll);
	}	  

	if (eventCount > MAX_NOWAIT_EVENTCOUNT_)
	{
	    usleep(EVENT_WAITER_SLEEP_TIME_USEC_);
	}
	eventCount++;
    }

    gEventLogging->log("Do you want to use Mission Expert? = %d", generalYesNoWindowResult_);
    gEventLogging->end("Do you want to use Mission Expert?");

    return generalYesNoWindowResult_;
}

//-----------------------------------------------------------------------
// This function asks users if the mission was successful or not. The
// result will be reported to the server.
//-----------------------------------------------------------------------
void MissionExpert::confirmSuccessfulMission(void)
{
    Widget w;
    int eventCount;

    if (disableMExp_)
    {
	return;
    }

    gEventLogging->start("Was the mission successful?");

    w = createGeneralYesNoWindow_("Was the mission successful?");
    popupGeneralYesNoWindow_(w);

    
    // Wait for the answer.
    eventCount = 0;
    while(!generalYesNoWindowDone_)
    {
	// Run X
	if (XtAppPending(MExpAppContext_))
	{
	    XtAppProcessEvent(MExpAppContext_, XtIMAll);
	}	  

	if (eventCount > MAX_NOWAIT_EVENTCOUNT_)
	{
	    usleep(EVENT_WAITER_SLEEP_TIME_USEC_);
	}
	eventCount++;
    }

    if (generalYesNoWindowResult_)
    {
	// It was successful.
	cbrclient_->sendPositiveFeedbackToServer(currentCBRLibraryDataIndexList_);
    }
    else
    {
	// It was unsuccessful.
	cbrclient_->sendNegativeFeedbackToServer(currentCBRLibraryDataIndexList_);
    }

    gEventLogging->log("Was the mission successful? = %d", generalYesNoWindowResult_);
    gEventLogging->end("Was the mission successful?");
}

//-----------------------------------------------------------------------
// This function asks users if they want to use the Mission Expert.
//-----------------------------------------------------------------------
bool MissionExpert::confirmNewMission_(void)
{
    Widget w;
    int eventCount;

    gEventLogging->start("Current mission was not generated by Mission Expert. View anyway?");

    w = createGeneralYesNoWindow_("Current mission was not generated by\nMission Expert.\n\nView anyway?");
    popupGeneralYesNoWindow_(w);

    // Wait for the answer.
    eventCount = 0;
    while(!generalYesNoWindowDone_)
    {
	// Run X
	if (XtAppPending(MExpAppContext_))
	{
	    XtAppProcessEvent(MExpAppContext_, XtIMAll);
	}	  

	if (eventCount > MAX_NOWAIT_EVENTCOUNT_)
	{
	    usleep(EVENT_WAITER_SLEEP_TIME_USEC_);
	}
	eventCount++;
    }

    gEventLogging->log("View anyway = %d", generalYesNoWindowResult_);
    gEventLogging->end("Current mission was not generated by Mission Expert. View anyway?");

    return generalYesNoWindowResult_;
}

//-----------------------------------------------------------------------
// This function creates the window that simply asks a "Yes" / "No"
// question. The question statement is the input.
//-----------------------------------------------------------------------
Widget MissionExpert::createGeneralYesNoWindow_(char *question)
{
    XmString yes_str, no_str, msg_str, title_str;
    Arg wargs[16];
    int n=0;
    GeneralYesNoWindowCallbackData_t *callBackData;
    Widget generalYesNoWindow;

    msg_str = XmStringCreateLtoR(question, XmSTRING_DEFAULT_CHARSET);
    yes_str = XmStringCreateSimple("  Yes  ");
    no_str = XmStringCreateSimple("  No  ");
    title_str = XmStringCreateSimple(" ");

    XtSetArg(wargs[n], XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL); n++;
    XtSetArg(wargs[n], XmNmessageString, msg_str); n++; 
    XtSetArg(wargs[n], XmNokLabelString, yes_str); n++;
    XtSetArg(wargs[n], XmNhelpLabelString, no_str); n++;
    XtSetArg(wargs[n], XmNdialogTitle, title_str); n++;
    generalYesNoWindow = XmCreateMessageDialog(MExpParent_, "", wargs, n);

    // "Yes" button
    callBackData = new GeneralYesNoWindowCallbackData_t;
    callBackData->missionExpertInstance = this;
    callBackData->result = true;
    XtAddCallback
	(generalYesNoWindow,
	 XmNokCallback,
	 (XtCallbackProc)cbGeneralYesNoWindow_,
	 (void *)callBackData);

    // "No" button
    callBackData = new GeneralYesNoWindowCallbackData_t;
    callBackData->missionExpertInstance = this;
    callBackData->result = false;
    XtAddCallback
	(generalYesNoWindow,
	 XmNhelpCallback,
	 (XtCallbackProc)cbGeneralYesNoWindow_,
	 (void *)callBackData);

    XtUnmanageChild(XmMessageBoxGetChild(generalYesNoWindow, XmDIALOG_CANCEL_BUTTON));

    XmStringFree(msg_str);
    XmStringFree(yes_str);
    XmStringFree(no_str);

    return generalYesNoWindow;
}

//-----------------------------------------------------------------------
// This function popups the window above.
//-----------------------------------------------------------------------
void MissionExpert::popupGeneralYesNoWindow_(Widget w)
{
    generalYesNoWindowResult_ = false;
    generalYesNoWindowDone_ = false;
    XtManageChild(w);
}

//-----------------------------------------------------------------------
// This function sets the "Yes" / "No" variable according to the answer
// given by the user.
//-----------------------------------------------------------------------
void MissionExpert::sendGeneralYesNoWindowResult_(bool result)
{
    generalYesNoWindowResult_ = result;
    generalYesNoWindowDone_ = true;
}

//-----------------------------------------------------------------------
// This function creates a unique prefix for output files, based on the
// user name, date, and time.
//-----------------------------------------------------------------------
string MissionExpert::createPrefix_(void)
{
    time_t cur_time;
    tm local_time;
    string prefix;
    char date[100], month[100], day[100], year[100], hour[100], min[100], sec[100];
    char *userName = NULL, buf[1024];

    // Get the user name.
    userName = getenv("USER");

    // Get the current date and time.
    cur_time = time(NULL);
    strncpy(date,ctime(&cur_time),sizeof(date));
    local_time = *localtime(&cur_time);
    strftime(month, sizeof(month), "%m", &local_time);
    strftime(day, sizeof(day), "%d", &local_time);
    strftime(year, sizeof(year), "%Y", &local_time);
    strftime(hour, sizeof(hour), "%H", &local_time);
    strftime(min, sizeof(min), "%M", &local_time);
    strftime(sec, sizeof(sec), "%S", &local_time);

    // Create the prefix.
    sprintf(buf, "mexp-%s-%s%s%s-%s%s%s",
	    userName,
	    month,
	    day,
	    year,
	    hour,
	    min,
	    sec);
    prefix = buf;

    return prefix;
}

//-----------------------------------------------------------------------
// This function creates names for various files that will be used in
// Mission Expert.
//-----------------------------------------------------------------------
void MissionExpert::nameDataFiles_(void)
{
    string prefix = EMPTY_STRING_;
    string defaultPrefix;

    // Create a default prefix.
    defaultPrefix = createPrefix_();

    // Create the prefix of the file (including the directory path if appropriate)
    if (dataFilesDirectory_ != EMPTY_STRING_)
    {
	prefix = dataFilesDirectory_;
	prefix += "/";
    }
    prefix += defaultPrefix;

    // Create the feature file string to be sent to mlab.
    featureFileString_ = prefix;
    featureFileString_ += ".";
    featureFileString_ += MEXP_STRING_FEATURE;

    // Name the feature file.
    featureFileName_ = featureFileString_;
    featureFileName_ += ".";
    featureFileName_ += MEXP_EXTENTION_FEATUREFILENAME;

    // Name the feature status file.
    featureStatusFileName_ = featureFileString_;
    featureStatusFileName_ += ".";
    featureStatusFileName_ += MEXP_EXTENTION_FEATURESTATUSFILENAME;

    // Name the result file for retrieving.
    resultFileName_ = prefix;
    resultFileName_ += ".";
    resultFileName_ += MEXP_STRING_RETRIEVE;
    resultFileName_ += ".";
    resultFileName_ += MEXP_EXTENTION_CDL;

    // Name the case file for adding.
    missionPlanFileName_ = prefix;
    missionPlanFileName_ += ".";
    missionPlanFileName_ += MEXP_STRING_ADD;
    missionPlanFileName_ += ".";
    missionPlanFileName_ += MEXP_EXTENTION_ACDL;

    // Name the robot.
    robotName_ = defaultPrefix;
    robotName_ += MEXP_STRING_ROBOT;

    // Set the flag.
    dataFilesNamed_ = true;
}

//-----------------------------------------------------------------------
// This function initializs the CBRClient class. 
//-----------------------------------------------------------------------
bool MissionExpert::initCBRClient_(void)
{
    bool initialized;

    cbrclient_ = new CBRClient();
    initialized = cbrclient_->initialize();

    if (!initialized)
    {
	warn_userf("CBRServer unreachable. Please make sure it is running.\n");
	return false;
    }

    CBRClientInstantiated_ = true;

    return true;
}

//-----------------------------------------------------------------------
// This function forks mlab in order for the user to select the mission.
//-----------------------------------------------------------------------
void MissionExpert::runMlabMissionDesign_(void)
{
    int pid, p;
    char *prog, *argv[32], buf[256];
    bool initialized = false, done = false, forkFailed = false;
    bool eventLogIsON;
    int status, options, rtn;
    const char *eventLogfileName = gEventLogging->getLogfileName();
    config_fsa_lists_t fsaLists;

    if (disableMExp_)
    {
	return;
    }

    eventLogIsON = gEventLogging->eventLoggingIsON();

    if (!CBRClientInstantiated_)
    {
	// Instantiate the CBRClient first.
	initialized = initCBRClient_();

	if (!initialized)
	{
	    // Something went wrong during the initialization.
	    // Just return.
	    return;
	}

	// Upload the FSA names to the server
	fsaLists = getFSALists_();
	cbrclient_->saveFSANames(fsaLists);
    }

    // Set executable name
    p = 0;
    prog = strdup("mlab");
    argv[p++] = prog;
    argv[p++] = "-M";
    argv[p++] = strdup(featureFileString_.c_str());

    // Usability log
    if(eventLogIsON)
    {
	argv[p++] = "-e";
	sprintf(buf,"%s", eventLogfileName);
	argv[p++] = strdup(buf);
	gEventLogging->start
	    ("MExp fork mlab -M %s -e %s",
	     featureFileString_.c_str(),
	     eventLogfileName);
	gEventLogging->pause();
    }

    // Add NULL at the end of the command
    argv[p] = NULL;

    if ((pid = fork()) == 0)
    {
	// In child.
	if (execvp(prog, argv)) forkFailed = true;
      
	// Kill this child
	exit(2);
    }

    if ((pid == -1) || (forkFailed == true))
    {
	gEventLogging->resume(false);
	gEventLogging->end("MExp fork mlab");
	fprintf(stderr, "Error(cfgedit): MissionExpert::runMlabMissionDesign_().\nUnable to fork %s\n", prog);
	config->quit();
    }
    else
    {
	// wait for mlab to finish
	while (!done)
	{
	    // Run X
	    if (XtAppPending(MExpAppContext_))
	    {
		XtAppProcessEvent(MExpAppContext_, XtIMAll);
	    }	  

	    options = WNOHANG | WUNTRACED;
	    rtn = waitpid(pid, &status, options);
      
	    if ((rtn == pid) || (rtn == -1))
	    {
		// mlab finished or died with signal
		done = true;
	    }

	    usleep(EVENT_WAITER_SLEEP_TIME_USEC_);
	}
    }

    if(eventLogIsON)
    {
	gEventLogging->resume(false);
	gEventLogging->end("MExp fork mlab");
    }
}

//-----------------------------------------------------------------------
// This function checks the feature status file being sent by mlab to see
// if reconfiguration was specified or not.
//-----------------------------------------------------------------------
bool MissionExpert::checkFeatureStatusFile_(void)
{
    FILE *featureFile = NULL;
    char buf1[1024], buf2[1024];
    string bufString1, bufString2;
    bool status = false;

    // Make sure the feature file is readable.
    featureFile = fopen(featureFileName_.c_str(), "r");
    if (featureFile == NULL)
    {
	return false;
    }
    else
    {
	fclose(featureFile);
	addCreatedDataFileList_(featureFileName_);
    }

    // Check the status.
    featureStatusFile_ = fopen(featureStatusFileName_.c_str(), "r");
    if (featureStatusFile_ == NULL)
    {
	return false;
    }
    else 
    {
	addCreatedDataFileList_(featureStatusFileName_);
    }

    while(!feof(featureStatusFile_))
    {
	if (fscanf(featureStatusFile_, "%s", buf1) == EOF)
	{
	    break;
	}

	bufString1 = buf1;

	if (bufString1 == MEXP_STRING_CONFIGURE)
	{
	    fscanf(featureStatusFile_, "%s", buf2);

	    bufString2 = buf2;

	    status = (buf2 == MEXP_STRING_YES)? true : false;

	    break;
	}
    }

    fclose(featureStatusFile_);
    return status;
}

//-----------------------------------------------------------------------
// This function retrieve the case from the CBRServer based on the featurees
// being specified by the user.
//-----------------------------------------------------------------------
bool MissionExpert::buildMission_(void)
{
    vector<MExpRetrievedMission_t> retrievedMissions;
    string overlayFileName = EMPTY_STRING_;
    bool missionLoaded = false, nextBtnIsSensitive = false;
    int index;

    // Check the feature status file being sent by mlab to see if the
    // mission has to be rebuilt with CBR.
    if (!checkFeatureStatusFile_())
    {
	return false;
    }

    // Load the features from the Mission Expert RC file.
    if (!featuresLoaded_)
    {
	featuresLoaded_ = loadFeaturesFromRCFile_();

	if (!featuresLoaded_)
	{
	    warn_userf("Mission Expert RC file \"%s\" could not be loaded.\n",
		       MEXP_RC_FILENAME.c_str());
	    return false;
	}
    }
    
    popupStatusWindow_(
	0,
	"Searching for a mission. Please wait...",
	false,
	false,
	false,
	false,
	-1);

    gEventLogging->start("MExp Building a mission");

    // Retrieve the case from CBRServer via CBRClient.
    retrievedMissions = cbrclient_->retrieveMissionPlan(
	featureFileName_,
	defaultTaskList_,
	defaultGlobalFeatures_,
	defaultLocalFeatures_,
	defaultTaskWeight_,
	MAX_SUITABILITY_RATING_,
	&overlayFileName);

    if ((int)(retrievedMissions.size()) > 0)
    {
	retrievedMissionSummary_ = summarizeRetrievedMission_(retrievedMissions);

	// Load the summarize the first mission.
	nextBtnIsSensitive = ((int)(retrievedMissions.size()) > 1)? true : false;
	index = 0;
	updateStatusWindow_(
	    retrievedMissionSummary_.ratings[index],
	    retrievedMissionSummary_.fsaSummaries[index],
	    false,
	    nextBtnIsSensitive,
	    true,
	    true,
	    index);

	// Loads the mission that was selected by the user from the summary
	// (i.e., wait for the user input to the status window).
	missionLoaded = loadRetrievedMissionChosenByUser_(
	    retrievedMissions,
	    overlayFileName);
    }
    else
    {
	// No mission was returned...
	updateStatusWindow_(
	    0,
	    "Sorry, no suitable mission was found.",
	    false,
	    false,
	    false,
	    true,
	    -1);

	missionLoaded = false;
    }

    if (missionLoaded)
    {
	missionLoadedFromCBRLibrary_ = true;
	shouldSaveMissionToCBRLibrary_ = false;
    }

    gEventLogging->end("MExp Building a mission");

    return missionLoaded;
}

//-----------------------------------------------------------------------
// This creates a summary of the retrieved mission to be posted on the
// summary window.
//-----------------------------------------------------------------------
MExpRetrievedMissionSummary_t MissionExpert::summarizeRetrievedMission_(
    const vector<MExpRetrievedMission_t> retrievedMissions)
{
    FILE *tmpACDLFile = NULL;
    MExpRetrievedMissionSummary_t summary;
    const MExpRetrievedMissionSummary_t EMPTY_SUMMARY;
    char tmpACDLFilename[256];
    char buf[1024], buf1[1024], buf2[1024];
    string bufString1, bufString2;
    string fsaSummary;
    string lastTaskIndex, fsaName;
    vector<string> fsaNames;
    int i, j, rating, numMissions = 0, count;
    int fd;
    vector<int> counts;
    bool nameFound = false;

    numMissions = retrievedMissions.size();

    if (numMissions > 0)
    {
	// Create a temporary file (to dump the FSA summay later).
	sprintf(tmpACDLFilename, "/tmp/%s-cfgedit-summarizeRetrievedMission_-acdl-XXXXXX", getenv("USER"));
	fd = mkstemp(tmpACDLFilename);
	unlink(tmpACDLFilename);
    }
    else
    {
	return EMPTY_SUMMARY;
    }

    for (i = 0; i < numMissions; i++)
    {
	// Copy rating
	rating = retrievedMissions[i].rating;
	summary.ratings.push_back(rating);

	// Get the FSA summary. Dump the ACDL code in a temporary file first.
	tmpACDLFile = fopen(tmpACDLFilename, "w");
	fprintf(tmpACDLFile, "%s", retrievedMissions[i].acdlSolution.c_str());
	fclose(tmpACDLFile);
	
	// Now, open up the file, again, for reading.
	tmpACDLFile = fopen(tmpACDLFilename, "r");
	
	fsaNames.clear();
	counts.clear();
	lastTaskIndex = EMPTY_STRING_;
    
	// Process the data.
	while (fscanf(tmpACDLFile, "%s %s", buf1, buf2) != EOF)
	{
	    bufString1 = buf1;
	    bufString2 = buf2;

	    if (bufString1 == ACDL_STRING_TASK_INDEX_)
	    {
		lastTaskIndex = bufString2;
	    }

	    if (bufString1 == ACDL_STRING_AGENT_NAME_)
	    {
		// Check to see if this fsa name is for the Start state.
		// The start state is named "Stop", a little confusing for the
		// user to see in the summary.
		fsaName = (lastTaskIndex == ACDL_STRING_START_)? ACDL_STRING_START_ : bufString2;

		nameFound = false;

		for (j = 0; j < ((int)(fsaNames.size())); j++)
		{
		    if (fsaNames[j] == fsaName)
		    {
			(counts[j])++;
			nameFound = true;
			break;
		    }
		}

		if (!nameFound)
		{
		    // New FSA name.
		    fsaNames.push_back(fsaName);
		    count = 1;
		    counts.push_back(count);
		}
	    }
	    
	}

	// Make sure the file is closed.
	fclose(tmpACDLFile);
	unlink(tmpACDLFilename);

	if (((int)(fsaNames.size())) == 0)
	{
	    fsaSummary = "This is an empty mission.";
	}
	else
	{
	    fsaSummary = "States/Triggers Used in this mission:\n\n";

	    for (j = 0; j < ((int)(fsaNames.size())); j++)
	    {
		sprintf(buf, "%s (x%d)\n", fsaNames[j].c_str(), counts[j]);
		fsaSummary += buf;
	    }
	}

	summary.fsaSummaries.push_back(fsaSummary);
    }

    return summary;
}

//-----------------------------------------------------------------------
// This function runs the Mission Expert.
//-----------------------------------------------------------------------
void MissionExpert::runMissionExpert(int mode)
{
    bool missionIsBuilt = false;

    if (disableMExp_)
    {
	return;
    }

    switch (mode) {

    case MEXP_RUNMODE_NEW:

	gEventLogging->log("MExp Run Mode = New");

	if (overlayFileName_ != EMPTY_STRING_)
	{
	    overlayFileName_ = EMPTY_STRING_;
	}
	nameDataFiles_();

	break;

    case MEXP_RUNMODE_VIEW:

	gEventLogging->log("MExp Run Mode = View");

	if (shouldSaveMissionToCBRLibrary_)
	{
	    saveSelectionToCBRLibrary(config->root());
	}

	if (!dataFilesNamed_)
	{
	    if (confirmNewMission_())
	    {
		nameDataFiles_();
	    }
	    else
	    {
		return;
	    }
	}

	break;

    default:
	return;
    }

    runMlabMissionDesign_();

    missionIsBuilt = buildMission_();

    if (missionIsBuilt)
    {
	config->load(resultFileName_.c_str());
	config->made_change();

	save_cdl_for_replay("mexp_retrieved_mission");
    }
}

//-----------------------------------------------------------------------
// This function saves the selected cases to the CBR library.
//-----------------------------------------------------------------------
void MissionExpert::saveSelectionToCBRLibrary(Symbol * agent)
{
    bool initialized = false, saved = false, hasStartSubMission = false;
    bool errors;
    config_fsa_lists_t fsaLists;
    string tempFilename;
    const char *tempFilelist[2];

    if (disableMExp_)
    {
	return;
    }

    if (!dataFilesNamed_)
    {
	nameDataFiles_();
    }

    // Make sure it can talk to the CBR library.
    if (!CBRClientInstantiated_)
    {
	// Instantiate the CBRClient first.
	initialized = initCBRClient_();

	if (!initialized)
	{
	    // Something went wrong during the initialization.
	    // Just return.
	    return;
	}

	// Upload the FSA names to the server
	fsaLists = getFSALists_();
	cbrclient_->saveFSANames(fsaLists);
    }

    // Save the agent in a temporary file first, and then load it
    // again. This will allow the configuration tree to be ordered
    // correctly.
    tempFilename = "/tmp/";
    tempFilename += createPrefix_();
    tempFilename += ".";
    tempFilename += MEXP_EXTENTION_CDL;
    saved = save_workspace(agent, tempFilename.c_str(), false, false);

    if (saved)
    {
	tempFilelist[0] = tempFilename.c_str();
	tempFilelist[1] = NULL;
	agent = load_cdl(tempFilelist, errors, false, false);
	saved = save_abstracted_workspace(agent, missionPlanFileName_.c_str());
	unlink(tempFilename.c_str());
    }

    if (!saved)
    {
	warn_userf("Selection cannot be saved.\n");
	return;
    }
    else
    {
	hasStartSubMission = hasAgentInMissionPlan_(CDL_STRING_START_SUB_MISSION_);
	addCreatedDataFileList_(missionPlanFileName_);
    }

    // Load the features from the Mission Expert RC file.
    if (!featuresLoaded_)
    {
	featuresLoaded_ = loadFeaturesFromRCFile_();

	if (!featuresLoaded_)
	{
	    warn_userf("Mission Expert RC file \"%s\" could not be loaded.\n",
		       MEXP_RC_FILENAME.c_str());
	    return;
	}
    }
    
    // Assign temporary values for the task and global features used in
    // the feature menu window.
    createTemporaryTaskAndGlobalFeatures_();

    // Create and popup the feature menu window.
    popupFeatureMenu_(!hasStartSubMission);
 }
//-----------------------------------------------------------------------
// This function creates a temporary task based on the default task.
//-----------------------------------------------------------------------
void MissionExpert::createTemporaryTaskAndGlobalFeatures_(void)
{
    int i, j;
    double maxVel, baseVel, aggressiveness;
    string taskName, maxVelValue, baseVelValue;
    MExpFeature_t localFeature, fixedGlobalFeature;
    vector<MExpFeature_t> localFeatures;
    vector<string> valueList;
    MExpMissionTask_t defaultTask;
    MExpFeatureOption_t option;

    // At first, create a list of local features with default values.
    localFeatures = defaultLocalFeatures_;

    // Next, find a task in the list that matches with the default task.
    defaultTask = defaultTaskList_[0];
    for (i = 0; i < numTaskTypes_; i++)
    {
	if (defaultTaskList_[i].name == defaultTaskName_)
	{
	    defaultTask = defaultTaskList_[i];
	    break;
	}
    }

    // Use the specified option for the default task if it is specified.
    for (i = 0; i < (int)(defaultTask.localFeatures.size()); i++)
    {
	for (j = 0; j < (int)(localFeatures.size()); j++)
	{
	    if (defaultTask.localFeatures[i].name == localFeatures[j].name)
	    {
		localFeatures[j].selectedOption = defaultTask.localFeatures[i].selectedOption;
	    }
	}
    }

    // Assign values for the task.
    featureMenuUpTask_.localFeatures = localFeatures;
    featureMenuUpTask_.number = 0;
    featureMenuUpTask_.id = defaultTask.id;
    featureMenuUpTask_.name = defaultTask.name;
    featureMenuUpTask_.position.x = 0;
    featureMenuUpTask_.position.y = 0;

    // Copy the values of the default global features to the menu specific one.
    featureMenuUpGlobalFeatures_.clear();
    fixedGlobalFeatures_.clear();

    for (i = 0; i < (int)(defaultGlobalFeatures_.size()); i++)
    {
	if (defaultGlobalFeatures_[i].name == FEATURE_NAME_NUMROBOTS_)
	{
	    fixedGlobalFeature = defaultGlobalFeatures_[i];

	    // Assign the number of robots (from renumber_robots.h).
	    fixedGlobalFeature.selectedOption = (float)num_robots; 
	    fixedGlobalFeatures_.push_back(fixedGlobalFeature);
	}
	else if (defaultGlobalFeatures_[i].name == FEATURE_NAME_MAXVELOCITY_)
	{
	    fixedGlobalFeature = defaultGlobalFeatures_[i];

	    // Get the max velocity from configuration.
	    maxVelValue = config->root()->findData(CDL_STRING_MAXVELOCITY_);
	    maxVel = atof(maxVelValue.c_str());

	    fixedGlobalFeature.selectedOption = maxVel; 
	    fixedGlobalFeatures_.push_back(fixedGlobalFeature);
	}
	else if (defaultGlobalFeatures_[i].name == FEATURE_NAME_AGGRESSIVENESS_)
	{
	    fixedGlobalFeature = defaultGlobalFeatures_[i];

	    // Get the max velocity from configuration.
	    baseVelValue = (config->root()->findDataList(CDL_STRING_BASEVELOCITY_)).back();
	    baseVel = atof(baseVelValue.c_str());

	    aggressiveness = baseVel/maxVel;
	    aggressiveness = (aggressiveness > 1.0)? 1.0 : aggressiveness;

	    fixedGlobalFeature.selectedOption = aggressiveness;
	    fixedGlobalFeatures_.push_back(fixedGlobalFeature);
	}
	else
	{
	    featureMenuUpGlobalFeatures_.push_back(defaultGlobalFeatures_[i]);
	}
    }
}

//-----------------------------------------------------------------------
// This function creates a popup window for the feature menu to save the case. 
//-----------------------------------------------------------------------
void MissionExpert::popupFeatureMenu_(bool askTaskType)
{
    XmString sliderTitle;
    Widget main_frame, main;
    Widget label_frame, label_box, label;
    Widget feature_frame, feature_box;
    Widget task_frame, task_box;
    Widget action_frame, action;
    Widget button_frame;
    Widget apply_frame, apply;
    Widget cancel_frame, cancel;
    MExpMissionTask_t task;
    vector<Widget> stackedWidget;
    FeatureMenuTaskCallbackData_t *taskData;
    FeatureMenuCallbackData_t *featureOptionData;
    GlobalFeatureMenuCallbackData_t *globalFeatureData;
    string featureMenuTitle;
    string taskLabel;
    string saveLabel;
    string cancelLabel;
    float value, conversion;
    int featureMenuHeight, featureMenuWidth;
    int featureMenuLineHeight, taskBoxHeight, taskBoxHeightOffset;
    int i, j, options, line = 0;
    int set;
    int screenNumber;
    int posX, posY, displayWidth, displayHeight;
    int numStackedWidget, numLocalFeatures, numGlobalFeatures;
    int eventCount;
    int maxValue, decimalPoints;

    featureMenuIsUp_ = true;

    featureMenuTitle = DEFAULT_FEATUREMENU_TITLE_;
    taskLabel = DEFAULT_FEATUREMENU_TASK_LABEL_;
    saveLabel = DEFAULT_FEATUREMENU_SAVE_BUTTON_LABEL_;
    cancelLabel = DEFAULT_FEATUREMENU_CANCEL_BUTTON_LABEL_;

    // Setting the size of the window.
    numLocalFeatures = featureMenuUpTask_.localFeatures.size();
    numGlobalFeatures = featureMenuUpGlobalFeatures_.size();
    featureMenuWidth = DEFAULT_FEATUREMENU_WIDTH_;
    featureMenuLineHeight = DEFAULT_FEATUREMENU_LINEHEIGHT_;

    if (askTaskType && (numTaskTypes_ > 0))
    {
	taskBoxHeight = (1+numTaskTypes_)*(featureMenuLineHeight+2);
	taskBoxHeightOffset = 6;
    }
    else
    {
	taskBoxHeight = 0;
	taskBoxHeightOffset = 0;
    }

    featureMenuHeight = (taskBoxHeight+taskBoxHeightOffset) + // Task
	((featureMenuLineHeight+3)*
	 (1 + // Top label
	  2 + // Apply & Cancel Buttons
	  2*(numLocalFeatures+numGlobalFeatures))); // Others
	    
    // Make sure the window does go outside the desktop.
    screenNumber = DefaultScreen(MExpDisplay_);
    displayWidth = DisplayWidth(MExpDisplay_, screenNumber);
    displayHeight = DisplayHeight(MExpDisplay_, screenNumber);
    posX = (displayWidth - featureMenuWidth)/2;
    posY = (displayHeight - featureMenuHeight)/2;
    
    // Popup shell.
    featureMenu_w_ = XtVaCreatePopupShell
	("",
	 xmDialogShellWidgetClass, MExpParent_, 
	 XmNdeleteResponse, XmUNMAP,
	 XmNallowShellResize, true,
	 XmNx, posX,
	 XmNy, posY,
	 XmNheight, featureMenuHeight,
	 XmNwidth, featureMenuWidth,
	 NULL);

    // The biggest frame.
    main_frame = XtVaCreateWidget
	("main_frame",
	 xmFormWidgetClass, featureMenu_w_,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, featureMenuHeight,
	 XmNwidth, featureMenuWidth+6,
	 NULL);

    stackedWidget.push_back(main_frame);

    main = XtVaCreateWidget
	("main",
	 xmRowColumnWidgetClass, main_frame,
	 XmNpacking, XmPACK_TIGHT,
	 XmNorientation, XmVERTICAL,
	 XmNnumColumns, 1,
	 XmNisAligned, True,
	 XmNentryAlignment, XmALIGNMENT_CENTER,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 XmNheight, featureMenuHeight,
	 XmNwidth, featureMenuHeight,
	 NULL);

    stackedWidget.push_back(main);

    // The frame for the label.
    label_frame = XtVaCreateWidget
	("label_frame",
	 xmFrameWidgetClass, main,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, featureMenuLineHeight-2,
	 XmNwidth, featureMenuWidth,
	 NULL);

    stackedWidget.push_back(label_frame);

    // The widget for the main label.
    label_box = XtVaCreateWidget
	("label_box",
	 xmRowColumnWidgetClass, label_frame,
	 XmNpacking, XmPACK_COLUMN,
	 XmNorientation, XmVERTICAL,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNnumColumns, 1,
	 XmNheight, featureMenuLineHeight,
	 XmNwidth, featureMenuWidth,
	 NULL);

    stackedWidget.push_back(label_box);

    // The main label.
    label = XtVaCreateWidget
	(featureMenuTitle.c_str(),
	 xmLabelGadgetClass, label_box,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, featureMenuLineHeight,
	 XmNwidth, featureMenuWidth,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, label_box,
	 XmNleftAttachment, XmATTACH_WIDGET,
	 XmNleftWidget, label_box,
	 XmNrightAttachment, XmATTACH_WIDGET,
	 XmNrightWidget, label_box,
	 NULL);

    stackedWidget.push_back(label);

    line++;

    // It is a 2D array. Create the 1D part (feature) first.
    globalFeaturesMenuOption_w_ = new Widget*[numGlobalFeatures];

    //  Create the global feature raws sequentially.
    for (i = 0; i < numGlobalFeatures; i++)
    {
	options = featureMenuUpGlobalFeatures_[i].options.size();

	// The frame for the feature.
	feature_frame = XtVaCreateWidget
	    ("feature_frame",
	     xmFrameWidgetClass, main,
	     XmNy, line*featureMenuLineHeight,
	     XmNheight, 2*featureMenuLineHeight,
	     XmNwidth, featureMenuWidth,
	     NULL);

	stackedWidget.push_back(feature_frame);

	// Two-colum widget for the feature.
	feature_box = XtVaCreateWidget
	    ("feature_box",
	     xmRowColumnWidgetClass, feature_frame,
	     XmNpacking, XmPACK_COLUMN,
	     XmNorientation, XmHORIZONTAL,
	     XmNalignment, XmALIGNMENT_BEGINNING,
	     XmNnumColumns, options,
	     XmNheight, 2*featureMenuLineHeight,
	     XmNwidth, featureMenuWidth,
	     XmNtopAttachment, XmATTACH_FORM,
	     XmNrightAttachment, XmATTACH_FORM,
	     XmNleftAttachment, XmATTACH_FORM,
	     XmNbottomAttachment, XmATTACH_WIDGET,
	     NULL);

	stackedWidget.push_back(feature_box);

	// Create the second dimention (option) of this array.
	globalFeaturesMenuOption_w_[i] = new Widget[options];

	switch (featureMenuUpGlobalFeatures_[i].optionType) {

	case MEXP_FEATURE_OPTION_TOGGLE:

	    // The label column.
	    label = XtVaCreateWidget
		(featureMenuUpGlobalFeatures_[i].name.c_str(),
		 xmLabelGadgetClass, feature_box,
		 XmNalignment, XmALIGNMENT_BEGINNING,
		 XmNheight, featureMenuLineHeight,
		 XmNwidth, featureMenuWidth/options,
		 XmNtopAttachment, XmATTACH_WIDGET,
		 XmNtopWidget, feature_box,
		 XmNleftAttachment, XmATTACH_WIDGET,
		 XmNleftWidget, feature_box,
		 NULL);

	    stackedWidget.push_back(label);
	
	    // Fill the blanks in the label row.
	    for (j = 0; j < (options-1); j++)
	    {
		label = XtVaCreateWidget
		    (" ",
		     xmLabelGadgetClass, feature_box,
		     XmNalignment, XmALIGNMENT_BEGINNING,
		     XmNheight, featureMenuLineHeight,
		     XmNwidth, featureMenuWidth/options,
		     XmNtopAttachment, XmATTACH_WIDGET,
		     XmNtopWidget, feature_box,
		     NULL);

		stackedWidget.push_back(label);
	    }
	
	    for (j = 0; j < options; j++)
	    {
		globalFeatureData = new GlobalFeatureMenuCallbackData_t;
		globalFeatureData->globalFeaturesId = i;
		globalFeatureData->optionValue = (float)j;
		globalFeatureData->missionExpertInstance = this;
		globalFeatureData->optionType = featureMenuUpGlobalFeatures_[i].optionType;

		value = ((int)(featureMenuUpGlobalFeatures_[i].selectedOption) == j)? 1.0 : 0.0;

		button_frame = XtVaCreateManagedWidget
		    ("button_frame",
		     xmFormWidgetClass, feature_box,
		     XmNshadowThickness, 0,
		     XmNalignment, XmALIGNMENT_BEGINNING,
		     XmNheight, featureMenuLineHeight,
		     XmNwidth, featureMenuWidth/options-10,
		     NULL);

		stackedWidget.push_back(button_frame);

		// Toggle features.
		globalFeaturesMenuOption_w_[i][j] = XtVaCreateWidget
		    (featureMenuUpGlobalFeatures_[i].options[j].value.c_str(),
		     xmToggleButtonGadgetClass, button_frame,
		     XmNradioBehavior, true,
		     XmNradioAlwaysOne, true,
		     XmNheight, featureMenuLineHeight,
		     XmNwidth, featureMenuWidth/options-10,
		     XmNset, (int)value,
		     XmNalignment, XmALIGNMENT_BEGINNING,
		     XmNshadowThickness, 0,
		     XmNhighlightColor, MExpParentBg_,
		     NULL);
		
		stackedWidget.push_back(globalFeaturesMenuOption_w_[i][j]);

		XtAddCallback
		    (globalFeaturesMenuOption_w_[i][j],
		     XmNvalueChangedCallback,
		     (XtCallbackProc)cbChangeGlobalFeatureMenuValue_,
		     (void *)globalFeatureData);
	    }

	    break;

	case MEXP_FEATURE_OPTION_SLIDER10:
	case MEXP_FEATURE_OPTION_SLIDER1:

	    switch (featureMenuUpGlobalFeatures_[i].optionType) {

	    case MEXP_FEATURE_OPTION_SLIDER10:
		sliderTitle = XmStringCreateLtoR(
		    (String)MEXP_DEFAULT_SLIDER10_RANAGE_TITLE.c_str(),
		    XmSTRING_DEFAULT_CHARSET);
		maxValue = 10;
		decimalPoints = 0;
		conversion = 1.0;
		break;

	    case MEXP_FEATURE_OPTION_SLIDER1:
	    default:
		sliderTitle = XmStringCreateLtoR(
		    (String)MEXP_DEFAULT_SLIDER1_RANAGE_TITLE.c_str(),
		    XmSTRING_DEFAULT_CHARSET);
		maxValue = 100;
		decimalPoints = 2;
		conversion = 100.0;
		break;
	    }

	    // The label column.
	    label = XtVaCreateWidget
		(featureMenuUpGlobalFeatures_[i].name.c_str(),
		 xmLabelGadgetClass, feature_box,
		 XmNalignment, XmALIGNMENT_BEGINNING,
		 XmNheight, 2*featureMenuLineHeight,
		 XmNwidth, featureMenuWidth/2,
		 XmNtopAttachment, XmATTACH_WIDGET,
		 XmNtopWidget, feature_box,
		 XmNleftAttachment, XmATTACH_WIDGET,
		 XmNleftWidget, feature_box,
		 NULL);

	    stackedWidget.push_back(label);

	    globalFeatureData = new GlobalFeatureMenuCallbackData_t;
	    globalFeatureData->globalFeaturesId = i;
	    globalFeatureData->optionValue = featureMenuUpGlobalFeatures_[i].selectedOption;
	    globalFeatureData->missionExpertInstance = this;
	    globalFeatureData->optionType = featureMenuUpGlobalFeatures_[i].optionType;

	    value  = globalFeatureData->optionValue*conversion;

	    // Toggle features.
	    globalFeaturesMenuOption_w_[i][0] = XtVaCreateWidget
		("",
		 xmScaleWidgetClass, feature_box,
		 XmNmaximum, maxValue,
		 XmNminimum, 1,
		 XmNvalue, (int)value,
		 XmNorientation, XmHORIZONTAL,
		 XmNprocessingDirection, XmMAX_ON_RIGHT,
		 XmNshowValue, true,
		 XmNdecimalPoints, decimalPoints,
		 XmNheight, 2*featureMenuLineHeight,
		 XmNwidth, 3*featureMenuWidth/7,
		 XmNtopAttachment, XmATTACH_WIDGET,
		 XmNtopWidget, label,
		 XmNrightAttachment, XmATTACH_FORM,
		 XmNtitleString, sliderTitle,
		 XmNhighlightColor, MExpParentBg_,
		 NULL);

	    stackedWidget.push_back(globalFeaturesMenuOption_w_[i][0]);

	    XtAddCallback
		(globalFeaturesMenuOption_w_[i][0],
		 XmNvalueChangedCallback,
		 (XtCallbackProc)cbChangeGlobalFeatureMenuValue_,
		 (void *)globalFeatureData);

	    XmStringFree(sliderTitle);

	    break;
	}
	line += 2;
    }

    // The frame for the task.
    if (askTaskType && (numTaskTypes_ > 0))
    {
	task_frame = XtVaCreateWidget
	    ("task_frame",
	     xmFrameWidgetClass, main,
	     XmNy, line*featureMenuLineHeight+1,
	     XmNheight, taskBoxHeight,
	     XmNwidth, featureMenuWidth,
	     NULL);

	stackedWidget.push_back(task_frame);

	task_box = XtVaCreateWidget
	    ("task_box",
	     xmRowColumnWidgetClass, task_frame,
	     XmNpacking, XmPACK_COLUMN,
	     XmNorientation, XmVERTICAL,
	     XmNalignment, XmALIGNMENT_BEGINNING,
	     XmNnumColumns, 1,
	     XmNheight, taskBoxHeight,
	     XmNwidth, featureMenuWidth,
	     NULL);

	stackedWidget.push_back(task_box);

	// The label column.
	label = XtVaCreateWidget
	    (taskLabel.c_str(),
	     xmLabelGadgetClass, task_box,
	     XmNalignment, XmALIGNMENT_BEGINNING,
	     XmNheight, featureMenuLineHeight,
	     XmNwidth, featureMenuWidth,
	     XmNtopAttachment, XmATTACH_WIDGET,
	     XmNtopWidget, task_box,
	     XmNleftAttachment, XmATTACH_WIDGET,
	     XmNleftWidget, task_box,
	     NULL);

	stackedWidget.push_back(label);

	line++;

	featureMenuTask_w_ = new Widget[numTaskTypes_];

	for (i = 0; i < numTaskTypes_; i++)
	{
	    task = defaultTaskList_[i];

	    set = (task.name == featureMenuUpTask_.name)? true : false;

	    featureMenuTask_w_[i] = XtVaCreateWidget
		(task.name.c_str(),
		 xmToggleButtonGadgetClass, task_box,
		 XmNradioBehavior, true,
		 XmNradioAlwaysOne, true,
		 XmNheight, featureMenuLineHeight,
		 XmNwidth, featureMenuWidth,
		 XmNset, set,
		 XmNtopAttachment, XmATTACH_WIDGET,
		 XmNtopWidget, label,
		 XmNleftAttachment, XmATTACH_WIDGET,
		 XmNleftWidget, task_box,
		 NULL);

	    stackedWidget.push_back(featureMenuTask_w_[i]);

	    line++;

	    taskData = new FeatureMenuTaskCallbackData_t;
	    taskData->taskId = task.id;
	    taskData->missionExpertInstance = this;
	    XtAddCallback
		(featureMenuTask_w_[i],
		 XmNvalueChangedCallback,
		 (XtCallbackProc)cbFeatureMenuTaskValue_,
		 (void *)taskData);

	}
    }

    featureMenuOption_w_ = new Widget*[numLocalFeatures];

    //  Create the feature raws sequentially.
    for (i = 0; i<numLocalFeatures; i++)
    {
	options = featureMenuUpTask_.localFeatures[i].options.size();

	// The frame for the feature.
	feature_frame = XtVaCreateWidget
	    ("feature_frame",
	     xmFrameWidgetClass, main,
	     XmNy, line*featureMenuLineHeight+2,
	     XmNheight, 2*featureMenuLineHeight,
	     XmNwidth, featureMenuWidth,
	     NULL);

	stackedWidget.push_back(feature_frame);

	// Two-colum widget for the feature.
	feature_box = XtVaCreateWidget
	    ("feature_box",
	     xmRowColumnWidgetClass, feature_frame,
	     XmNpacking, XmPACK_COLUMN,
	     XmNorientation, XmHORIZONTAL,
	     XmNalignment, XmALIGNMENT_BEGINNING,
	     XmNnumColumns, options,
	     XmNheight, 2*featureMenuLineHeight,
	     XmNwidth, featureMenuWidth,
	     XmNtopAttachment, XmATTACH_FORM,
	     XmNrightAttachment, XmATTACH_FORM,
	     XmNleftAttachment, XmATTACH_FORM,
	     XmNbottomAttachment, XmATTACH_WIDGET,
	     NULL);

	stackedWidget.push_back(feature_box);

	
	// Create the second dimention (option) of this array.
	featureMenuOption_w_[i] = new Widget[options];

	switch (featureMenuUpTask_.localFeatures[i].optionType) {

	case MEXP_FEATURE_OPTION_TOGGLE:
	    // The label column.
	    label = XtVaCreateWidget
		(featureMenuUpTask_.localFeatures[i].name.c_str(),
		 xmLabelGadgetClass, feature_box,
		 XmNalignment, XmALIGNMENT_BEGINNING,
		 XmNheight, featureMenuLineHeight,
		 XmNwidth, featureMenuWidth/options,
		 XmNtopAttachment, XmATTACH_WIDGET,
		 XmNtopWidget, feature_box,
		 XmNleftAttachment, XmATTACH_WIDGET,
		 XmNleftWidget, feature_box,
		 NULL);

	    stackedWidget.push_back(label);

	    // Fill the blanks in the label row.
	    for (j = 0; j < (options-1); j++)
	    {
		label = XtVaCreateWidget
		    (" ",
		     xmLabelGadgetClass, feature_box,
		     XmNalignment, XmALIGNMENT_BEGINNING,
		     XmNheight, featureMenuLineHeight,
		     XmNwidth, featureMenuWidth/options,
		     XmNtopAttachment, XmATTACH_WIDGET,
		     XmNtopWidget, feature_box,
		     NULL);

		stackedWidget.push_back(label);
	    }

	    for (j = 0; j < options; j++)
	    {
		featureOptionData = new FeatureMenuCallbackData_t;
		featureOptionData->featureId = i;
		featureOptionData->optionValue = (float)j;
		featureOptionData->missionExpertInstance = this;
		featureOptionData->optionType = featureMenuUpTask_.localFeatures[i].optionType;

		value = ((int)(featureMenuUpTask_.localFeatures[i].selectedOption) == j)? 1.0 : 0.0;

		button_frame = XtVaCreateManagedWidget
		    ("button_frame",
		     xmFormWidgetClass, feature_box,
		     XmNshadowThickness, 0,
		     XmNalignment, XmALIGNMENT_BEGINNING,
		     XmNheight, featureMenuLineHeight,
		     XmNwidth, featureMenuWidth/options-10,
		     NULL);

		stackedWidget.push_back(button_frame);

		// Toggle features.
		featureMenuOption_w_[i][j] = XtVaCreateWidget
		    (featureMenuUpTask_.localFeatures[i].options[j].value.c_str(),
		     xmToggleButtonGadgetClass, button_frame,
		     XmNradioBehavior, true,
		     XmNradioAlwaysOne, true,
		     XmNheight, featureMenuLineHeight,
		     XmNwidth, featureMenuWidth/options-10,
		     XmNset, (int)value,
		     XmNalignment, XmALIGNMENT_BEGINNING,
		     XmNshadowThickness, 0,
		     XmNhighlightColor, MExpParentBg_,
		     NULL);

		stackedWidget.push_back(featureMenuOption_w_[i][j]);

		XtAddCallback
		    (featureMenuOption_w_[i][j],
		     XmNvalueChangedCallback,
		     (XtCallbackProc)cbFeatureMenuValue_,
		     (void *)featureOptionData);
	    }

	    break;

	case MEXP_FEATURE_OPTION_SLIDER10:
	case MEXP_FEATURE_OPTION_SLIDER1:

	    switch (featureMenuUpTask_.localFeatures[i].optionType) {

	    case MEXP_FEATURE_OPTION_SLIDER10:
		sliderTitle = XmStringCreateLtoR(
		    (String)MEXP_DEFAULT_SLIDER10_RANAGE_TITLE.c_str(),
		    XmSTRING_DEFAULT_CHARSET);
		maxValue = 10;
		decimalPoints = 0;
		conversion = 1.0;
		break;

	    case MEXP_FEATURE_OPTION_SLIDER1:
	    default:
		sliderTitle = XmStringCreateLtoR(
		    (String)MEXP_DEFAULT_SLIDER1_RANAGE_TITLE.c_str(),
		    XmSTRING_DEFAULT_CHARSET);
		maxValue = 100;
		decimalPoints = 2;
		conversion = 100.0;
		break;
	    }

	    // The label column.
	    label = XtVaCreateWidget
		(featureMenuUpTask_.localFeatures[i].name.c_str(),
		 xmLabelGadgetClass, feature_box,
		 XmNalignment, XmALIGNMENT_BEGINNING,
		 XmNheight, 2*featureMenuLineHeight,
		 XmNwidth, featureMenuWidth/2,
		 XmNtopAttachment, XmATTACH_WIDGET,
		 XmNtopWidget, feature_box,
		 XmNleftAttachment, XmATTACH_WIDGET,
		 XmNleftWidget, feature_box,
		 NULL);

	    stackedWidget.push_back(label);

	    featureOptionData = new FeatureMenuCallbackData_t;
	    featureOptionData->featureId = i;
	    featureOptionData->optionValue = featureMenuUpTask_.localFeatures[i].selectedOption;
	    featureOptionData->missionExpertInstance = this;
	    featureOptionData->optionType = featureMenuUpTask_.localFeatures[i].optionType;

	    value = featureOptionData->optionValue*conversion;

	    featureMenuOption_w_[i][0] = XtVaCreateWidget
		("",
		 xmScaleWidgetClass, feature_box,
		 XmNmaximum, maxValue,
		 XmNminimum, 1,
		 XmNvalue, (int)value,
		 XmNorientation, XmHORIZONTAL,
		 XmNprocessingDirection, XmMAX_ON_RIGHT,
		 XmNshowValue, true,
		 XmNdecimalPoints, decimalPoints,
		 XmNheight, 2*featureMenuLineHeight,
		 XmNwidth, 3*featureMenuWidth/7,
		 XmNtopAttachment, XmATTACH_WIDGET,
		 XmNtopWidget, label,
		 XmNrightAttachment, XmATTACH_FORM,
		 XmNtitleString, sliderTitle,
		 XmNhighlightColor, MExpParentBg_,
		 NULL);

	    stackedWidget.push_back(featureMenuOption_w_[i][0]);

	    XtAddCallback
		(featureMenuOption_w_[i][0],
		 XmNvalueChangedCallback,
		 (XtCallbackProc)cbFeatureMenuValue_,
		 (void *)featureOptionData);

	    XmStringFree(sliderTitle);

	    break;
	}
	line += 2;
    }

    // The frame for the action field.
    action_frame = XtVaCreateManagedWidget
	("label_frame",
	 xmFormWidgetClass, main,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, 2*featureMenuLineHeight,
	 XmNwidth, featureMenuWidth-5,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 NULL);

    stackedWidget.push_back(action_frame);

    action = XtVaCreateWidget
	("action",
	 xmRowColumnWidgetClass, action_frame,
	 XmNpacking, XmPACK_COLUMN,
	 XmNorientation, XmVERTICAL,
	 XmNnumColumns, 2,
	 XmNisAligned, true,
	 XmNentryAlignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*featureMenuLineHeight,
	 XmNwidth, featureMenuWidth-5,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 NULL);

    stackedWidget.push_back(action);

    // The frame for the "Apply" buttons.
    apply_frame = XtVaCreateWidget
	("apply_frame",
	 xmFormWidgetClass, action,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*featureMenuLineHeight-10,
	 XmNwidth, featureMenuWidth/2-10,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, action,
	 XmNleftAttachment, XmATTACH_WIDGET,
	 XmNleftWidget, action,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 XmNbottomWidget, action,
	 NULL);

    stackedWidget.push_back(apply_frame);

    // The "Save" button.
    apply = XtVaCreateWidget
	(saveLabel.c_str(),
	 xmPushButtonGadgetClass, apply_frame,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*featureMenuLineHeight-10,
	 XmNwidth, featureMenuWidth/2,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 NULL);

    stackedWidget.push_back(apply);

    XtAddCallback
	(apply,
	 XmNactivateCallback,
	 (XtCallbackProc)cbApplyFeatureMenu_,
	 this);

    // The frame for the "Cancel" buttons.
    cancel_frame = XtVaCreateWidget
	("cancel_frame",
	 xmFormWidgetClass, action,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNx, featureMenuWidth/2,
	 XmNheight, 2*featureMenuLineHeight-10,
	 XmNwidth, featureMenuWidth/2-10,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, action,
	 XmNrightAttachment, XmATTACH_WIDGET,
	 XmNrightWidget, action,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 XmNbottomWidget, action,
	 NULL);

    stackedWidget.push_back(cancel_frame);

    // The "Cancel" button
    cancel = XtVaCreateWidget
	(cancelLabel.c_str(),
	 xmPushButtonGadgetClass, cancel_frame,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*featureMenuLineHeight-10,
	 XmNwidth, featureMenuWidth/2,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 NULL);

    stackedWidget.push_back(cancel);

    XtAddCallback
	(cancel,
	 XmNactivateCallback,
	 (XtCallbackProc)cbCloseFeatureMenu_,
	 (void *)this);

    numStackedWidget = stackedWidget.size();

    for (i = 0; i < numStackedWidget; i++)
    {
	XtManageChild(stackedWidget.back());
	stackedWidget.pop_back();
    }

    XtPopup (featureMenu_w_, XtGrabNone);

    // Wait for the user to close the window.
    eventCount = 0;
    while(featureMenuIsUp_)
    {
	// Run X
	if (XtAppPending(MExpAppContext_))
	{
	    XtAppProcessEvent(MExpAppContext_, XtIMAll);
	}

	if (eventCount > MAX_NOWAIT_EVENTCOUNT_)
	{
	    usleep(EVENT_WAITER_SLEEP_TIME_USEC_);
	}
	eventCount++;
    }
}

//-----------------------------------------------------------------------
// This function changes the value of the case feature.
//-----------------------------------------------------------------------
void MissionExpert::changeFeatureMenuValue_(XtPointer client_data)
{
    int i, j;
    bool selected;
    int sliderBarValue;
    float optionValue;
    char optionValueStr[256];

    FeatureMenuCallbackData_t *data = (FeatureMenuCallbackData_t *)client_data;
    i = data->featureId;

    switch (data->optionType) {

    case MEXP_FEATURE_OPTION_TOGGLE:
	featureMenuUpTask_.localFeatures[i].selectedOption = data->optionValue;
	
	for (j = 0; j < ((int)(featureMenuUpTask_.localFeatures[i].options.size())); j++)
	{
	    selected = ((int)(featureMenuUpTask_.localFeatures[i].selectedOption) == j)? true : false;

	    XtVaSetValues(
		featureMenuOption_w_[i][j],
		XmNset, selected,
		NULL);
	}
	break;

    case MEXP_FEATURE_OPTION_SLIDER10:
    case MEXP_FEATURE_OPTION_SLIDER1:

	XtVaGetValues(
	    featureMenuOption_w_[i][0],
	    XmNvalue, &sliderBarValue,
	    NULL);

	switch (data->optionType) {

	case MEXP_FEATURE_OPTION_SLIDER10:
	    optionValue = (float)sliderBarValue;
	    sprintf(optionValueStr, "%d", sliderBarValue);
	    break;

	case MEXP_FEATURE_OPTION_SLIDER1:
	default:
	    optionValue = (float)sliderBarValue/100.0;
	sprintf(optionValueStr, "%.2f", optionValue);
	    break;
	}

	featureMenuUpTask_.localFeatures[i].selectedOption = optionValue;
	featureMenuUpTask_.localFeatures[i].options[0].value = optionValueStr;

	break;
    }
}

//-----------------------------------------------------------------------
// This function changes the value of the global feature.
//-----------------------------------------------------------------------
void MissionExpert::changeGlobalFeatureMenuValue_(XtPointer client_data)
{
    int i, id, index;
    bool selected;
    int sliderBarValue;
    float optionValue;
    char optionValueStr[256];

    GlobalFeatureMenuCallbackData_t *data = (GlobalFeatureMenuCallbackData_t *)client_data;
    id = data->globalFeaturesId;

    index = 0;

    for (i = 0; i < ((int)(featureMenuUpGlobalFeatures_.size())); i++)
    {
	if (featureMenuUpGlobalFeatures_[i].id == id)
	{
	    index = i;
	    break;
	}
    }

    switch (data->optionType) {

    case MEXP_FEATURE_OPTION_TOGGLE:
	featureMenuUpGlobalFeatures_[index].selectedOption = data->optionValue;
	
	for (i = 0; i < ((int)(featureMenuUpGlobalFeatures_[index].options.size())); i++)
	{
	    selected = ((int)(featureMenuUpGlobalFeatures_[index].selectedOption) == i)? true : false;

	    XtVaSetValues(
		globalFeaturesMenuOption_w_[index][i],
		XmNset, selected,
		NULL);
	}

	break;

    case MEXP_FEATURE_OPTION_SLIDER10:
    case MEXP_FEATURE_OPTION_SLIDER1:

	XtVaGetValues(
	    globalFeaturesMenuOption_w_[index][0],
	    XmNvalue, &sliderBarValue,
	    NULL);

	switch (data->optionType) {

	case MEXP_FEATURE_OPTION_SLIDER10:
	    optionValue = (float)sliderBarValue;
	    sprintf(optionValueStr, "%d", sliderBarValue);
	    break;

	case MEXP_FEATURE_OPTION_SLIDER1:
	default:
	    optionValue = (float)sliderBarValue/100.0;
	    sprintf(optionValueStr, "%.2f", optionValue);
	    break;
	}

	featureMenuUpGlobalFeatures_[index].selectedOption = optionValue;
	featureMenuUpGlobalFeatures_[index].options[0].value = optionValueStr;

	break;
    }
}

//-----------------------------------------------------------------------
// This function changes the ID of the current task to be saved, and
// highlights the appropriate button of the case feature menu window.
//-----------------------------------------------------------------------
void MissionExpert::changeFeatureMenuTaskValue_(int taskId)
{
    int i;
    bool set;

    featureMenuUpTask_.id = taskId;

    for (i = 0; i < numTaskTypes_; i++)
    {
	if (defaultTaskList_[i].id == taskId)
	{
	    featureMenuUpTask_.name = defaultTaskList_[i].name;
	    break;
	}
    }

    for (i = 0; i < numTaskTypes_; i++)
    {
	set = (i == taskId)? true : false;

	XtVaSetValues(
	    featureMenuTask_w_[i],
	    XmNset, set,
	    NULL);
    }
}
//-----------------------------------------------------------------------
// This function creates a status window and pops it up.
//-----------------------------------------------------------------------
void MissionExpert::popupStatusWindow_(
    int rating,
    string missionSummary,
    bool prevBtnIsSensitive,
    bool nextBtnIsSensitive,
    bool loadBtnIsSensitive,
    bool cancelBtnIsSensitive,
    int currentMissionId)
{
    Widget mainFrame, main;
    Widget labelFrame, labelBox, label;
    Widget ratingFrame, ratingBox;
    Widget textFrame;
    Widget navigationFrame, navigation;
    Widget prevFrame;
    Widget nextFrame;
    Widget actionFrame, action;
    Widget loadFrame;
    Widget cancelFrame;
    vector<Widget> stackedWidget;
    Pixmap pixmap;
    string titleBoxText;
    string windowTitle;
    string ratingLabel;
    string navigationLabel;
    string prevLabel;
    string nextLabel;
    string loadLabel;
    string cancelLabel;
    Arg wargs[16];
    char buf[1024];
    int i, n, line = 0;
    int screenNumber;
    int displayHeight, displayWidth;
    int posX, posY;
    int statusWindowHeight, statusWindowWidth;
    int statusWindowLineHeight;
    int toptitleHeight, textHeight;
    int numStackedWidget;
    int numTextRows;
    int maxRetrievedMissions;
    bool isSensitive = false;

    // Create the title box comment.
    if (currentMissionId > -1)
    {
	maxRetrievedMissions = retrievedMissionSummary_.fsaSummaries.size();
	sprintf(buf, " %d/%d", currentMissionId+1, maxRetrievedMissions);
	titleBoxText = buf;
    }
    else
    {
	titleBoxText = " N/A";
    }

    // Create other labels.
    windowTitle =  DEFAULT_STATUSWINDOW_TITLE_;
    ratingLabel = DEFAULT_STATUSWINDOW_RATING_LABEL_;
    navigationLabel = DEFAULT_STATUSWINDOW_NAVIGATION_LABEL_;
    prevLabel = DEFAULT_STATUSWINDOW_PREV_BUTTON_LABEL_;
    nextLabel = DEFAULT_STATUSWINDOW_NEXT_BUTTON_LABEL_;
    loadLabel = DEFAULT_STATUSWINDOW_LOAD_BUTTON_LABEL_;
    cancelLabel = DEFAULT_STATUSWINDOW_CANCEL_BUTTON_LABEL_;

    // Create dimensions of the window.
    statusWindowWidth = DEFAULT_STATUSWINDOW_WIDTH_;
    statusWindowLineHeight = DEFAULT_STATUSWINDOW_LINEHEIGHT_;
    toptitleHeight = DEFAULT_STATUSWINDOW_TOPTITLE_HEIGHT_;
    textHeight = DEFAULT_STATUSWINDOW_TEXT_HEIGHT_;
    numTextRows = DEFAULT_STATUSWINDOW_TEXT_NUMROWS_;
    statusWindowHeight = (int)(
	textHeight + 15 +
	(statusWindowLineHeight+5)*
	(1 + // Top label
	 2 + // Navigation frame
	 2) // Save & Cancel buttons;
	);

    // Make sure the window does go outside the desktop.
    screenNumber = DefaultScreen(MExpDisplay_);
    displayWidth = DisplayWidth(MExpDisplay_, screenNumber);
    displayHeight = DisplayHeight(MExpDisplay_, screenNumber);
    posX = (displayWidth - statusWindowWidth)/2;
    posY = (displayHeight - statusWindowHeight)/2;
    
    // Popup shell.
    statusWindows_.popupShell = XtVaCreatePopupShell
	("",
	 xmDialogShellWidgetClass, MExpParent_, 
	 XmNdeleteResponse, XmUNMAP,
	 XmNallowShellResize, true,
	 XmNx, posX,
	 XmNy, posY,
	 XmNheight, statusWindowHeight,
	 XmNwidth, statusWindowWidth,
	 NULL);

    // The biggest frame.
    mainFrame = XtVaCreateWidget
	("mainFrame",
	 xmFormWidgetClass, statusWindows_.popupShell,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, statusWindowHeight,
	 XmNwidth, statusWindowWidth,
	 NULL);

    stackedWidget.push_back(mainFrame);

    main = XtVaCreateWidget
	("main",
	 xmRowColumnWidgetClass, mainFrame,
	 XmNpacking, XmPACK_TIGHT,
	 XmNorientation, XmVERTICAL,
	 XmNnumColumns, 1,
	 XmNisAligned, true,
	 XmNentryAlignment, XmALIGNMENT_CENTER,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 XmNheight, statusWindowHeight,
	 XmNwidth, statusWindowHeight,
	 NULL);

    stackedWidget.push_back(main);

    // The frame for the label.
    labelFrame = XtVaCreateWidget
	("labelFrame",
	 xmFrameWidgetClass, main,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, toptitleHeight,
	 XmNwidth, statusWindowWidth-5,
	 NULL);

    stackedWidget.push_back(labelFrame);

    // The widget for the main label.
    labelBox = XtVaCreateWidget
	("labelBox",
	 xmRowColumnWidgetClass, labelFrame,
	 XmNpacking, XmPACK_TIGHT,
	 XmNorientation, XmHORIZONTAL,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNnumColumns, 2,
	 XmNheight, toptitleHeight,
	 XmNwidth, statusWindowWidth-5,
	 NULL);

    stackedWidget.push_back(labelBox);

    // The main label.
    label = XtVaCreateWidget
	(windowTitle.c_str(),
	 xmLabelGadgetClass, labelBox,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, toptitleHeight,
	 XmNwidth, 4*statusWindowWidth/5-10,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, labelBox,
	 XmNleftAttachment, XmATTACH_WIDGET,
	 XmNleftWidget, labelBox,
	 NULL);

    stackedWidget.push_back(label);

    statusWindows_.titleBox = XtVaCreateWidget
	("",
	 xmTextWidgetClass, labelBox,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, toptitleHeight-5,
	 XmNwidth, statusWindowWidth/5,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, labelBox,
	 XmNrightAttachment, XmATTACH_WIDGET,
	 XmNrightWidget, labelBox,
	 XmNleftAttachment, XmATTACH_WIDGET,
	 XmNleftWidget, label,
	 XmNvalue, titleBoxText.c_str(),
	 XmNcursorPositionVisible, false,
	 XmNeditable, false,
	 XmNhighlightOnEnter, false,
	 XmNtraversalOn, false,
	 NULL);

    stackedWidget.push_back(statusWindows_.titleBox);

    line++;

    // The frame for the status.
    ratingFrame = XtVaCreateWidget
	("ratingFrame",
	 xmFrameWidgetClass, main,
	 XmNy, line*statusWindowLineHeight,
	 XmNheight, statusWindowLineHeight,
	 XmNwidth, statusWindowWidth-5,
	 NULL);

    stackedWidget.push_back(ratingFrame);

    ratingBox = XtVaCreateWidget
	("ratingBox",
	 xmRowColumnWidgetClass, ratingFrame,
	 XmNpacking, XmPACK_TIGHT,
	 XmNorientation, XmHORIZONTAL,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNnumColumns, MAX_SUITABILITY_RATING_+1,
	 XmNheight, statusWindowLineHeight,
	 XmNwidth, statusWindowWidth-5,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 NULL);

    stackedWidget.push_back(ratingBox);

    // The label column.
    label = XtVaCreateWidget
	(ratingLabel.c_str(),
	 xmLabelGadgetClass, ratingBox,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, statusWindowLineHeight,
	 XmNwidth, statusWindowWidth/2,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, ratingBox,
	 XmNleftAttachment, XmATTACH_WIDGET,
	 XmNleftWidget, ratingBox,
	 NULL);

    stackedWidget.push_back(label);
	
    for (i = 0; i < MAX_SUITABILITY_RATING_; i++)
    {
	pixmap = (i < rating)? starPixmap_ : emptyStarPixmap_;

	// Rating box
	statusWindows_.starBox[i] = XtVaCreateWidget
	    ("star",
	     xmLabelGadgetClass, ratingBox,
	     XmNlabelType, XmPIXMAP,
	     XmNlabelPixmap, pixmap,
	     XmNtopAttachment, XmATTACH_WIDGET,
	     XmNtopWidget, ratingBox,
	     XmNbottomAttachment, XmATTACH_WIDGET,
	     XmNbottomWidget, ratingBox,
	     NULL);
	
	stackedWidget.push_back(statusWindows_.starBox[i]);
    }

    line ++;

    // The frame for the text field.
    textFrame = XtVaCreateManagedWidget
	("textFrame",
	 xmFormWidgetClass, main,
	 XmNy, line*statusWindowLineHeight,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, 4*statusWindowLineHeight,
	 XmNwidth, textHeight,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 NULL);

    stackedWidget.push_back(textFrame);

    n = 0;
    XtSetArg(wargs[n], XmNscrollVertical, true); n++;
    XtSetArg(wargs[n], XmNscrollHorizontal, false); n++;
    XtSetArg(wargs[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg(wargs[n], XmNeditable, false); n++;
    XtSetArg(wargs[n], XmNcursorPositionVisible, false); n++;
    XtSetArg(wargs[n], XmNwordWrap, true); n++;
    XtSetArg(wargs[n], XmNrows, numTextRows); n++;
    statusWindows_.textBox = XmCreateScrolledText(textFrame, "textBox", wargs, n);

    XmTextReplace(statusWindows_.textBox, 0, statusWindows_.textPos, NULL);
    statusWindows_.textPos = 0;
    XmTextInsert(statusWindows_.textBox, 0, (char *)(missionSummary.c_str()));
    statusWindows_.textPos += missionSummary.size();
    XtVaSetValues(
	statusWindows_.textBox,
	XmNcursorPosition, statusWindows_.textPos,
	NULL);
    XmTextShowPosition(statusWindows_.textBox, 0);

    XtVaSetValues(
	XtParent(statusWindows_.textBox),
	XmNleftAttachment, XmATTACH_FORM,
	XmNtopAttachment, XmATTACH_FORM,
	XmNrightAttachment, XmATTACH_FORM,
	XmNbottomAttachment, XmATTACH_FORM,
	XmNhighlightOnEnter, false,
	XmNtraversalOn, false,
	NULL);

    stackedWidget.push_back(statusWindows_.textBox);

    // The frame for the navigation field.
    navigationFrame = XtVaCreateManagedWidget
	("navigationFrame",
	 xmFormWidgetClass, main,
	 XmNy, line*statusWindowLineHeight,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, 2*statusWindowLineHeight,
	 XmNwidth, statusWindowWidth-5,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 NULL);

    stackedWidget.push_back(navigationFrame);

    navigation = XtVaCreateWidget
	("navigation",
	 xmRowColumnWidgetClass, navigationFrame,
	 XmNpacking, XmPACK_COLUMN,
	 XmNorientation, XmVERTICAL,
	 XmNnumColumns, 2,
	 XmNisAligned, true,
	 XmNentryAlignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*statusWindowLineHeight,
	 XmNwidth, statusWindowWidth-5,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 NULL);

    stackedWidget.push_back(navigation);

    // The label column.
    label = XtVaCreateWidget
	(navigationLabel.c_str(),
	 xmLabelGadgetClass, navigation,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, statusWindowLineHeight-10,
	 XmNwidth, statusWindowWidth/2-5,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, navigation,
	 XmNleftAttachment, XmATTACH_WIDGET,
	 XmNleftWidget, navigation,
	 NULL);

    stackedWidget.push_back(label);
	
    // The frame for the "Previous" buttons.
    prevFrame = XtVaCreateWidget
	("prevFrame",
	 xmFormWidgetClass, navigation,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, statusWindowLineHeight-5,
	 XmNwidth, statusWindowWidth/2-10,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, navigation,
	 XmNleftAttachment, XmATTACH_WIDGET,
	 XmNleftWidget, navigation,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 XmNbottomWidget, navigation,
	 NULL);

    stackedWidget.push_back(prevFrame);

    // The "Previous" button.
    statusWindows_.prevBtn = XtVaCreateWidget
	(prevLabel.c_str(),
	 xmPushButtonGadgetClass, prevFrame,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, statusWindowLineHeight-5,
	 XmNwidth, statusWindowWidth/2-10,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 XmNhighlightOnEnter, false,
	 XmNtraversalOn, false,
	 NULL);


    isSensitive = (prevBtnIsSensitive)? true : false;
    XtSetSensitive(statusWindows_.prevBtn, isSensitive);

    XtAddCallback
	(statusWindows_.prevBtn,
	 XmNactivateCallback,
	 (XtCallbackProc)cbPrevBtnStatusWindow_,
	 this);

    stackedWidget.push_back(statusWindows_.prevBtn);

    // A blank.
    label = XtVaCreateWidget
	(EMPTY_STRING_.c_str(),
	 xmLabelGadgetClass, navigation,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, statusWindowLineHeight-10,
	 XmNwidth, statusWindowWidth/2-5,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, navigation,
	 XmNrightAttachment, XmATTACH_WIDGET,
	 XmNrightWidget, navigation,
	 NULL);

    stackedWidget.push_back(label);
	
    // The frame for the "Next" buttons.
    nextFrame = XtVaCreateWidget
	("nextFrame",
	 xmFormWidgetClass, navigation,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNx, statusWindowWidth/2,
	 XmNheight, statusWindowLineHeight-5,
	 XmNwidth, statusWindowWidth/2-10,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, navigation,
	 XmNrightAttachment, XmATTACH_WIDGET,
	 XmNrightWidget, navigation,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 XmNbottomWidget, navigation,
	 NULL);

    stackedWidget.push_back(nextFrame);

    // The "Next" button
    statusWindows_.nextBtn = XtVaCreateWidget
	(nextLabel.c_str(),
	 xmPushButtonGadgetClass, nextFrame,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, statusWindowLineHeight-5,
	 XmNwidth, statusWindowWidth/2-10,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 XmNhighlightOnEnter, false,
	 XmNtraversalOn, false,
	 NULL);

    isSensitive = (nextBtnIsSensitive)? true : false;
    XtSetSensitive(statusWindows_.nextBtn, isSensitive);

    XtAddCallback
	(statusWindows_.nextBtn,
	 XmNactivateCallback,
	 (XtCallbackProc)cbNextBtnStatusWindow_,
	 this);

    stackedWidget.push_back(statusWindows_.nextBtn);

    // The frame for the action field.
    actionFrame = XtVaCreateManagedWidget
	("actionFrame",
	 xmFormWidgetClass, main,
	 XmNy, line*statusWindowLineHeight,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNheight, 2*statusWindowLineHeight,
	 XmNwidth, statusWindowWidth-5,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 NULL);

    stackedWidget.push_back(actionFrame);

    action = XtVaCreateWidget
	("action",
	 xmRowColumnWidgetClass, actionFrame,
	 XmNpacking, XmPACK_COLUMN,
	 XmNorientation, XmVERTICAL,
	 XmNnumColumns, 2,
	 XmNisAligned, true,
	 XmNentryAlignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*statusWindowLineHeight,
	 XmNwidth, statusWindowWidth-5,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 NULL);

    stackedWidget.push_back(action);

    // The frame for the "Load" buttons.
    loadFrame = XtVaCreateWidget
	("loadFrame",
	 xmFormWidgetClass, action,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*statusWindowLineHeight-10,
	 XmNwidth, statusWindowWidth/2-10,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, action,
	 XmNleftAttachment, XmATTACH_WIDGET,
	 XmNleftWidget, action,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 XmNbottomWidget, action,
	 NULL);

    stackedWidget.push_back(loadFrame);

    // The "Load" button.
    statusWindows_.loadBtn = XtVaCreateWidget
	(loadLabel.c_str(),
	 xmPushButtonGadgetClass, loadFrame,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*statusWindowLineHeight-10,
	 XmNwidth, statusWindowWidth/2,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 XmNhighlightOnEnter, false,
	 XmNtraversalOn, false,
	 NULL);

    isSensitive = (loadBtnIsSensitive)? true : false;
    XtSetSensitive(statusWindows_.loadBtn, isSensitive);

    XtAddCallback
	(statusWindows_.loadBtn,
	 XmNactivateCallback,
	 (XtCallbackProc)cbLoadBtnStatusWindow_,
	 this);

    stackedWidget.push_back(statusWindows_.loadBtn);

    // The frame for the "Cancel" buttons.
    cancelFrame = XtVaCreateWidget
	("cancelFrame",
	 xmFormWidgetClass, action,
	 XmNalignment, XmALIGNMENT_BEGINNING,
	 XmNx, statusWindowWidth/2,
	 XmNheight, 2*statusWindowLineHeight-10,
	 XmNwidth, statusWindowWidth/2-10,
	 XmNtopAttachment, XmATTACH_WIDGET,
	 XmNtopWidget, action,
	 XmNrightAttachment, XmATTACH_WIDGET,
	 XmNrightWidget, action,
	 XmNbottomAttachment, XmATTACH_WIDGET,
	 XmNbottomWidget, action,
	 NULL);

    stackedWidget.push_back(cancelFrame);

    // The "Cancel" button
    statusWindows_.cancelBtn = XtVaCreateWidget
	(cancelLabel.c_str(),
	 xmPushButtonGadgetClass, cancelFrame,
	 XmNalignment, XmALIGNMENT_CENTER,
	 XmNheight, 2*statusWindowLineHeight-10,
	 XmNwidth, statusWindowWidth/2,
	 XmNtopAttachment, XmATTACH_FORM,
	 XmNrightAttachment, XmATTACH_FORM,
	 XmNleftAttachment, XmATTACH_FORM,
	 XmNbottomAttachment, XmATTACH_FORM,
	 XmNhighlightOnEnter, false,
	 XmNtraversalOn, false,
	 NULL);

    isSensitive = (cancelBtnIsSensitive)? true : false;
    XtSetSensitive(statusWindows_.cancelBtn, isSensitive);

    XtAddCallback
	(statusWindows_.cancelBtn,
	 XmNactivateCallback,
	 (XtCallbackProc)cbCancelBtnStatusWindow_,
	 (void *)this);

    stackedWidget.push_back(statusWindows_.cancelBtn);

    numStackedWidget = stackedWidget.size();

    for (i = 0; i < numStackedWidget; i++)
    {
	XtManageChild(stackedWidget.back());
	stackedWidget.pop_back();
    }

    statusWindows_.isUp = true;
    statusWindows_.currentMissionId = currentMissionId;
    statusWindows_.selectedMissionId = -1;

    XtPopup (statusWindows_.popupShell, XtGrabNone);
}

//-----------------------------------------------------------------------
// This function updates the status windows.
//-----------------------------------------------------------------------
bool MissionExpert::updateStatusWindow_(
    int rating,
    string missionSummary,
    bool prevBtnIsSensitive,
    bool nextBtnIsSensitive,
    bool loadBtnIsSensitive,
    bool cancelBtnIsSensitive,
    int currentMissionId)
{
    Pixmap pixmap, oldPixmap;
    string titleBoxText;
    char buf[1024];
    int i, maxRetrievedMissions;
    bool isSensitive = false;

    // Create the title box comment.
    if (currentMissionId > -1)
    {
	maxRetrievedMissions = retrievedMissionSummary_.fsaSummaries.size();
	sprintf(buf, " %d/%d", currentMissionId+1, maxRetrievedMissions);
	titleBoxText = buf;
    }
    else
    {
	titleBoxText = " N/A";
    }

    XtVaSetValues(
	statusWindows_.titleBox,
	XmNvalue, titleBoxText.c_str(),
	NULL);

    // Update rating.
    for (i = 0; i < MAX_SUITABILITY_RATING_; i++)
    {
	XtVaGetValues(
	    statusWindows_.starBox[i],
	    XmNlabelPixmap, &oldPixmap,
	    NULL);

	XmDestroyPixmap(XtScreen(statusWindows_.popupShell), oldPixmap);

	pixmap = (i < rating)? starPixmap_ : emptyStarPixmap_;

	XtVaSetValues(
	    statusWindows_.starBox[i],
	    XmNlabelType, XmPIXMAP,
	    XmNlabelPixmap, pixmap,
	    NULL);
    }

    // Update the mission summary
    XmTextReplace(statusWindows_.textBox, 0, statusWindows_.textPos, NULL);
    statusWindows_.textPos = 0;
    XmTextInsert(statusWindows_.textBox, 0, (char *)(missionSummary.c_str()));
    statusWindows_.textPos += missionSummary.size();
    XtVaSetValues(
	statusWindows_.textBox,
	XmNcursorPosition, statusWindows_.textPos,
	NULL);
    XmTextShowPosition(statusWindows_.textBox, 0);

    // Update sensitivity of buttons.
    isSensitive = (prevBtnIsSensitive)? true : false;
    XtSetSensitive(statusWindows_.prevBtn, isSensitive);

    isSensitive = (nextBtnIsSensitive)? true : false;
    XtSetSensitive(statusWindows_.nextBtn, isSensitive);

    isSensitive = (loadBtnIsSensitive)? true : false;
    XtSetSensitive(statusWindows_.loadBtn, isSensitive);

    isSensitive = (cancelBtnIsSensitive)? true : false;
    XtSetSensitive(statusWindows_.cancelBtn, isSensitive);

    statusWindows_.currentMissionId = currentMissionId;

    return true;
}

//-----------------------------------------------------------------------
// This function closes status window.
//-----------------------------------------------------------------------
void MissionExpert::closeStatusWindow_(void)
{
    Pixmap oldPixmap;
    int i;

    statusWindows_.textPos = 0;
    statusWindows_.isUp = false;

    for (i = 0; i < MAX_SUITABILITY_RATING_; i++)
    {
	XtVaGetValues(
	    statusWindows_.starBox[i],
	    XmNlabelPixmap, &oldPixmap,
	    NULL);

	XmDestroyPixmap(XtScreen(statusWindows_.popupShell), oldPixmap);
    }

    XtDestroyWidget(statusWindows_.popupShell);
}

//-----------------------------------------------------------------------
// This function browse next loadable mission in the status window.
//-----------------------------------------------------------------------
void MissionExpert::changeStatusWindow_(int direction)
{
    int index, maxRetrievedMissions;
    bool prevBtnIsSensitive, nextBtnIsSensitive;

    index = statusWindows_.currentMissionId;
    maxRetrievedMissions = retrievedMissionSummary_.fsaSummaries.size();

    switch (direction) {

    case STATUS_WINDOW_PREV:
	if ((index-1) >= 0)
	{
	    index--;
	}
	break;

    case STATUS_WINDOW_NEXT:
	if ((index+1) < maxRetrievedMissions)
	{
	    index++;
	}
	break;
    }

    prevBtnIsSensitive = (index > 0)? true : false;
    nextBtnIsSensitive = ((index+1) < maxRetrievedMissions)? true : false;

    updateStatusWindow_(
	retrievedMissionSummary_.ratings[index],
	retrievedMissionSummary_.fsaSummaries[index],
	prevBtnIsSensitive,
	nextBtnIsSensitive,
	true,
	true,
	index);
}

//-----------------------------------------------------------------------
// This function loads the mission that was selected by the user from
// the summary (i.e., the user input to the status window).
//-----------------------------------------------------------------------
bool MissionExpert::loadRetrievedMissionChosenByUser_(
    vector<MExpRetrievedMission_t> retrievedMissions,
    string overlayFileName)
{
    FILE *resultFile = NULL;
    char *cdlSolution = NULL;
    int numRetrievedMisssions = 0, selectedMissionId = -1;
    int eventCount;
    int status;
    bool missionLoaded = false;

    numRetrievedMisssions = retrievedMissions.size();

    // Wait for the user input
    eventCount = 0;
    while (statusWindows_.isUp)
    {
	if (XtAppPending(MExpAppContext_))
	{
	    XtAppProcessEvent(MExpAppContext_, XtIMAll);
	}

	if (eventCount > MAX_NOWAIT_EVENTCOUNT_)
	{
	    usleep(EVENT_WAITER_SLEEP_TIME_USEC_);
	}
	eventCount++;
    }

    // Make sure a valid mission ID is selected.
    selectedMissionId = statusWindows_.selectedMissionId;
    if ((selectedMissionId < 0) || (selectedMissionId >= numRetrievedMisssions))
    {
	return false;
    }
    
    // Open the result file.
    resultFile = fopen(resultFileName_.c_str(), "w");
    if (resultFile == NULL)
    {
	return false;
    }
    else
    {
	addCreatedDataFileList_(resultFileName_);
    }

    cdlSolution = acdl2cdl(
	retrievedMissions[selectedMissionId].acdlSolution.c_str(),
	robotName_,
	&status);

    if ((cdlSolution != NULL) && (status != ACDL2CDL_STATUS_FAILURE))
    {
	if (status == ACDL2CDL_STATUS_EMPTYMISSION)
	{
	    warn_userf("Retrieved mission is empty.");
	}

	// Copy the solution to the result file.
	fprintf(resultFile, "%s", cdlSolution);

	// Check the overlay file name.
	if (overlayFileName != EMPTY_STRING_)
	{
	    overlayFileName_ = overlayFileName;
	}

	missionLoaded = true;
	currentCBRLibraryDataIndexList_ = retrievedMissions[selectedMissionId].dataIndexList;
    }
    else
    {
	warn_userf("Retrieved mission cound not be convered to CDL.");
	missionLoaded = false;
    }
    
    fclose(resultFile);

    return missionLoaded;
}

//-----------------------------------------------------------------------
// This function loads the features from the RC file.
//-----------------------------------------------------------------------
bool MissionExpert::loadFeaturesFromRCFile_(void)
{
    MExpFeature_t feature;
    MExpFeatureOption_t option;
    MExpMissionTask_t task;
    const MExpFeature_t EMPTY_FEATURE;
    const MExpFeatureOption_t EMPTY_FEATURE_OPTION;
    const MExpMissionTask_t EMPTY_TASK;
    FILE *rcFile = NULL;
    string rcFilename;
    string bufString1, bufString2;
    char buf1[1024], buf2[1024], buf3[1024];

    // First, find the RC file.
    rcFilename = find_file_in_envpath(MEXP_RC_FILENAME);

    if (rcFilename == EMPTY_STRING_)
    {
	fprintf(stderr, "Error(mlab): RC file (%s) could not be loaded.\n", rcFilename.c_str());
	return false;
    }

    // Load the RC file.
    rcFile = fopen(rcFilename.c_str(), "r");

    // Initialize the vectors.
    defaultGlobalFeatures_.clear();
    defaultLocalFeatures_.clear();
		    
    while (fscanf(rcFile, "%s", buf1) != EOF)
    {
	bufString1 = buf1;

	if (bufString1[0] == '#')
	{
	    // It's a comment.
	    while ((fgetc(rcFile) != '\n') && (!feof(rcFile)))
	    {
		// Keep reading until next line.
	    }
	}
	else if (bufString1 == MEXP_RC_STRING_FEATURE_KEY)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    bufString2 = buf2;

	    if (bufString2 == MEXP_RC_STRING_FEATURE_BEGIN)
	    {
		feature = EMPTY_FEATURE;
		option = EMPTY_FEATURE_OPTION;
	    }
	    else if (bufString2 == MEXP_RC_STRING_FEATURE_END)
	    {
		switch (feature.type) {
		    
		case MEXP_FEATURE_GLOBAL:
		    feature.id = defaultGlobalFeatures_.size();
		    defaultGlobalFeatures_.push_back(feature);
		    break;

		case MEXP_FEATURE_LOCAL:
		    feature.id = defaultLocalFeatures_.size();
		    defaultLocalFeatures_.push_back(feature);
		    break;
		}
	    }
	}
	else if (bufString1 == MEXP_RC_STRING_FEATURE_TYPE)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    feature.type = atoi(buf2);
	}
	else if (bufString1 == MEXP_RC_STRING_FEATURE_NAME)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    feature.name = buf2;
	}
	else if (bufString1 == MEXP_RC_STRING_FEATURE_NON_INDEX)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    feature.nonIndex = atoi(buf2);
	}
	else if (bufString1 == MEXP_RC_STRING_FEATURE_WEIGHT)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    feature.weight = atof(buf2);
	}
	else if (bufString1 == MEXP_RC_STRING_FEATURE_OPTION_TYPE)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    feature.optionType = atoi(buf2);
	}
	else if (bufString1 == MEXP_RC_STRING_FEATURE_SELECTED_OPTION)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    feature.selectedOption = atof(buf2);
	}
	else if (bufString1 == MEXP_RC_STRING_FEATURE_OPTION)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    option.value = buf2;
	    feature.options.push_back(option);
	}
	else if (bufString1 == MEXP_RC_STRING_TASK_KEY)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    bufString2 = buf2;

	    if (bufString2 == MEXP_RC_STRING_TASK_BEGIN)
	    {
		task = EMPTY_TASK;
	    }
	    else if (bufString2 == MEXP_RC_STRING_TASK_END)
	    {
		task.id = defaultTaskList_.size();
		defaultTaskList_.push_back(task);
		numTaskTypes_ = defaultTaskList_.size();
	    }
	}
	else if (bufString1 == MEXP_RC_STRING_TASK_NAME)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    task.name = buf2;
	}
	else if (bufString1 == MEXP_RC_STRING_TASK_BUTTON_NAME)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    task.buttonName = buf2;
	}
	else if (bufString1 == MEXP_RC_STRING_TASK_SPECIFIC_SELECTED_OPTION)
	{
	    if (fscanf(rcFile, "%s %s", buf2, buf3) == EOF)
	    {
		break;
	    }
	    
	    feature.name = buf2;
	    feature.selectedOption = atof(buf3);
	    task.localFeatures.push_back(feature);
	}
	else if (bufString1 == MEXP_RC_STRING_DEFAULT_TASK)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    defaultTaskName_ = buf2;
	}
	else if (bufString1 == MEXP_RC_STRING_DEFAULT_TASK_WEIGHT)
	{
	    if (fscanf(rcFile, "%s", buf2) == EOF)
	    {
		break;
	    }
	    
	    defaultTaskWeight_ = atof(buf2);
	}
    }

    return true;
}

//-----------------------------------------------------------------------
// This function saves the current feature and the closes the case feature
// menu window.
//-----------------------------------------------------------------------
void MissionExpert::applyFeatureMenu_(void) 
{
    vector<MExpFeature_t> globalFeatures;
    int i;

    // Combine the global features.
    globalFeatures = fixedGlobalFeatures_;

    for (i = 0; i < (int)(featureMenuUpGlobalFeatures_.size()); i++)
    {
	globalFeatures.push_back(featureMenuUpGlobalFeatures_[i]);
    }

    currentCBRLibraryDataIndexList_ = cbrclient_->saveMissionPlan(
	globalFeatures,
	featureMenuUpTask_,
	defaultTaskWeight_,
	missionPlanFileName_);

    closeFeatureMenu_();

    missionSavedToCBRLibrary_ = true;
    shouldSaveMissionToCBRLibrary_ = false;
}

//-----------------------------------------------------------------------
// This function deletes the arrays of the instantiated widgets and
// closes the case feature menu window.
//-----------------------------------------------------------------------
void MissionExpert::closeFeatureMenu_(void)
{
    // Note: Here, assuming that each array has more than one element.
    //       Otherwise, "delete []" will fail.

    delete [] globalFeaturesMenuOption_w_;
    globalFeaturesMenuOption_w_ = NULL;

    delete [] featureMenuOption_w_;
    featureMenuOption_w_ = NULL;

    delete [] featureMenuTask_w_;
    featureMenuTask_w_ = NULL;

    XtDestroyWidget(featureMenu_w_);

    featureMenuIsUp_ = false;
}

//-----------------------------------------------------------------------
// This function deltes the data files which are created.
// Uncommenting "MExpDataFilesDir" in .cfgeditrc will prevent from
// the files being deleted.
//-----------------------------------------------------------------------
void MissionExpert::deleteCreatedDataFiles_(void)
{
    int i;

    for (i = 0; i < (int)(createdDataFileList_.size()); i++)
    {
	unlink(createdDataFileList_[i].c_str());
    }
}

//-----------------------------------------------------------------------
// This function adds the data file name into the list.
//-----------------------------------------------------------------------
void MissionExpert::addCreatedDataFileList_(string filename)
{
    createdDataFileList_.push_back(filename);
}

//-----------------------------------------------------------------------
// This function returns the name of the overlay file being specified
// earlier.
//-----------------------------------------------------------------------
string MissionExpert::getOverlayFileName(void)
{
    if (disableMExp_)
    {
	return EMPTY_STRING_;
    }

    return overlayFileName_;
}

//-----------------------------------------------------------------------
// This function returns the name of the feature file being specified
// earlier.
//-----------------------------------------------------------------------
string MissionExpert::getFeatureFileString(void)
{
    if (disableMExp_)
    {
	return EMPTY_STRING_;
    }

    return featureFileString_;
}

//-----------------------------------------------------------------------
// This function saves the name of the specifed overlay file.
//-----------------------------------------------------------------------
void MissionExpert::saveOverlayFileName(string overlayFileName)
{
    if (disableMExp_) return;

    if (overlayFileName != EMPTY_STRING_)
    {
	overlayFileName_ = overlayFileName;
    }
}

//-----------------------------------------------------------------------
// This callback function gets called when a toggle button for the option
// of case feature menu was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbFeatureMenuValue_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;

    missionExpertInstance = ((FeatureMenuCallbackData_t *)client_data)->missionExpertInstance;
    missionExpertInstance->changeFeatureMenuValue_(client_data);
}

//-----------------------------------------------------------------------
// This callback function gets called when a toggle button for the option
// of global feature menu was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbChangeGlobalFeatureMenuValue_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;

    missionExpertInstance = ((FeatureMenuCallbackData_t *)client_data)->missionExpertInstance;
    missionExpertInstance->changeGlobalFeatureMenuValue_(client_data);
}

//-----------------------------------------------------------------------
// This callback function gets called when a task button for the case
// feature menu was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbFeatureMenuTaskValue_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;
    int taskId;

    missionExpertInstance = ((FeatureMenuTaskCallbackData_t *)client_data)->missionExpertInstance;
    taskId = ((FeatureMenuTaskCallbackData_t *)client_data)->taskId;

    missionExpertInstance->changeFeatureMenuTaskValue_(taskId);
}

//-----------------------------------------------------------------------
// This callback function gets called when the "Apply" button of the
// case feature menu window was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbApplyFeatureMenu_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;

    missionExpertInstance = (MissionExpert *)client_data;
    missionExpertInstance->applyFeatureMenu_();
}

//-----------------------------------------------------------------------
// This callback function gets called when the "Close" button of the
// case feature menu window was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbCloseFeatureMenu_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;

    missionExpertInstance = (MissionExpert *)client_data;
    missionExpertInstance->closeFeatureMenu_();
}

//-----------------------------------------------------------------------
// This callback function sets the "Yes" / "No" variable according to the
// answer given by the user.
//-----------------------------------------------------------------------
void MissionExpert::cbGeneralYesNoWindow_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;
    bool result;

    missionExpertInstance = ((GeneralYesNoWindowCallbackData_t *)client_data)->missionExpertInstance;
    result = ((GeneralYesNoWindowCallbackData_t *)client_data)->result;

    missionExpertInstance->sendGeneralYesNoWindowResult_(result);
    XtDestroyWidget(w);
}

//-----------------------------------------------------------------------
// This callback function gets called when the "Previous" button of the
// status window window was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbPrevBtnStatusWindow_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;

    missionExpertInstance = (MissionExpert *)client_data;
    missionExpertInstance->changeStatusWindow_(STATUS_WINDOW_PREV);
}

//-----------------------------------------------------------------------
// This callback function gets called when the "Next" button of the
// status window window was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbNextBtnStatusWindow_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;

    missionExpertInstance = (MissionExpert *)client_data;
    missionExpertInstance->changeStatusWindow_(STATUS_WINDOW_NEXT);
}

//-----------------------------------------------------------------------
// This callback function gets called when the "Load" button of the
// status window window was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbLoadBtnStatusWindow_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;

    missionExpertInstance = (MissionExpert *)client_data;
    missionExpertInstance->selectMissionForStatusWindow_();
    missionExpertInstance->closeStatusWindow_();
}

//-----------------------------------------------------------------------
// This callback function gets called when the "Cancel" button of the
// status window window was pressed.
//-----------------------------------------------------------------------
void MissionExpert::cbCancelBtnStatusWindow_(Widget w, XtPointer client_data, XtPointer callback_data)
{
    MissionExpert *missionExpertInstance;

    missionExpertInstance = (MissionExpert *)client_data;
    missionExpertInstance->closeStatusWindow_();
}

//-----------------------------------------------------------------------
// This function gathers all the names (and descriptions) of states and
// triggers.
//-----------------------------------------------------------------------
config_fsa_lists_t MissionExpert::getFSALists_(void)
{
    config_fsa_lists_t fsaLists;

    fsaLists.taskList = config->get_task_list();
    fsaLists.triggerList = config->get_trigger_list();

    return fsaLists;
}

//-----------------------------------------------------------------------
// This function creates pixmaps to be used, and put them in a list.
//-----------------------------------------------------------------------
void MissionExpert::createPixmaps_(Pixel fg, Pixel bg, Widget parent)
{
    Pixel yellowPixel, grayPixel;
    Colormap colorMap;
    XColor color, ignore;

    colorMap = DefaultColormap(MExpDisplay_, DefaultScreen(MExpDisplay_));

    // Create star.

    yellowPixel = fg;

    if (XAllocNamedColor(MExpDisplay_, colorMap, "yellow", &color, &ignore))
    {
	yellowPixel = color.pixel;
    }

    starPixmap_ = MEXP_CREATE_PIXMAP(star, yellowPixel, bg, parent);

    // Create empty star.

    grayPixel = fg;

    if (XAllocNamedColor(MExpDisplay_, colorMap, "gray40", &color, &ignore))
    {
	grayPixel = color.pixel;
    }

    emptyStarPixmap_ = MEXP_CREATE_PIXMAP(empty_star, grayPixel, bg, parent);
}

//-----------------------------------------------------------------------
// This function is called when the mission is modified by the user.
//-----------------------------------------------------------------------
void MissionExpert::madeChange(void)
{
    if (disableMExp_)
    {
	return;
    }

    if (missionSavedToCBRLibrary_ || missionLoadedFromCBRLibrary_)
    {
	shouldSaveMissionToCBRLibrary_ = true;
	currentCBRLibraryDataIndexList_.clear();
	missionSavedToCBRLibrary_ = false;
	missionLoadedFromCBRLibrary_ = false;
    }
}

//-----------------------------------------------------------------------
// This function checks whether the user-feedback should be inquired.
//-----------------------------------------------------------------------
bool MissionExpert::shouldAskFeedback(void)
{
    if (disableMExp_)
    {
	return false;
    }

    if ((int)(currentCBRLibraryDataIndexList_.size()) > 0)
    {
	return true;
    }
    
    return false;
}

//-----------------------------------------------------------------------
// This function checks to see if the mission (ACDL) contains a specified
// agent.
//-----------------------------------------------------------------------
bool MissionExpert::hasAgentInMissionPlan_(string agentName)
{
    FILE *tmpACDLFile = NULL;
    char buf1[1024], buf2[1024];
    string bufString1, bufString2;
    bool found = false;
    
    // Now, open up the file, again, for reading.
    tmpACDLFile = fopen(missionPlanFileName_.c_str(), "r");

    // Process the data.
    while (fscanf(tmpACDLFile, "%s", buf1) != EOF)
    {
	bufString1 = buf1;

	if (bufString1 == ACDL_STRING_AGENT_NAME_)
	{
	    if (fscanf(tmpACDLFile, "%s", buf2) != EOF)
	    {
		bufString2 = buf2;

		if (bufString2 == agentName)
		{
		    found = true;
		    break;
		}
	    }
	}
    }

    fclose(tmpACDLFile);

    return found;
}

/**********************************************************************
 * $Log: mission_expert.cc,v $
 * Revision 1.3  2003/04/06 08:50:37  endo
 * Updated for CBR Wizard Prototype II.
 *
 * Revision 1.2  2002/01/31 10:44:20  endo
 * Parameters chaged due to the change of max_vel and base_vel.
 *
 * Revision 1.1  2002/01/12 23:09:58  endo
 * Initial revision
 *
 **********************************************************************/
