/**********************************************************************
 **                                                                  **
 **                             popups.h                             **
 **                                                                  **
 **                                                                  **
 **  Written by:  Douglas C. MacKenzie                               **
 **                                                                  **
 **  Copyright 1995, 1996, 1997 Georgia Tech Research Corporation    **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **********************************************************************/

/* $Id: popups.h,v 1.14 2002/01/12 23:23:16 endo Exp $ */


#ifndef POPUPS_H
#define POPUPS_H

const int MAX_ARGS = 32;
const int MAX_CBs  = 16;
const XtPointer PASS_WIDGET  = (XtPointer)-31895;  // Weird string to match 

typedef Widget (*DIALOG_BLDR)(Widget, char *, ArgList, Cardinal);
typedef struct S_args {
      char     *name;
      int  	value;
   } S_args; 		// the arg array

typedef struct S_cbs {
      char           *name;	// Name of callbacks
      XtCallbackProc  proc; 	// fnc to call
      void 	     *data;	// Client data
   } S_cbs;

typedef struct dialog_rec {
   DIALOG_BLDR 	func;		// Function creates and returns dialog widget
   S_args 	args[MAX_ARGS];	// the arg array
   S_cbs  	cbs[MAX_CBs];
} dialog_rec;

extern void file_open_ok_cb(Widget w,caddr_t client_data, XmFileSelectionBoxCallbackStruct * fcb);
extern void file_open_dialog();

void warn_user(char *msg);
void warn_userf(const char *fmt, ...);
void create_error_dialog(Widget parent);
void help_cb(Widget w, int item_num);
void create_help_dialog(Widget parent);
void open_dialog_box(Widget w, dialog_rec *rec, XmAnyCallbackStruct *cbs);
int wait_dialog_box(Widget w, dialog_rec *rec, XmAnyCallbackStruct *cbs);
int wait_SelectionBox(Widget w, dialog_rec *rec, char **result);

class PopUpWindows {

typedef enum InitialDialogResult_t {
    IDB_CANCEL, IDB_NEW, IDB_LOAD
};

protected:
    Widget PopWinParent_;
    XtAppContext PopWinAppContext_;

    Widget InitialDialog_w_;
    int PopWinIDBResult_;
    bool PopWinIDBDone_;

    void createInitialDialog_(void);
    void popupInitialDialog_(void);

public:
    PopUpWindows(Widget parent, XtAppContext app);

    void runInitialDialog(void);
    void sendInitialDialogResult(int result);
    bool quitConfirm(Widget w);
    bool discardConfirm(Widget w);
};

void InitializeDialog_cb(Widget w, XtPointer client_data, XtPointer callback_data);

extern PopUpWindows *gPopWin;
#endif  


/**********************************************************************
 * $Log: popups.h,v $
 * Revision 1.14  2002/01/12 23:23:16  endo
 * Mission Expert functionality added.
 *
 * Revision 1.13  2000/10/16 19:33:40  endo
 * Modified due to the compiler upgrade.
 *
 * Revision 1.12  2000/03/14 00:01:27  endo
 * The "Start Over" button was added to CfgEdit.
 *
 * Revision 1.11  1997/02/14 16:42:11  zchen
 * *** empty log message ***
 *
 * Revision 1.10  1995/06/29  18:09:37  jmc
 * Added copyright message.
 *
 * Revision 1.9  1995/02/28  14:52:28  doug
 * *** empty log message ***
 *
 * Revision 1.7  1995/02/08  20:34:38  doug
 * *** empty log message ***
 *
 * Revision 1.6  1995/02/01  18:50:43  doug
 * *** empty log message ***
 *
 * Revision 1.5  1995/02/01  00:09:16  doug
 * *** empty log message ***
 *
 * Revision 1.5  1995/02/01  00:09:16  doug
 * *** empty log message ***
 *
 * Revision 1.4  1995/01/26  00:02:31  doug
 * *** empty log message ***
 *
 * Revision 1.3  1994/12/09  19:07:20  doug
 * *** empty log message ***
 *
 * Revision 1.3  1994/12/09  19:07:20  doug
 * *** empty log message ***
 *
 * Revision 1.2  1994/12/06  17:08:49  doug
 * *** empty log message ***
 *
 * Revision 1.2  1994/12/06  17:08:49  doug
 * *** empty log message ***
 *
 * Revision 1.1  1994/10/26  15:57:50  doug
 * Initial revision
 *
 **********************************************************************/
