/**********************************************************************
 **                                                                  **
 **                     renumber_robots.cc                           **
 **                                                                  **
 **                                                                  **
 **  Written by:  Douglas C. MacKenzie                               **
 **                                                                  **
 **  Copyright 1995, 1996, 1997 Georgia Tech Research Corporation    **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **********************************************************************/

/* $Id: renumber_robots.cc,v 1.6 2003/04/06 12:25:34 endo Exp $ */

#include <stdio.h>
#include <malloc.h>

#include <Xm/Xm.h>

#include "list.hpp"
#include "load_cdl.h"
#include "configuration.hpp"
#include "globals.h"
#include "cfg_apply.h"
#include "renumber_robots.h"

int num_robots;
static SymbolList robot_records;
static bool changed;

// NOTE: The robots are named "CfgRobotxxx" where xxx is a 1 based integer.
//        and cfg is the name of the current configuration.

/*-----------------------------------------------------------------------*/

char *
RobotName(const int n)
{
   char buf[256];
   sprintf(buf,"%sRobot%d", config->configuration_name(), n); 
   return strdup(buf);
}


/*-----------------------------------------------------------------------*/

// Renumber this robot (if this is in fact a robot)
// Returns true if successful
static bool
renumber(Symbol * rec, Symbol *page_def, Symbol *robot,
         SymbolList * page_stack, char **message, bool *restart)
{
   // Don't care if isn't a robot
   if (rec == NULL || rec->symbol_type != ROBOT_NAME )
   {
      return true;
   }

   // Already handled
   if( robot_records.exists(rec) )
   {
      return true;
   }

   num_robots ++;
   char *name = RobotName(num_robots);
   if( rec->name == NULL || strcmp(rec->name, name) )
   {
      if( rec->name )
         free( rec->name );
      rec->name = name;
      changed = true;
   }
   else
   {
      free(name);
   }

   // Assign the ID of the robot.
   rec->id = num_robots;

   // Remember we did this one.
   robot_records.append(rec);

   return true;
}

/*-----------------------------------------------------------------------*/

/* called when binding button is clicked in glyph */
void
renumber_robots(void)
{
   robot_records.clear();
   num_robots = 0;

   // Add any missing robot binding points
   changed = false;
   cfg_apply(config->root(), renumber);

   if( changed )
   {
      // Regenerate the drawing to get the changes
      config->regen_page();

      // Mark drawing as changed
      config->made_change();
   }
}

///////////////////////////////////////////////////////////////////////
// $Log: renumber_robots.cc,v $
// Revision 1.6  2003/04/06 12:25:34  endo
// gcc 3.1.1
//
// Revision 1.5  2001/02/01 20:06:55  endo
// Modification to display the robot ID and name in the
// "Individual Robot" glyph was added.
//
// Revision 1.4  1997/02/14 16:53:35  zchen
// *** empty log message ***
//
// Revision 1.3  1996/05/08  15:16:39  doug
// fixing compile warnings
//
// Revision 1.2  1996/05/07  19:53:20  doug
// fixing compile warnings
//
// Revision 1.1  1996/02/28  03:57:00  doug
// Initial revision
//
///////////////////////////////////////////////////////////////////////
