/**********************************************************************
 **                                                                  **
 **                              hclient.c                           **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  HServer external references for robot_side_com.c                **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: hclient.h,v 1.5 2003/04/02 22:16:30 zkira Exp $ */

#ifndef HCLIENT_H
#define HCLIENT_H

#include "hserver/hserver.h"


/* Stop robot motion */
int hclient_stop(void);

/* Start sonar */
int hclient_range_start();

/* Stop sonar */
int hclient_range_stop(void);

/* Set robot speed */
int hclient_drive(float speed); /* meter per second */

/* Set robot angular speed */
int hclient_steer(float avel); /* degrees per second */

/* Set robot location */
int hclient_setxyt(float x,float y,float theta); /* meters, meters, degrees */

/* Close robot connection */
int hclient_close_robot(void);

/* Open robot connection. Name is the hserver name */
int hclient_open_robot(char *name,bool multipleHservers);

/* Steer toward theta and when within drive_wait_angle start to drive at speed */
int hclient_steer_toward_and_drive(float speed,float theta,float drive_wait_angle = 15.0,int use_reverse = true);

/* Return robot location */
int hclient_getxyt(double *x,double *y,double *t);

/* Return battery reading */
int hclient_get_battery(int *level);

/* Return GPS */
int hclient_get_gps(double *lat,double *lon,double *dir);

/* Pan/tilt camera to location */
int hclient_move_camera( int pandegree, int tiltdegree );

/* Set camera tracking mode. */
int hclient_set_camera_tracker_mode(int mode);

/* Return side ranges from laser. Used for door trigger */
int hclient_get_side_ranges(float *left,float *right);

/* Return new sensor readings from hserver */
int hclient_get_sensor_readings(int mask,int *num,sensor_t **sensorPtr);

/* Return obstacles from latest sensor reading */
int hclient_get_obstacles(int mask,float danger_range,sensor_reading_t **reading,bool adjust = false);

/* Return array of latest laser reading */
int hclient_laser_read(float *reading);

/* Update sensor readings from hserver */
int hclient_update_sensors();

/* Capture fram and save to file */
int hclient_frame_upload(int channel,char *name);

#endif

/**********************************************************************
# $Log: hclient.h,v $
# Revision 1.5  2003/04/02 22:16:30  zkira
# GPS and battery information for telop GUI
#
# Revision 1.4  2000/09/19 03:52:33  endo
# Sonar bug fixed.
#
#
#**********************************************************************/
