/**********************************************************************
 **                                                                  **
 **                        ContrilInterfaces.c                       **
 **                                                                  **
 **                                                                  **
 **  Written by:  Brian Lee                                          **
 **                                                                  **
 **  Copyright 2002, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: ControlInterfaces.c,v 1.3 2002/02/18 13:48:21 endo Exp $ */

#include <string>

#include "ControlInterfaces.h"
#include "message.h"
#include "fred.h"
#include "nomad.h"
#include "pioneer.h"
#include "amigobot.h"

#ifndef NO_MOBILITY
#include "urban.h"
#include "atrvjr.h"
#endif

void robotConnect( Robot*& robot )
{
    char c;
    string strHost;
    string strPortString;

    char robot_type = messageGetChar( EnMessageFilterType_ROBOT_TYPE, 
                                      EnMessageType_ROBOT_TYPE, EnMessageErrType_ROBOT_TYPE );
    if ( robot_type != 'x' )
    {
        const unsigned int uPIONEER_PORT_ECHO_LINE = 4;
        const unsigned int uAMIGOBOT_PORT_ECHO_LINE = 4;

        switch( robot_type )
        {
        case '1': // Emulated Robot (fred)
            new Fred(&robot);
            break;

        case '2': // Pioneer AT
            strPortString = messageGetString( EnMessageType_PIONEER_PORT, uPIONEER_PORT_ECHO_LINE );
            new Pioneer( &robot, strPortString );
            break;

        case '3': // AmigoBot
            strPortString = messageGetString( EnMessageType_AMIGOBOT_PORT, uAMIGOBOT_PORT_ECHO_LINE );
            new AmigoBot( &robot, strPortString );
            break;

        case '4': // Nomad 150/200
            c = messageGetChar( EnMessageFilterType_NOMAD, EnMessageType_NOMAD, 
                                EnMessageErrType_NOMAD_TYPE );
            if ( c != 'x' )
            {
                const unsigned int uNOMAD_150_PORT_ECHO_LINE = 4;
                const unsigned int uNOMAD_200_HOST_NAME_ECHO_LINE = 3;

                Nomad::EnNomadType enType;

                switch( c - '0' )
                {
                case 1:
                    enType = Nomad::EnNomadType_150;
                    strPortString = messageGetString( EnMessageType_NOMAD_150_PORT, 
                                                      uNOMAD_150_PORT_ECHO_LINE );
                    strHost = "";
                    break;
                case 2:
                    enType = Nomad::EnNomadType_200;
                    strPortString = "";
                    strHost = messageGetString( EnMessageType_NOMAD_200_HOST_NAME, 
                                                uNOMAD_200_HOST_NAME_ECHO_LINE );
                    break;
                default:
                    printTextWindow( "Bad Nomad type in robotConnect()" );
                    return;
                }
		 // Ernest - add global flag indicating watchdog enable/disable
		new Nomad( &robot, enType, strPortString, strHost, g_bWatchDogEnable );
            }
            break;

        case '5': // Urban Robot
#ifndef NO_MOBILITY	
            new Urban( &robot );
#else 
            printTextWindow( "No Mobility support" );
#endif
            break;

        case '6': // ATRV-Jr Robot
#ifndef NO_MOBILITY	
	    // Ernest - add global flag indicating watchdog enable/disable
	    new ATRVJr( &robot,g_bWatchDogEnable );
#else 
            printTextWindow( "No Mobility support" );
#endif
            break;
        }
    }
}

void connectToGps( Gps*& gps, double base_lat, double base_lon, double base_x, double base_y, 
                 double mperlat, double mperlon )
{
    int c = messageGetChar( EnMessageFilterType_GPS, EnMessageType_GPS_SELECT, EnMessageErrType_GPS );
    if ( c != 'x' )
    {
        bool base = ( c == '0' );
        unsigned const int uPORT_ECHO_LINE = 4;

        // get the connection port from the user
        string strPort = messageGetString( EnMessageType_GPS_PORT, uPORT_ECHO_LINE );

        new Gps( &gps, strPort, base_lat, base_lon, base_x, base_y, mperlat, mperlon, base );
    }
}

void controlGps( Gps* gps )
{
    bool bDone = false;
    int msgData[ 4 ];

    msgData[ 0 ] = gps->GetSavingPackets();
    msgData[ 1 ] = (int) gps->GetBaseX();
    msgData[ 2 ] = (int) gps->GetBaseY();

    messageDrawWindow( EnMessageType_GPS_CONTROL, EnMessageErrType_NONE, msgData );
    do
    {
        int c = getch();
        switch( c )
        {
        case 'd':
            messageHide();
            delete gps;
            return;
            break;

        case 's':
            gps->SetSavePackets( !gps->GetSavingPackets() );
            msgData[ 0 ] = gps->GetSavingPackets();
            messageDrawWindow( EnMessageType_GPS_CONTROL, EnMessageErrType_NONE, msgData );
            break;
        case 'q':
            gps->AlterBaseYBy( 1.0 );
            msgData[ 2 ] = (int) gps->GetBaseY();
            messageDrawWindow( EnMessageType_GPS_CONTROL, EnMessageErrType_NONE, msgData );
            break; 
        case 'a':
            gps->AlterBaseYBy( -1.0 );
            msgData[ 2 ] = (int) gps->GetBaseY();
            messageDrawWindow( EnMessageType_GPS_CONTROL, EnMessageErrType_NONE, msgData );
            break; 
        case 'w':
            gps->AlterBaseXBy( -1.0 );
            msgData[ 1 ] = (int) gps->GetBaseX();
            messageDrawWindow( EnMessageType_GPS_CONTROL, EnMessageErrType_NONE, msgData );
            break; 
        case 'e':
            gps->AlterBaseXBy( 1.0 );
            msgData[ 1 ] = (int) gps->GetBaseX();
            messageDrawWindow( EnMessageType_GPS_CONTROL, EnMessageErrType_NONE, msgData );
            break; 
            gps->information();
            break;
        case 'x':
        case 'Q':
        case KEY_ESC:
            bDone = true;
            break;
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case KEY_UP:
        case KEY_DOWN:
        case KEY_LEFT:
        case KEY_RIGHT:
            messageMovePanelKey( c );
            break;
        }
    } while ( !bDone );
    messageHide();
}

#ifndef NO_MOBILITY	
void connectToGyroDMUVGX( GyroDMUVGX*& pclGyroDMUVGX )
{
    new GyroDMUVGX( &pclGyroDMUVGX );
}

void controlGyroDMUVGX( GyroDMUVGX* pclGyroDMUVGX )
{
    int c;
    bool bDone = false;

    messageDrawWindow( EnMessageType_GYRO_DMU_VGX_CONTROL, EnMessageErrType_NONE );
    redrawWindows();
    do
    {
        c = getch();
        switch( c )
        {
        case 'd':
            messageHide();
            delete pclGyroDMUVGX;
            bDone = true;
            break;
        case 'r':
            pclGyroDMUVGX->resetAngles();
            break;
        case 'x':
        case 'Q':
        case KEY_ESC:
            bDone = true;
            break;
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case KEY_UP:
        case KEY_DOWN:
        case KEY_LEFT:
        case KEY_RIGHT:
            messageMovePanelKey( c );
            break;
        }
    } while ( !bDone );
    messageHide();
}
#endif




#ifndef NO_MOBILITY	
void connectToCompassKVHC100( CompassKVHC100*& pclCompassKVHC100 )
{
    new CompassKVHC100( &pclCompassKVHC100 );
}

void controlCompassKVHC100( CompassKVHC100* pclCompassKVHC100 )
{
    int c;
    bool bDone = false;

    messageDrawWindow( EnMessageType_COMPASS_KVH_C100_CONTROL, EnMessageErrType_NONE );
    redrawWindows();
    do
    {
        c = getch();
        switch( c )
        {
        case 'd':
            messageHide();
            delete pclCompassKVHC100;
            bDone = true;
            break;
        case 'x':
        case 'Q':
        case KEY_ESC:
            bDone = true;
            break;
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case KEY_UP:
        case KEY_DOWN:
        case KEY_LEFT:
        case KEY_RIGHT:
            messageMovePanelKey( c );
            break;
        }
    } while ( !bDone );
    messageHide();
}
#endif

/**********************************************************************
# $Log: ControlInterfaces.c,v $
# Revision 1.3  2002/02/18 13:48:21  endo
# AmigoBot added.
#
#
#**********************************************************************/
