/**********************************************************************
 **                                                                  **
 **                             WatchDog.h                           **
 **                                                                  **
 **  Written by: Ernest P. Merrill                                   **
 **                                                                  **
 **  Copyright 2002 - 2003, Georgia Tech Research Corporation        **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: WatchDog.h,v 1.2 2003/04/06 13:32:37 endo Exp $ */

#ifndef WATCHDOG_H_
#define WATCHDOG_H_

#include <pthread.h>
#include <vector>
#include <sys/time.h>
#include <unistd.h>
#include <fstream>
#include <iostream>

using std::vector;
using std::ofstream;

struct SuWDData 
{

    struct timeval* tv_wd; // timestamp for watchdog thread
    pthread_mutex_t* wd_mutex; // mutex for watchdog
    bool enable_wd;       // flag to enable or disable watchdog

    SuWDData() {}
    SuWDData( const SuWDData& suOriginal ) { *this = suOriginal; }
    inline SuWDData& operator=( const SuWDData& suOriginal );

};

inline SuWDData& SuWDData::operator=( const SuWDData& suOriginal )
{
    if ( this != &suOriginal )
    {
        tv_wd = suOriginal.tv_wd;
        wd_mutex = suOriginal.wd_mutex;
	enable_wd = suOriginal.enable_wd;
    }

    return *this;
}

class ClWatchDog
{

public:

    ClWatchDog ();
    ~ClWatchDog ();
    void StartWatchdog();
    void StopWatchdog();
    void AddNomad(struct timeval* nomad_tv, pthread_mutex_t* nomad_mutex);
    void EnableNomadWD();
    void DisableNomadWD();
    void AddPositionCalculator(struct timeval* watchdog_tv, pthread_mutex_t* watchdog_mutex);
    void AddLaser(struct timeval* watchdog_tv, pthread_mutex_t* watchdog_mutex);
    void AddIPTHandler(struct timeval* watchdog_tv, pthread_mutex_t* watchdog_mutex);
    void AddATRVJr(struct timeval* nomad_tv, pthread_mutex_t* nomad_mutex);
     

protected:
    vector<SuWDData> g_clNomadList;
    vector<SuWDData> g_clLaserList;
    vector<SuWDData> g_clATRVJrList;
    ofstream* wddebugout;
    void WatchdogLoop();
    static void StartThread(void* wd_instance);
    pthread_t watchdog_th;
    bool position_calculator;
    bool ipt_handler_check;
    bool ipt_handler_present; 
    void CheckPosCalculator();
    void CheckNomad();
    void CheckLaser();
    void CheckATRVJr();
    void CheckIPTHandler();
    bool keeprunning;
    pthread_mutex_t keeprunning_mutex;
    pthread_mutex_t nomadenable_mutex;
    pthread_mutex_t debug_mutex; 
    void PrintDebug (const char* fmt, ...);

    static const int MAX_PRINT_BUFFER_SIZE;
};

extern ClWatchDog* WatchDog;

#endif

/**********************************************************************
 * $Log: WatchDog.h,v $
 * Revision 1.2  2003/04/06 13:32:37  endo
 * gcc 3.1.1
 *
 *
 **********************************************************************/
