/**********************************************************************
 **                                                                  **
 **                             atrvjr.c                             **
 **                                                                  **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  RWI ATRV-Jr robot control for HServer                           **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: atrvjr.c,v 1.6 2003/04/02 22:20:19 zkira Exp $ */

#define USE_ATRVJR
#include "atrvjr.h"

static void start_thread( void* atrvjr_instance )
{
    ( (ATRVJr*) atrvjr_instance )->reader_thread();
}

// Ernest - added watchdog enable flag
ATRVJr::ATRVJr( Robot** a, bool watchDogEnable = true ) : Robot( a, rt_atrvjr, "ATRVJr" )
{
    int rtn;
    goalvelocity = 0; // Ernest - initialize the variable
    goalavelocity = 0; // Ernest - initialize the variable
    if ( report_level > HS_REP_LEV_NONE )
    {
        printTextWindow( "Connecting to ATRV-Jr... ", false );
    }

    // Set the sonar angles.
    float tmp_angle[ ATRVJR_NUM_SONAR ] = ATRVJR_RANGE_ANGLE;
    atrvjr_range_angle = tmp_angle;

    // Set the sonar locations.
    float tmp_loc[ ATRVJR_NUM_SONAR ] = ATRVJR_RANGE_LOC;
    atrvjr_range_loc = tmp_loc;

    for ( int i = 0; i < 3; i++ )
    {
        atrvjr_xyt[ i ] = 0.0;
    }
    Robot::setXYT( 0.0, 0.0, 0.0 );

    atrvjr_xOffset = 0;
    atrvjr_yOffset = 0;
    atrvjr_tOffset = 0;

    rtn = initializeMobility();
    refreshScreen();
    if ( rtn )
    {
        if ( report_level > HS_REP_LEV_NONE )
        {
            printTextWindow( "failed", true );
        }
        return;
    }
 
/* Ernest start */
    this->watchDogEnable = watchDogEnable; // initialize local watchdog flag
    if (this->watchDogEnable == true)
    {
	pthread_mutex_init( &watchdog_mutex, NULL);
	gettimeofday( &tv_watchdog,NULL);
	// tell watchdog to watch atrvjr
	WatchDog->AddATRVJr( &tv_watchdog, &watchdog_mutex);
    }
/* Ernest end */

    // *** initialize xyt needed
    pthread_create( &reader, NULL, (void*(*)(void*)) &start_thread, (void*)this );

    if ( report_level > HS_REP_LEV_NONE )
    {
        printTextWindow( "connected", true );
    }
    statusbarUpdate();
    addSensors();
}

ATRVJr::~ATRVJr()
{
    pthread_cancel( reader );
    pthread_join( reader, NULL );
    delete sensorSonar;
    printTextWindow( "ATRV-Jr disconnected" );
}

void ATRVJr::statusbarUpdate()
{
    static int c = 0;
    char buf[ 100 ];
  
    statusStr[ 0 ] = 0;

    strcat( statusStr, "ATRV-Jr: " );
    switch( c )
    {
    case 0:
        strcat( statusStr, "-" );
        break;
    case 1:
        strcat( statusStr, "\\" );
        break;
    case 2:
        strcat( statusStr, "|" );
        break;
    case 3:
        strcat( statusStr, "/" );
        break;
    }
    c = ( c + 1 ) % 4;
    float fX, fY, fT;
    getXYT( fX, fY, fT );
    sprintf( buf, " %.2fV  x: %.2f  y: %.2f  t: %.2f", atrvjr_battery[ 0 ], fX, fY, fT );
    strcat( statusStr, buf );
    statusbar->update( statusLine );

/* Ernest Start */
    if (this->watchDogEnable == true)
    {
	// update timestamp to let watchdog know we're still alive
	pthread_cleanup_push((void(*)(void*)) pthread_mutex_unlock, (void *) &watchdog_mutex);
	pthread_mutex_lock(&watchdog_mutex);
	gettimeofday( &tv_watchdog,NULL);
	pthread_cleanup_pop(1);
    }
/* Ernest End */
}

void ATRVJr::adjustXYT( float x, float y, float t )
{
    atrvjr_xOffset += x;
    atrvjr_yOffset += y;
    atrvjr_tOffset += t;
}

int ATRVJr::initializeMobility()
{
    int argc;
    char* argv[ 4 ];
    char* robotName = "ATRVJr";
    char pathName[ 255 ];

    argv[ 0 ] = "hserver";
    argv[ 1 ] = "-robot";
    argv[ 2 ] = "ATRVJr";
    argv[ 3 ] = 0;
    argc = 3;

    pHelper = new mbyClientHelper( argc, argv );

    // Build a pathname to the component we want to use to get sensor data.
    sprintf( pathName, "%s/Sonar/Segment", robotName ); // Use robot name arg.

    // Locate the component we want.
    ptempObj = pHelper->find_object( pathName );

    // Request the interface we want from the object we found
    pSonarSeg = MobilityGeometry::SegmentState::_narrow( ptempObj ); //,env);
    
    // Build pathname to the component we want to use to drive the robot.
    sprintf( pathName, "%s/Drive/Command", robotName ); // Use robot name arg.

    // Locate object within robot.
    ptempObj = pHelper->find_object( pathName );

    // We'll send two axes of command. Axis[ 0 ] == translate, Axis[ 1 ] == rotate.
    dCommand.velocity.length( 2 );
    dCommand.velocity[ 0 ] = 0.0; 
    dCommand.velocity[ 1 ] = 0.0;

    // Request the interface we need from the object we found.
    pDriveCommand = MobilityActuator::ActuatorState::_duplicate(
        MobilityActuator::ActuatorState::_narrow( ptempObj ) );

    // Build pathname to the component we want to use to drive the robot.
    sprintf( pathName, "%s/Drive/State", robotName ); // Use robot name arg.

    // Locate object within robot.
    ptempObj = pHelper->find_object( pathName );

    // Request the interface we need from the object we found.
    pLocation = MobilityActuator::ActuatorState::_duplicate(
        MobilityActuator::ActuatorState::_narrow( ptempObj ) );

    // Build pathname to the component we want to use to drive the robot.
    sprintf( pathName, "%s/Power", robotName ); // Use robot name arg.
  
    // Locate object within robot.
    ptempObj = pHelper->find_object( pathName );
  
    // Request the interface we need from the object we found.
    pPower = MobilityData::PowerManagementState::_duplicate(
        MobilityData::PowerManagementState::_narrow( ptempObj ) );

    return 0;
}  

void ATRVJr::sendMovement()
{
    dCommand.velocity[ 0 ] = goalvelocity;
    dCommand.velocity[ 1 ] = goalavelocity;
    pDriveCommand->new_sample( dCommand, 0 );
}

void ATRVJr::drive( float vel )
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        sprintf( rstr, "ATRV-Jr Drive: %f", vel );
        printTextWindow( rstr );
    }
    goalvelocity = vel;
    sendMovement();
}

void ATRVJr::initXYT()
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printTextWindow( "ATRV-Jr InitXY" );
    }
}

void ATRVJr::stop()
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        sprintf( rstr, "ATRV-Jr Stop" );
        printTextWindow( rstr );
    }
    goalvelocity = goalavelocity = 0;
    dCommand.velocity[ 0 ] = 0.0;
    dCommand.velocity[ 1 ] = 0.0;
    pDriveCommand->new_sample( dCommand, 0 );
} 

void ATRVJr::steer( float avel )
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        sprintf( rstr, "ATRV-Jr Steer: %f", avel );
        printTextWindow( rstr );
    }
  
    avel *= M_PI / 180.0; /* deg/sec to rad/sec */

    goalavelocity = avel;
    sendMovement();
}

void ATRVJr::steerToward( int angle )
{
    float avel; // deg/sec
    int absang;
    if ( report_level >= HS_REP_LEV_DEBUG )
    { 
        sprintf( rstr, "ATRV-Jr steer towards: %d", angle );
        printTextWindow( rstr );
    }
    absang = abs( angle );

    // Works on carpet with fishtailing
    if ( absang > 90 )
    {
        avel = 60;
    }
    else if ( absang > 60 )
    {
        avel = 50;
    }
    else if ( absang > 30 )
    {
        avel = 40;
    }
    else if ( absang > 20 )
    {
        avel = 27;
    }
    else if ( absang > 10 )
    {
        avel = 22;
    }
    else if ( absang > 5 )
    {
        avel = 17;    
    }
    else
    {
        avel = 12;
    }

    if ( angle < 0 )
    {
        avel*=-1;
    }
    if ( fabs( angle ) < 4 )
    {
        avel = 0;
    }
    steer( int(avel) );
}	


void ATRVJr::setXYT( float x, float y,float t )
{
    char buf[ 100 ];
    float xu, yu, tu;
    pLocationData = pLocation->get_sample( 0 );

    xu = pLocationData->position[ 0 ];
    yu = pLocationData->position[ 1 ];
    tu = pLocationData->position[ 2 ];

    tu = tu * 180.0 / M_PI;

    if ( tu < 0 )
    {
        tu += 360;
    }

    atrvjr_xOffset = x - xu;
    atrvjr_yOffset = y - yu;
    atrvjr_tOffset = t - tu;

    updateXYTFromInternalXYT();

    printfTextWindow( "setxyt %f %f %f", atrvjr_xOffset, atrvjr_yOffset, atrvjr_tOffset );

    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printfTextWindow( "ATRV-Jr Set XYT X:%f Y:%f T:%f", x, y, t );
    }
}


// This function retrieves the sonar information from mobility.
void ATRVJr::readSonarRanges()
{
    unsigned int i;

    pSegData = pSonarSeg->get_sample( 0 );
  
    for ( i = 0; i < pSegData->org.length(); i++ )
    {
        // Compute segment lengths.
        atrvjr_sonar[ i ] = sqrt(
            ( pSegData->org[ i ].x - pSegData->end[ i ].x )*
            ( pSegData->org[ i ].x - pSegData->end[ i ].x )+
            ( pSegData->org[ i ].y - pSegData->end[ i ].y )*
            ( pSegData->org[ i ].y - pSegData->end[ i ].y ) );

        // If the distance being read was bigger than sonar's detectable range, mark it as
        // no reading. Otherwise, adjust the readings by considering the width of the robot.
        if ( atrvjr_sonar[ i ] > ATRVJR_SONAR_MAX_RANGE )
        {
            atrvjr_sonar[ i ] = SONAR_READING_OUTSIDE_RANGE;
        }
        else
        {
            atrvjr_sonar[ i ] += atrvjr_range_loc[ i ];
        }
    }
}

// This function uodates the battery information.
void ATRVJr::readPowerData(void)
{
    pPowerData = pPower->get_sample(0);
    atrvjr_battery[ 0 ] = pPowerData->SourceVoltage[ 0 ];
    battery_level = atrvjr_battery[ 0 ] * 10;
}

void ATRVJr::updateXYTFromInternalXYT()
{
    atrvjr_xyt[ 0 ] = atrvjr_internal_xyt[ 0 ] + atrvjr_xOffset;
    atrvjr_xyt[ 1 ] = atrvjr_internal_xyt[ 1 ] + atrvjr_yOffset;
    atrvjr_xyt[ 2 ] = atrvjr_internal_xyt[ 2 ] * 180 / M_PI;
    while ( atrvjr_xyt[ 2 ] < 0 )
    {
        atrvjr_xyt[ 2 ] += 360;
    }
    while ( atrvjr_xyt[ 2 ] >= 360 )
    {
        atrvjr_xyt[ 2 ] -= 360;
    }
    Robot::setXYT( atrvjr_xyt[ 0 ], atrvjr_xyt[ 1 ], atrvjr_xyt[ 2 ] );
}

// This function updates the current postion of the robot.
void ATRVJr::readDeadReckoningData(void)
{
    pLocationData = pLocation->get_sample( 0 );
    atrvjr_internal_xyt[ 0 ] = pLocationData->position[ 0 ];
    atrvjr_internal_xyt[ 1 ] = pLocationData->position[ 1 ];
    atrvjr_internal_xyt[ 2 ] = pLocationData->position[ 2 ];

    updateXYTFromInternalXYT();
}

/* Thread to update ATRV-Jr information */
void ATRVJr::reader_thread(void)
{
    while( 1 )
    {

        pthread_testcancel();
        usleep( ATRVJR_READER_THREAD_UPDATE );
        readPowerData();
        readDeadReckoningData();
        readSonarRanges();
        drive( goalvelocity );	 /* watchdog */
        statusbarUpdate();
    }
}

void ATRVJr::addSensors(){
    int i;
    
    partXyt = new RobotPartXyt();

    partSonar = new RobotPartSonar( ATRVJR_NUM_SONAR, atrvjr_range_angle );
    sensorSonar = new Sensor( SENSOR_SONAR, 0, NULL, ATRVJR_NUM_SONAR, 
                              atrvjr_range_angle, ATRVJR_NUM_SONAR, atrvjr_sonar );
    for( i = 0; i < ATRVJR_NUM_SONAR; i++ )
    {
        atrvjr_sonar[ i ] = 0.0;
    }
}

void ATRVJr::control()
{
    int c;
    int done = 0;

    messageDrawWindow( EnMessageType_ATRVJR_CONTROL, EnMessageErrType_NONE );
    redrawWindows();
    do
    {
        c = getch();
        switch( c )
        {
        case 'd':
            messageHide();
            delete this;
            return;
            break;
        case 'i':
            setXYT( 5, 5, 0 );
            break;
        case 'x':
        case 'Q':
        case KEY_ESC:
            done = true;
            break;
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case KEY_UP:
        case KEY_DOWN:
        case KEY_LEFT:
        case KEY_RIGHT:
            messageMovePanelKey( c );
            break;
        }
    } while ( !done );
    messageHide();
}

/**********************************************************************
# $Log: atrvjr.c,v $
# Revision 1.6  2003/04/02 22:20:19  zkira
# Battery level for telop GUI
#
# Revision 1.5  2002/01/16 21:40:57  ernest
# Added WatchDog function -Ernest
#
# Revision 1.4  2001/05/29 22:34:10  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.3  2001/03/08 23:32:51  blee
# Fixed a bug so that GPS works and fixed a compiler error.
#
# Revision 1.2  2000/12/12 23:25:17  blee
# Changed #defines for the user interface to enumerated types.
#
# Revision 1.1  2000/11/13 20:09:23  endo
# Initial revision
#
#
#**********************************************************************/
