/**********************************************************************
 **                                                                  **
 **                              atrvjr.h                            **
 **                                                                  **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  RWI ATRV-Jr robot control for HServer                           **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: atrvjr.h,v 1.3 2002/01/16 21:44:27 ernest Exp $ */

#ifndef ATRVJR_H_
#define ATRVJR_H_

#include "mobilitycomponents_i.h"
#include "mobilitydata_i.h"
#include "mobilitygeometry_i.h"
#include "mobilityactuator_i.h"
#include "mobilityutil.h"
#include <pthread.h>
#include <unistd.h>
#include <math.h>
#include "hserver.h"
#include "message.h"
#include "robot_config.h"
#include "sensors.h"
#include "statusbar.h"
#include "robot.h"
#include "sensors.h"

// Time to update the reader thread (in microsecond).
#define ATRVJR_READER_THREAD_UPDATE 100000

// ATRV-JR has 17 sonar sensors.
#define ATRVJR_NUM_SONAR 17

// Angles of the sonar sensors. 0 is the forward direction. Counter clockwise.
#define ATRVJR_RANGE_ANGLE {180,90,90,75,60,45,30,15,0,-15,-30,-45,-60,-75,-90,-90,-180}

// Distances of the sonar sensors from the center of the robot. In meters.
#define ATRVJR_RANGE_LOC {0.32,0.20,0.20,0.25,0.28,0.32,0.32,0.32,0.32,0.32,0.32,0.32,0.28,0.25,0.20,0.20,0.32}

// Usually, the ATRV-Jr sonars can detect about 4.0 meters
#define ATRVJR_SONAR_MAX_RANGE 3.75

class ATRVJr : public Robot
{
private:
    Sensor* sensorSonar;
    RobotPartXyt* partXyt;
    RobotPartSonar* partSonar;
    int initializeMobility();
    void readSonarRanges();
    void readPowerData();
    void readDeadReckoningData();
    void sendMovement();
    void addSensors();
    void statusbarUpdate();
    void updateXYTFromInternalXYT();
    pthread_t reader;
    float goalvelocity;
    float goalavelocity;
    struct timeval tv_watchdog;  // Ernest
    pthread_mutex_t watchdog_mutex; // Ernest
    bool watchDogEnable; // Ernest

public:
    MobilityGeometry::SegmentState_var       pSonarSeg;
    MobilityGeometry::SegmentData_var        pSegData;
    mbyClientHelper*                         pHelper;
    CORBA::Object_ptr                        ptempObj;
    MobilityActuator::ActuatorState_var      pDriveCommand;
    MobilityActuator::ActuatorData           dCommand;
    MobilityActuator::ActuatorState_var      pLocation;
    MobilityActuator::ActuatorData*          pLocationData;
    MobilityData::PowerManagementState_var   pPower;
    MobilityData::PowerManagementStatus*     pPowerData;

    float atrvjr_xOffset;
    float atrvjr_yOffset;
    float atrvjr_tOffset;
    float atrvjr_internal_xyt[ 3 ];
    float atrvjr_xyt[ 3 ];
    float atrvjr_sonar[ ATRVJR_NUM_SONAR ];
    float atrvjr_range_angle[ ATRVJR_NUM_SONAR ];
    float atrvjr_range_loc[ ATRVJR_NUM_SONAR ];
    float atrvjr_battery[ 1 ];

    void reader_thread();
    void drive( float vel );
    void stop();
    void steer( float avel );
    void setXYT( float x, float y, float t );
    void initXYT();
    void adjustXYT( float x, float y, float t );
    void control();
    void steerToward( int angle );
    ATRVJr( Robot** a ,bool watchDogEnable); //Ernest
    ~ATRVJr();
};


#endif

/**********************************************************************
# $Log: atrvjr.h,v $
# Revision 1.3  2002/01/16 21:44:27  ernest
# Added variables to support WatchDog function
#
# Revision 1.2  2001/05/29 22:34:20  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.1  2000/11/13 20:09:23  endo
# Initial revision
#
#
#**********************************************************************/
