/**********************************************************************
 **                                                                  **
 **                           gyro_dmu-vgx.c                         **
 **                                                                  **
 **                                                                  **
 **  Written by:  Alexander Stoytchev                                **
 **                                                                  **
 **  KVH C100 Compass driver for HServer                             **
 **                                                                  **
 **  Copyright 2001, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
/* $Id: compass_kvh-c100.h,v 1.1 2001/05/18 17:55:10 blee Exp $ */

#ifndef COMPASS_KVH_C100_H_
#define COMPASS_KVH_C100_H_

#include "mobilitycomponents_i.h"
#include "mobilitydata_i.h"
#include "mobilitygeometry_i.h"
#include "mobilityactuator_i.h"
#include "mobilityutil.h"
#include "MobilityData.hh"
#include <pthread.h>
#include <unistd.h>
#include <math.h>
#include <sys/time.h>
#include "hserver.h"
#include "message.h"
#include "robot_config.h"
#include "sensors.h"
#include "statusbar.h"
#include "robot.h"
#include "sensors.h"



// this should match FVectorData in ./docs/mobility_idl/MobilityData.idl
typedef struct {
  struct timeval timeStamp;
  float heading;
} CompassKVHC100_CompassData_t;

class CompassKVHC100 : public Module {

protected:
    bool firstDataReceived_;
    mbyClientHelper *pHelper_;
    CORBA::Object_ptr ptempObj_;

    //MobilityComponents::FVectorState pMbyCompassState_;
    //MobilityComponents::FVectorData  *pMbyCompassData_;
    MobilityData::FVectorState *pMbyCompassState_;
    MobilityData::FVectorData  *pMbyCompassData_;

    pthread_t readerThread_;

    CompassKVHC100_CompassData_t rawCompassData_;

    float compassKVHC100HeadingCalibFactor_; 
    float compassKVHC100HeadingOffset_; 

    float compassKVHC100_heading_;

    int initializeMobility_(void);
    void statusbarUpdate_(void);
    void updateCompassData_(void);
    CompassKVHC100_CompassData_t readMbyCompassData_(void);

    void readerLoop(void); 

    static void* CompassKVHC100_startMainThread( void* compass_kvh_c100_instance );
    
    float correctHeading_(float rawHeading);

public:

    void getHeading( float& heading );
    CompassKVHC100(CompassKVHC100** a);
    ~CompassKVHC100();
};


#endif

/**********************************************************************
# $Log: compass_kvh-c100.h,v $
# Revision 1.1  2001/05/18 17:55:10  blee
# Initial revision
#
#
#**********************************************************************/
