/**********************************************************************
 **                                                                  **
 **                           gyro_dmu-vgx.c                         **
 **                                                                  **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Corssbow DMU-VGX Gyroscope driver for HServer                   **
 **                                                                  **
 **  Copyright 2001, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
/* $Id: gyro_dmu-vgx.h,v 1.1 2001/05/18 17:54:52 blee Exp $ */

#ifndef GYRO_DMU_VGX_H_
#define GYRO_DMU_VGX_H_

#include "mobilitycomponents_i.h"
#include "mobilitydata_i.h"
#include "mobilitygeometry_i.h"
#include "mobilityactuator_i.h"
#include "mobilityutil.h"
#include <pthread.h>
#include <unistd.h>
#include <math.h>
#include <sys/time.h>
#include "hserver.h"
#include "message.h"
#include "robot_config.h"
#include "sensors.h"
#include "statusbar.h"
#include "robot.h"
#include "sensors.h"

typedef struct {
    double yaw;
    double pitch;
    double roll;
} GyroDMUVGX_YPR_t;

typedef struct {
    double x;
    double y;
    double z;
} GyroDMUVGX_XYZ_t;

typedef struct {
    GyroDMUVGX_YPR_t angPos, angVel;
    GyroDMUVGX_XYZ_t accel;
    struct timeval timeStamp;
} GyroDMUVGX_GyroData_t;

class GyroDMUVGX : public Module {

protected:
    bool firstDataReceived_;
    mbyClientHelper *pHelper_;
    CORBA::Object_ptr ptempObj_;
    MobilityActuator::ActuatorState_var pMbyGyroState_;
    MobilityActuator::ActuatorData *pMbyGyroData_;
    pthread_t readerThread_;
    GyroDMUVGX_GyroData_t rawGyroData_;
    GyroDMUVGX_YPR_t gyroDMUVGXAngPos_;
    GyroDMUVGX_YPR_t gyroDMUVGXAngPosCalibFactor_;
    GyroDMUVGX_YPR_t gyroDMUVGXAngPosOffset_;
    float gyroDMUVGX_xyt_[3];
    int initializeMobility_(void);
    void statusbarUpdate_(void);
    void updateGyroData_(void);
    GyroDMUVGX_GyroData_t readMbyGyroData_(void);
    GyroDMUVGX_YPR_t correctAngularPosition_(GyroDMUVGX_YPR_t rawAngularPosition);
    void setAngleYaw_(double yaw);
    void setAnglePitch_(double pitch);
    void setAngleRoll_(double roll);
    void readerLoop(void); // jbl

    static void* GyroDMUVGX_startMainThread( void* gyro_dmu_vgx_instance );

public:
// jbl    void readerLoop(void);
// jbl    void control(void);
    void resetAngles(void);
    void setXYT(float x, float y, float t);
    void getXYT(float *x, float *y, float *t);
    GyroDMUVGX(GyroDMUVGX** a);
    ~GyroDMUVGX();
};

// jbl extern GyroDMUVGX *gGyroDMUVGX;

// jbl void gyroDMUVGXConnect(void);

#endif

/**********************************************************************
# $Log: gyro_dmu-vgx.h,v $
# Revision 1.1  2001/05/18 17:54:52  blee
# Initial revision
#
#
#**********************************************************************/
