/**********************************************************************
 **                                                                  **
 **                             hserver.h                            **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  Hardware Server                                                 **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: hserver.h,v 1.20 2003/04/06 13:41:02 endo Exp $ */

#ifndef HSERVER_H_
#define HSERVER_H_

#include <curses.h>
#include <pthread.h>
#include "WatchDog.h"

#define HSERVER_VERSION "6.0.00"

#define SONAR_READING_OUTSIDE_RANGE 0

typedef unsigned char byte_t;
typedef unsigned short ushort;


// simple commands
#define HCLIENT_GET_ROBOT_SETUP_CMD           0x01
#define HCLIENT_RANGE_STOP_CMD                0x02  
#define HCLIENT_RANGE_START_CMD               0x03  
#define HCLIENT_INIT_XYT_CMD                  0x04
#define HCLIENT_CLOSE_ROBOT_CMD               0x05
#define HCLIENT_STOP_CMD                      0x06 
#define HCLIENT_GET_SENSOR_READINGS_CMD       0x07
#define HCLIENT_GET_BATTERY_READINGS_CMD      0x08

// sensors and masks
#define SENSOR_SICKLMS200_MASK      0x0001
#define SENSOR_SONAR_MASK           0x0002
#define SENSOR_IR_MASK              0x0004
#define SENSOR_XYT_MASK             0x0008
#define SENSOR_PANTILT_CAMERA_MASK  0x0010
#define SENSOR_COGNACHROME_MASK     0x0020
#define SENSOR_ARM_MASK             0x0040
#define SENSOR_VIDEO_MASK           0x0080
#define SENSOR_CAMERA_FOCUS_MASK    0x0100
#define SENSOR_COMPASS_MASK         0x0200
#define SENSOR_GPS_MASK             0x0400

#define SENSOR_SICKLMS200           0
#define SENSOR_SONAR                1
#define SENSOR_IR                   2
#define SENSOR_XYT                  3  
#define SENSOR_PANTILT_CAMERA       4 
#define SENSOR_COGNACHROME          5 
#define SENSOR_ARM                  6 
#define SENSOR_VIDEO                7   
#define SENSOR_CAMERA_FOCUS         8
#define SENSOR_COMPASS              9
#define SENSOR_GPS                  10

#define SENSOR_DANGER_COLOR         3
#define SENSOR_SICKLMS200_COLOR     0


// robot setup and sensors
typedef struct
{
    int type;
    int color;
    float angle;
    float reading;
} sensor_reading_t;

typedef struct
{
    float angle;
    float fov_x;
    float fov_y;
} info_pantilt_camera_t;

typedef struct
{
    float angle;
    float fov;
    float range;
    int num_readings;
} info_laser_t;

typedef struct
{
    int number;
    float fov;
    float range;
    //float* angle;
} info_sonar_t;

typedef struct
{
    int number;
    float fov;
    float range;
    //float* angle;
} info_ir_t;

typedef struct
{
    int num_channels;
    int max_per_channel;
} info_cog_t;

typedef struct
{
    int num_channels;
} info_video_t;

typedef struct
{
    float scale;
} info_xyt_t;

typedef struct
{
    int num_per_channel[ 3 ];
} extra_sensor_cog_t;

typedef struct
{
    int bufnum;
} extra_laser_t;

typedef struct
{
    int type;
    int len_info;
    char* info;
} robot_part_t;

typedef struct
{
    char* name;
    int num_parts;
    robot_part_t* part;
} robot_setup_t;

typedef struct
{
    int type;
    int len_extra;
    char* extra;
    int num_angles;
    float* angle;
    int num_readings;
    float* reading;
} sensor_t;

// window sizes and locations
#define HS_HELP_COL                  30
#define HS_HELP_ROW                  25
#define HS_HELP_X                    3
#define HS_HELP_Y                    2 
#define HS_CONSOLE_PACK_NUMBER_COL   34
#define HS_CONSOLE_PING_RATE         100   
#define HS_STATUS_X                  1
#define HS_STATUS_Y                  3

// report levels
#define HS_REP_LEV_NONE     0
#define HS_REP_LEV_ERR      1
#define HS_REP_LEV_NORMAL   2
#define HS_REP_LEV_DEBUG    3

#define HS_BUF_SIZE 1024

#define KEY_ESC 27

extern WINDOW* termBordWindow;
extern WINDOW* termWindow;
extern pthread_mutex_t screenMutex;

extern int mainPanelX, mainPanelY;
extern int screenX, screenY;

extern pthread_mutex_t screenMutex;

extern int statusbarDevideLine;

void printfTextWindow( const char* szFormat, ... );
void printTextWindow( char *str, int nl = true, int filter = false, int clear = false );
void printTerminalWindow( char *str, int x, int y );
int setupSerial( int* fd, const char* port, long baud, int report_errors, char* caller, 
                 int timeout, int nonblock = false );
void refreshScreen();
void drawTermWindow();
void redrawWindows();
void resizeWindows( bool erase = true );
void flushSerialLine(int fd);

void HserverSetXyt( float fX, float fY, float fTheta );



#endif

/**********************************************************************
# $Log: hserver.h,v $
# Revision 1.20  2003/04/06 13:41:02  endo
# 6.0.00
#
# Revision 1.19  2003/04/06 10:45:37  endo
# 5.0.09
#
# Revision 1.18  2003/04/03 23:07:43  endo
# 5.0.08
#
# Revision 1.17  2003/04/02 23:04:58  zkira
# Added HCLIENT_GET_BATTERY_READINGS
#
# Revision 1.16  2002/11/03 20:44:39  endo
# 5.0.07
#
# Revision 1.15  2002/09/26 14:37:46  endo
# 5.0.06
#
# Revision 1.14  2002/08/22 14:43:52  endo
# 5.0.05
#
# Revision 1.13  2002/07/18 17:18:18  endo
# 5.0.04
#
# Revision 1.12  2002/07/04 23:31:27  endo
# v5.0.03
#
# Revision 1.11  2002/04/01 22:03:23  endo
# 5.0.02
#
# Revision 1.10  2002/01/31 10:55:49  endo
# Version 5.0.01
#
# Revision 1.9  2002/01/16 21:48:10  ernest
# Added WatchDog functionality
#
# Revision 1.8  2001/05/29 22:36:35  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.7  2001/03/23 21:17:52  blee
# changed setupSerial prototype; made the char* a const char*
#
# Revision 1.6  2000/12/12 22:56:19  blee
# Added prototype for printfTextWindow().
#
# Revision 1.5  2000/11/13 20:09:23  endo
# ARTV-Jr class added to HServer.
#
# Revision 1.4  2000/10/16 20:03:53  endo
# steerToward() bug fixed. READING_OUTSIDE_RANGE added for the
# sonar reading.
#
# Revision 1.3  2000/09/19 03:14:21  endo
# RCS log added.
#
#
#**********************************************************************/
