/**********************************************************************
 **                                                                  **
 **                         HServerConfig.c                          **
 **                                                                  **
 **                                                                  **
 **  Written by:  Brian Lee                                          **
 **                                                                  **
 **  Copyright 2002, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: HServerConfig.c,v 1.2 2002/02/18 13:52:47 endo Exp $ */

#include "HServerConfig.h"

const int ClHServerConfig::m_aiSECTION_VALUE_COUNTS[] = 
{
    EnCameraValueType_COUNT,
    EnLaserValueType_COUNT,
    EnCognachromeValueType_COUNT,
    EnNomadValueType_COUNT,
    EnPioneerValueType_COUNT,
    EnAmigoBotValueType_COUNT,
    EnFrameGrabberValueType_COUNT,
    EnGpsValueType_COUNT
};

ClHServerConfig::ClHServerConfig()
{
}

ClHServerConfig::~ClHServerConfig()
{
}

bool ClHServerConfig::AddNewSection( int iSectionType, string strName )
{
    bool bSuccess = false;

    if ( ( iSectionType >= 0 ) && 
         ( iSectionType < EnHServerConfigSectionType_COUNT ) )
    {
        ClHServerConfigSection clNewSection( iSectionType, m_aiSECTION_VALUE_COUNTS[ iSectionType ], strName );
        m_clSectionList.push_back( clNewSection );
        bSuccess = true;
    }

    return bSuccess;
}

bool ClHServerConfig::InsertValue( int iSection, int iValueType, const string& strValue )
{
    bool bSuccess = false;

    if ( ValidSection( iSection ) )
    {
        bSuccess = m_clSectionList[ iSection ].InsertValue( iValueType, strValue );
    }

    return bSuccess;
}

bool ClHServerConfig::GetValue( int iSection, int iValueType, string& strValue )
{
    bool bSuccess = false;

    if ( ValidSection( iSection ) )
    {
        bSuccess = m_clSectionList[ iSection ].GetValue( iValueType, strValue );
    }

    return bSuccess;
}

bool ClHServerConfig::GetSectionName( int iSection, string& strName ) const
{
    bool bSuccess = ValidSection( iSection );

    if ( bSuccess )
    {
        strName = m_clSectionList[ iSection ].GetName();
    }

    return bSuccess;
}

bool ClHServerConfig::SetSectionName( int iSection, string strName )
{
    bool bSuccess = false;

    if ( ValidSection( iSection ) )
    {
        m_clSectionList[ iSection ].SetName( strName );
        bSuccess = true;
    }

    return bSuccess;
}

/**********************************************************************
# $Log: HServerConfig.c,v $
# Revision 1.2  2002/02/18 13:52:47  endo
# AmigoBot added.
#
#
#**********************************************************************/
