/**********************************************************************
 **                                                                  **
 **                         HServerConfig.h                          **
 **                                                                  **
 **                                                                  **
 **  Written by:  Brian Lee                                          **
 **                                                                  **
 **  Copyright 2002, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: HServerConfig.h,v 1.3 2003/04/06 13:35:53 endo Exp $ */

#ifndef _HSERVERCONFIG_H
#define _HSERVERCONFIG_H

#include <string>
#include <vector>
#include "HServerConfigSection.h"

using std::string;
using std::vector;

class ClHServerConfig
{
public:
    enum EnCameraValueType
    {
        EnCameraValueType_PORT_STRING = 0,
        EnCameraValueType_COUNT     // this must be declared last
    };

    enum EnLaserValueType
    {
        EnLaserValueType_NAME = 0,
        EnLaserValueType_PORT_STRING,
        EnLaserValueType_ANGLE_OFFSET,
        EnLaserValueType_X_OFFSET,
        EnLaserValueType_Y_OFFSET,
        EnLaserValueType_LISTEN_IPT,
        EnLaserValueType_SEND_IPT,
        EnLaserValueType_STREAM_HOST,
        EnLaserValueType_COUNT     // this must be declared last
    };

    enum EnCognachromeValueType
    {
        EnCognachromeValueType_PORT_STRING = 0,
        EnCognachromeValueType_COUNT     // this must be declared last
    };

    enum EnNomadValueType
    {
        EnNomadValueType_TYPE = 0,
        EnNomadValueType_PORT_STRING,
        EnNomadValueType_HOST_NAME,
        EnNomadValueType_COUNT     // this must be declared last
    };

    enum EnPioneerValueType
    {
        EnPioneerValueType_PORT_STRING = 0,
        EnPioneerValueType_COUNT     // this must be declared last
    };

    enum EnAmigoBotValueType
    {
        EnAmigoBotValueType_PORT_STRING = 0,
        EnAmigoBotValueType_COUNT     // this must be declared last
    };

    enum EnFrameGrabberValueType
    {
        EnFrameGrabberValueType_PORT_STRING = 0,
        EnFrameGrabberValueType_COUNT     // this must be declared last
    };

    enum EnGpsValueType
    {
        EnGpsValueType_USE_BASE = 0,
        EnGpsValueType_PORT_STRING,
        EnGpsValueType_BASE_LAT,
        EnGpsValueType_BASE_LONG,
        EnGpsValueType_X_DIFF,
        EnGpsValueType_Y_DIFF,
        EnGpsValueType_M_PER_LAT,
        EnGpsValueType_M_PER_LONG,
        EnGpsValueType_COUNT     // this must be declared last
    };

    enum EnHServerConfigSectionType
    {
        EnHServerConfigSectionType_CAMERA = 0,
        EnHServerConfigSectionType_LASER,
        EnHServerConfigSectionType_COGNACHROME,
        EnHServerConfigSectionType_NOMAD,
        EnHServerConfigSectionType_PIONEER,
        EnHServerConfigSectionType_AMIGOBOT,
        EnHServerConfigSectionType_FRAMEGRABBER,
        EnHServerConfigSectionType_GPS,
        EnHServerConfigSectionType_COUNT     // this must be declared last
    };

    enum EnHServerConfigError
    {
        EnHServerConfigError_NONE,
        EnHServerConfigError_BAD_CONFIG_TYPE
    };

    static const int m_aiSECTION_VALUE_COUNTS[];

    ClHServerConfig();
    ~ClHServerConfig();

    bool AddNewSection( int iSectionType, string strName );
    unsigned int GetSectionCount() const;
    int GetSectionType( int iSection ) const;
    bool GetValue( int iSection, int iValueType, string& strValue );
    bool InsertValue( int iSection, int iValueType, const string& strValue );
    bool GetSectionName( int iSection, string& strName ) const;
    bool SetSectionName( int iSection, string strName );

protected:
    vector<ClHServerConfigSection> m_clSectionList;

    bool ValidSection( int iSection ) const;
};

inline unsigned int ClHServerConfig::GetSectionCount() const
{
    return m_clSectionList.size();
}

inline bool ClHServerConfig::ValidSection( int iSection ) const
{
    return ( ( iSection >= 0 ) && ( iSection < (int) m_clSectionList.size() ) );
}

inline int ClHServerConfig::GetSectionType( int iSection ) const
{
    return ( ValidSection( iSection ) ? m_clSectionList[ iSection ].GetType() : -1 );
}

#endif // #ifndef _HSERVERCONFIG_H

/**********************************************************************
# $Log: HServerConfig.h,v $
# Revision 1.3  2003/04/06 13:35:53  endo
# gcc 3.1.1
#
# Revision 1.2  2002/02/18 13:52:47  endo
# AmigoBot added.
#
#
#**********************************************************************/
