/**********************************************************************
 **                                                                  **
 **                       HServerConfigurator.c                      **
 **                                                                  **
 **                                                                  **
 **  Written by:  Brian Lee                                          **
 **                                                                  **
 **  Copyright 2002, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: HServerConfigurator.c,v 1.2 2002/02/18 13:52:47 endo Exp $ */

#include <math.h> // for HUGE_VAL
#include "HServerConfigurator.h"
#include "gps.h"

ClHServerConfigurator::ClHServerConfigurator( ClHServerConfig& clConfig )
{
    m_pclConfig = &clConfig;
}

ClHServerConfigurator::~ClHServerConfigurator()
{
}

SuCognachromeInitData ClHServerConfigurator::GetCognachromeOptions( int iSection )
{
    SuCognachromeInitData suData;
    string strValue;

    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnCognachromeValueType_PORT_STRING,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strPortString = strValue;
    }

    return suData;
}

SuCameraInitData ClHServerConfigurator::GetCameraOptions( int iSection )
{
    SuCameraInitData suData;
    string strValue;

    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnCameraValueType_PORT_STRING,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strPortString = strValue;
    }

    return suData;
}

SuNomadInitData ClHServerConfigurator::GetNomadOptions( int iSection )
{
    SuNomadInitData suData;
    suData.enType = Nomad::EnNomadType_150;
    suData.strPortString = "";
    suData.strHost = "";

    string strValue;

    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnNomadValueType_TYPE,
                                strValue ) && 
         ( strValue != "" ) )
    {
        if ( strValue == "150" )
        {
            suData.enType = Nomad::EnNomadType_150;
        }
        else if ( strValue == "200" )
        {
            suData.enType = Nomad::EnNomadType_200;
        }
        else
        {
            string strName;
            m_pclConfig->GetSectionName( iSection, strName );
            fprintf( stderr, "hserver error: bad nomad type \"%s\" for\n", strValue.c_str() );
            fprintf( stderr, "    nomad section labeled: %s\n", strName.c_str() );
            fprintf( stderr, "    Legal values are \"150\" and \"200\".\n" );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnNomadValueType_PORT_STRING,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strPortString = strValue;
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnNomadValueType_HOST_NAME,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strHost = strValue;
    }

    return suData;
}

SuPioneerInitData ClHServerConfigurator::GetPioneerOptions( int iSection )
{
    SuPioneerInitData suData;
    suData.strPortString = "";

    string strValue;

    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnPioneerValueType_PORT_STRING,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strPortString = strValue;
    }

    return suData;
}

SuAmigoBotInitData ClHServerConfigurator::GetAmigoBotOptions( int iSection )
{
    SuAmigoBotInitData suData;
    suData.strPortString = "";

    string strValue;

    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnAmigoBotValueType_PORT_STRING,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strPortString = strValue;
    }

    return suData;
}

SuFrameGrabberInitData ClHServerConfigurator::GetFrameGrabberOptions( int iSection )
{
    SuFrameGrabberInitData suData;
    suData.strPortString = "";

    string strValue;

    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnFrameGrabberValueType_PORT_STRING,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strPortString = strValue;
    }

    return suData;
}

SuGpsInitData ClHServerConfigurator::GetGpsOptions( int iSection )
{
    SuGpsInitData suData;
    suData.strPortString = "";
    suData.bUseBase = false;
    suData.dBaseLat = DEFAULT_GPS_BASE_LAT;
    suData.dBaseLong = DEFAULT_GPS_BASE_LON;
    suData.dXDiff = DEFAULT_GPS_BASE_X;
    suData.dYDiff = DEFAULT_GPS_BASE_Y;
    suData.dMPerLat = DEFAULT_GPS_M_PER_LON;
    suData.dMPerLong = DEFAULT_GPS_M_PER_LAT;

    string strValue;
    char* pcEnd;

    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnGpsValueType_USE_BASE,
                                strValue ) && 
         ( strValue != "" ) )
    {
        if ( ( strcasecmp( strValue.c_str(), "true" ) == 0 ) || 
             ( strcasecmp( strValue.c_str(), "1" ) == 0 ) )
        {
            suData.bUseBase = true;
        }
        else if ( ( strcasecmp( strValue.c_str(), "false" ) == 0 ) || 
                  ( strcasecmp( strValue.c_str(), "0" ) == 0 ) )
        {
            suData.bUseBase = false;
        }
        else
        {
            fprintf( stderr, "hserver error: a GPS's \"use_base\" value had an invalid value.\n" );
            fprintf( stderr, "  Valid values: \"true\", \"false\", \"1\", \"0\"\n" );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnGpsValueType_PORT_STRING,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strPortString = strValue;
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnGpsValueType_BASE_LAT,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.dBaseLat = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"base_lat\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.dBaseLat == HUGE_VAL ) || ( suData.dBaseLat == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"base_lat\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnGpsValueType_BASE_LONG,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.dBaseLong = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"base_long\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.dBaseLong == HUGE_VAL ) || ( suData.dBaseLong == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"base_long\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnGpsValueType_X_DIFF,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.dXDiff = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"x_diff\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.dXDiff == HUGE_VAL ) || ( suData.dXDiff == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"x_diff\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnGpsValueType_Y_DIFF,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.dYDiff = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"y_diff\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.dYDiff == HUGE_VAL ) || ( suData.dYDiff == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"y_diff\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnGpsValueType_M_PER_LAT,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.dMPerLat = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"m_per_lat\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.dMPerLat == HUGE_VAL ) || ( suData.dMPerLat == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"m_per_lat\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnGpsValueType_M_PER_LONG,
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.dMPerLong = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"m_per_long\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.dBaseLong == HUGE_VAL ) || ( suData.dBaseLong == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"m_per_long\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
    }

    return suData;
}

SuLaserInitData ClHServerConfigurator::GetLaserOptions( int iSection )
{
    SuLaserInitData suData;
    suData.strPort = "";
    suData.bSend = false;
    suData.bListen = false;
    suData.fAngleOffset = 0.0;
    suData.fXOffset = 0.0;
    suData.fYOffset = 0.0;
    suData.strName = "";
    suData.strStreamAddress = "";

    string strValue = "";

    char* pcEnd;

    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnLaserValueType_NAME, 
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strName = strValue;
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnLaserValueType_PORT_STRING, 
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strPort = strValue;
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnLaserValueType_ANGLE_OFFSET, 
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.fAngleOffset = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a laser's \"angle_offset\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.fAngleOffset == HUGE_VAL ) || ( suData.fAngleOffset == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a laser's \"angle_offset\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnLaserValueType_X_OFFSET, 
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.fXOffset = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a laser's \"x_offset\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.fXOffset == HUGE_VAL ) || ( suData.fXOffset == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a laser's \"x_offset\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        suData.fXOffset /= 100.0; // convert cm to meters
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnLaserValueType_Y_OFFSET, 
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.fYOffset = strtod( strValue.c_str(), &pcEnd );
        if ( *pcEnd != '\0' )
        {
            fprintf( stderr, 
                     "hserver error: a laser's \"y_offset\" value, %s, was not valid.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        if ( ( suData.fYOffset == HUGE_VAL ) || ( suData.fYOffset == -HUGE_VAL ) )
        {
            fprintf( stderr, 
                     "hserver error: a GPS's \"y_offset\" value, %s, caused an overflow.\n", 
                     strValue.c_str() );
            exit( -1 );
        }
        suData.fYOffset /= 100.0; // convert cm to meters
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnLaserValueType_LISTEN_IPT, 
                                strValue ) && 
         ( strValue != "" ) )
    {
        if ( ( strcasecmp( strValue.c_str(), "true" ) == 0 ) || 
             ( strcasecmp( strValue.c_str(), "1" ) == 0 ) )
        {
            suData.bListen = true;
        }
        else if ( ( strcasecmp( strValue.c_str(), "false" ) == 0 ) || 
                  ( strcasecmp( strValue.c_str(), "0" ) == 0 ) )
        {
            suData.bListen = false;
        }
        else
        {
            fprintf( stderr, "hserver error: a laser's \"listen_ipt\" value had an invalid value.\n" );
            fprintf( stderr, "  Valid values: \"true\", \"false\", \"1\", and \"0\"\n" );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, ClHServerConfig::EnLaserValueType_SEND_IPT, strValue ) && 
         ( strValue != "" ) )
    {
        if ( ( strcasecmp( strValue.c_str(), "true" ) == 0 ) || 
             ( strcasecmp( strValue.c_str(), "1" ) == 0 ) )
        {
            suData.bSend = true;
        }
        else if ( ( strcasecmp( strValue.c_str(), "false" ) == 0 ) || 
                  ( strcasecmp( strValue.c_str(), "0" ) == 0 ) )
        {
            suData.bSend = false;
        }
        else
        {
            fprintf( stderr, "hserver error: a laser's \"send_ipt\" value had an invalid value.\n" );
            fprintf( stderr, "  Valid values: \"true\", \"false\", \"1\", and \"0\"\n" );
            exit( -1 );
        }
    }
    if ( m_pclConfig->GetValue( iSection, 
                                ClHServerConfig::EnLaserValueType_STREAM_HOST, 
                                strValue ) && 
         ( strValue != "" ) )
    {
        suData.strStreamAddress = strValue;
    }

    return suData;
}

/**********************************************************************
# $Log: HServerConfigurator.c,v $
# Revision 1.2  2002/02/18 13:52:47  endo
# AmigoBot added.
#
#
#**********************************************************************/
