/**********************************************************************
 **                                                                  **
 **                       HServerConfigurator.h                      **
 **                                                                  **
 **                                                                  **
 **  Written by:  Brian Lee                                          **
 **                                                                  **
 **  Copyright 2002, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: HServerConfigurator.h,v 1.2 2002/02/18 13:52:47 endo Exp $ */

#ifndef _HSERVERCONFIGURATOR_H
#define _HSERVERCONFIGURATOR_H

#include <string>
#include "HServerConfig.h"
#include "nomad.h"

// this holds initialization data for the lasers
struct SuLaserInitData
{
    string strPort;
    bool bSend;
    bool bListen;
    float fAngleOffset;
    float fXOffset;
    float fYOffset;
    string strName;
    string strStreamAddress;

    SuLaserInitData() {}
    SuLaserInitData( const SuLaserInitData& suOriginal ) { *this = suOriginal; }
    inline SuLaserInitData& operator=( const SuLaserInitData& suOriginal );
};

inline SuLaserInitData& SuLaserInitData::operator=( const SuLaserInitData& suOriginal )
{
    if ( this != & suOriginal )
    {
        strPort = suOriginal.strPort;
        bSend = suOriginal.bSend;
        bListen = suOriginal.bListen;
        fAngleOffset = suOriginal.fAngleOffset;
        fXOffset = suOriginal.fXOffset;
        fYOffset = suOriginal.fYOffset;
        strName = suOriginal.strName;
        strStreamAddress = suOriginal.strStreamAddress;
    }

    return *this;
}

struct SuCameraInitData
{
    string strPortString;

    SuCameraInitData() {}
    SuCameraInitData( const SuCameraInitData& suOriginal ) { *this = suOriginal; }
    inline SuCameraInitData& operator=( const SuCameraInitData& suOriginal );
};

inline SuCameraInitData& SuCameraInitData::operator=( const SuCameraInitData& suOriginal )
{
    if ( this != &suOriginal )
    {
        strPortString = suOriginal.strPortString;
    }

    return *this;
}

struct SuCognachromeInitData
{
    string strPortString;

    SuCognachromeInitData() {}
    SuCognachromeInitData( const SuCognachromeInitData& suOriginal ) { *this = suOriginal; }
    inline SuCognachromeInitData& operator=( const SuCognachromeInitData& suOriginal );
};

inline SuCognachromeInitData& SuCognachromeInitData::operator=( const SuCognachromeInitData& suOriginal )
{
    if ( this != &suOriginal )
    {
        strPortString = suOriginal.strPortString;
    }

    return *this;
}

struct SuNomadInitData
{
    Nomad::EnNomadType enType;
    string strPortString;
    string strHost;

    SuNomadInitData() {}
    SuNomadInitData( const SuNomadInitData& suOriginal ) { *this = suOriginal; }
    inline SuNomadInitData& operator=( const SuNomadInitData& suOriginal );
};

inline SuNomadInitData& SuNomadInitData::operator=( const SuNomadInitData& suOriginal )
{
    if ( this != &suOriginal )
    {
        enType = suOriginal.enType;
        strPortString = suOriginal.strPortString;
        strHost = suOriginal.strHost;
    }

    return *this;
}

struct SuPioneerInitData
{
    string strPortString;

    SuPioneerInitData() {}
    SuPioneerInitData( const SuPioneerInitData& suOriginal ) { *this = suOriginal; }
    inline SuPioneerInitData& operator=( const SuPioneerInitData& suOriginal );
};

inline SuPioneerInitData& SuPioneerInitData::operator=( const SuPioneerInitData& suOriginal )
{
    if ( this != &suOriginal )
    {
        strPortString = suOriginal.strPortString;
    }

    return *this;
}

struct SuAmigoBotInitData
{
    string strPortString;

    SuAmigoBotInitData() {}
    SuAmigoBotInitData( const SuAmigoBotInitData& suOriginal ) { *this = suOriginal; }
    inline SuAmigoBotInitData& operator=( const SuAmigoBotInitData& suOriginal );
};

inline SuAmigoBotInitData& SuAmigoBotInitData::operator=( const SuAmigoBotInitData& suOriginal )
{
    if ( this != &suOriginal )
    {
        strPortString = suOriginal.strPortString;
    }

    return *this;
}

struct SuFrameGrabberInitData
{
    string strPortString;

    SuFrameGrabberInitData() {}
    SuFrameGrabberInitData( const SuFrameGrabberInitData& suOriginal ) { *this = suOriginal; }
    inline SuFrameGrabberInitData& operator=( const SuFrameGrabberInitData& suOriginal );
};

inline SuFrameGrabberInitData& SuFrameGrabberInitData::operator=( const SuFrameGrabberInitData& suOriginal )
{
    if ( this != &suOriginal )
    {
        strPortString = suOriginal.strPortString;
    }

    return *this;
}

struct SuGpsInitData
{
    bool bUseBase;
    string strPortString;
    double dBaseLat;
    double dBaseLong;
    double dXDiff;
    double dYDiff;
    double dMPerLat;
    double dMPerLong;

    SuGpsInitData() {}
    SuGpsInitData( const SuGpsInitData& suOriginal ) { *this = suOriginal; }
    inline SuGpsInitData& operator=( const SuGpsInitData& suOriginal );
};

inline SuGpsInitData& SuGpsInitData::operator=( const SuGpsInitData& suOriginal )
{
    if ( this != &suOriginal )
    {
        bUseBase = suOriginal.bUseBase;
        strPortString = suOriginal.strPortString;
        dBaseLat = suOriginal.dBaseLat;
        dBaseLong = suOriginal.dBaseLong;
        dXDiff = suOriginal.dXDiff;
        dYDiff = suOriginal.dYDiff;
        dMPerLat = suOriginal.dMPerLat;
        dMPerLong = suOriginal.dMPerLong;
    }

    return *this;
}

class ClHServerConfigurator
{
public:
    ClHServerConfigurator( ClHServerConfig& clConfig );
    ~ClHServerConfigurator();

    SuCognachromeInitData GetCognachromeOptions( int iSection );
    SuCameraInitData GetCameraOptions( int iSection );
    SuNomadInitData GetNomadOptions( int iSection );
    SuPioneerInitData GetPioneerOptions( int iSection );
    SuAmigoBotInitData GetAmigoBotOptions( int iSection );
    SuFrameGrabberInitData GetFrameGrabberOptions( int iSection );
    SuGpsInitData GetGpsOptions( int iSection );
    SuLaserInitData GetLaserOptions( int iSection );

protected:
    ClHServerConfig* m_pclConfig;
};

#endif // _HSERVERCONFIGURATOR_H

/**********************************************************************
# $Log: HServerConfigurator.h,v $
# Revision 1.2  2002/02/18 13:52:47  endo
# AmigoBot added.
#
#
#**********************************************************************/
