/**********************************************************************
 **                                                                  **
 **                           hserver_ipt.h                          **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  HServer IPT messages                                            **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: hserver_ipt.h,v 1.4 2003/04/02 22:18:22 zkira Exp $ */

#ifndef HSERVER_IPT_H_
#define HSERVER_IPT_H_

#include "ipt/ipt.h"
#include "ipt/message.h"

#define MODULE_NOT_CONNECTED   0
#define MODULE_UNKNOWN         1
#define MODULE_ID_GPS          2
#define MODULE_ID_LASER        3

extern char *MSG_InitConnection;
#define FRM_InitConnection "string"
typedef struct REC_InitConnection {
    char *name;
} REC_InitConnection;


extern char *TYP_SensorReadingT;
#define FRM_SensorReadingT "{int,int,<char: 2>,int,<float: 4>,int,<float: 6>}"
typedef struct REC_SensorReadingT {
    int type;
    int len_extra;
    char *extra;
    int num_angles;
    float *angle;
    int num_readings;
    float *reading;
} REC_SensorReadingT;

extern char *TYP_RobotPartT;
#define FRM_RobotPartT "{int,int,<char: 2>}"
typedef struct REC_RobotPartT {
    int type;
    int len_info;
    char *info;
} REC_RobotPartT;

extern char *MSG_RobotSetupInfo;
#define FRM_RobotSetupInfo "{string,int,<RobotPartT: 2>}"
typedef struct REC_RobotSetupInfo {
    char *name;
    int num_parts;
    REC_RobotPartT *part;
} REC_RobotSetupInfo;

extern char *MSG_BatteryUpdate;    // trw zk
#define FRM_BatteryUpdate "{int}"
typedef struct REC_BatteryUpdate {
  int battery_level;
} REC_BatteryUpdate;

extern char *MSG_SensorUpdate;
#define FRM_SensorUpdate "{int,<SensorReadingT: 1>}"
typedef struct REC_SensorUpdate {
    int num_sensors;
    REC_SensorReadingT *sensor;
} REC_SensorUpdate;

extern char *MSG_SimpleCommand;
#define FRM_SimpleCommand "{int}"
typedef struct REC_SimpleCommand {
    int command;
} REC_SimpleCommand;

extern char *MSG_DriveCommand;
#define FRM_DriveCommand "{float}"
typedef struct REC_DriveCommand {
    float vel;  //m per sec
} REC_DriveCommand;

extern char *MSG_SteerCommand;
#define FRM_SteerCommand "{float}"
typedef struct REC_SteerCommand {
    float avel;  //deg per sec
} REC_SteerCommand;

extern char *MSG_SteerTowardDriveCommand;
#define FRM_SteerTowardDriveCommand "{float,float,float,int}"
typedef struct REC_SteerTowardDriveCommand {
    float theta;  //angle
    float speed;  //m per sec
    float drive_wait_angle; // deg
    int use_reverse;
} REC_SteerTowardDriveCommand;

extern char *MSG_SetXYT;
#define FRM_SetXYT "{float,float,float}"
typedef struct REC_SetXYT {
    float x;
    float y;
    float t;
} REC_SetXYT;

extern char *MSG_MoveCamera;
#define FRM_MoveCamera "{float,float}"
typedef struct REC_MoveCamera {
    float pan;
    float tilt;
} REC_MoveCamera;

extern char *MSG_SetCameraTrackerMode;
#define FRM_SetCameraTrackerMode "{int}"
typedef struct REC_SetCameraTrackerMode {
    int mode; 
} REC_SetCameraTrackerMode;

extern char *MSG_FrameUpload;
#define FRM_FrameUpload "{int,int,string}"
typedef struct REC_FrameUpload {
    int channel; 
    int send;
    char *name;
} REC_FrameUpload;

extern char  *MSG_VideoFrame;
#define FRM_VideoFrame "{int,int,int,<char: 3>}"
typedef struct REC_VideoFrame {
    int width;
    int height;
    int size;
    char *frame;
} REC_VideoFrame;

extern char *MSG_GpsRTCM;
#define FRM_GpsRTCM "{int,<char:1>}"
typedef struct REC_GpsRTCM {
    int size;
    char *data;
} REC_GpsRTCM;

extern char *MSG_ModuleId;
#define FRM_ModuleId "{int}"
typedef struct REC_ModuleId {
    int id;
} REC_ModuleId;

extern char *MSG_LaserScan;
#define FRM_LaserScan "{int,int,<int :1>}"
typedef struct REC_LaserScan {
    int size;
    int min;
    unsigned int *scan;
} REC_LaserScan;

extern IPFormatSpec iptHclientFormatArray[];

extern IPMessageSpec iptHclientMessageArray[];

extern IPMessageSpec iptGpsMessageArray[];

extern IPMessageSpec iptHandlerMessageArray[];

extern IPMessageSpec iptLaserMessageArray[];

extern IPMessageSpec iptMazeMessageArray[];

struct IPMsgCallbackHandlerSpec 
{
    char* msg_name;
    IPHandlerCallback *callback;
    int context;
};




#endif

/**********************************************************************
# $Log: hserver_ipt.h,v $
# Revision 1.4  2003/04/02 22:18:22  zkira
# Added BatteryUpdate message
#
# Revision 1.3  2000/09/19 03:14:21  endo
# RCS log added.
#
#
#**********************************************************************/
