/**********************************************************************
 **                                                                  **
 **                            ipt_handler.h                         **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  IPT handler for HServer                                         **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: ipt_handler.h,v 1.4 2002/01/16 21:51:53 ernest Exp $ */

#ifndef IPTHANDLER_H
#define IPTHANDLER_H

#include <pthread.h>
#include "module.h"
#include "hserver_ipt.h"
#include "gps.h"

#define IPT_MOD_CONSOLE 0
#define IPT_MOD_GPS     1
#define IPT_MOD_LASER   2

class IptHandler : public Module
{
protected:
    bool ipt_server_found, multiple;
    char* ipt_server_name;
    char* ipt_name;
    char* robot_name;
    IPCommunicator* control_comm;
    int total_bytes_recieved;
    bool gpsMod, laserMod, consoleMod;
    bool consoleConn, gpsConn, laserConn;
    char* hclient_name;
    char* hclient_host;
    IPHandlerCallback* callback;
    void initIpt();
    void control_connect_callback( IPConnection* conn );
    void control_disconnect_callback( IPConnection* conn );
    void statusbarUpdate();
    void information();
    void callbackModuleId( IPMessage* message );
    pthread_t reader;
    struct timeval tv_watchdog;  // Ernest
    pthread_mutex_t watchdog_mutex; // Ernest
    bool watchDogEnable; // Ernest
    Gps** gps;

public:
    IPCommunicator* communicator() { return control_comm; };
    void set_state( int mod, bool listen, bool up );
    void listen_hclient();
    void reader_thread();
    void control();
    void registerNamedFormatters( IPFormatSpec formats[] );
    void registerMessages( IPMessageSpec messages[] );
    void registerHandlers( int num, IPMsgHandlerSpec hndArray[] );
    void registerCallbackHandlers( int num, IPMsgCallbackHandlerSpec hndArray[] );
    void disableHandlers( int num, char* hndNames[] );
    IptHandler( IptHandler** a, char* name, char* robot_name, Gps** gps, 
		     bool multiple = FALSE, bool watchDogEnable = true ); // Ernest
    ~IptHandler();
};

extern IptHandler* iptHandler;

#endif

/**********************************************************************
# $Log: ipt_handler.h,v $
# Revision 1.4  2002/01/16 21:51:53  ernest
# Added WatchDog function
#
# Revision 1.3  2001/05/29 22:37:22  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.2  2000/09/19 03:14:21  endo
# RCS log added.
#
#
#**********************************************************************/
