/**********************************************************************
 **                                                                  **
 **                             pioneer.h                            **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  Pioneer robot driver for HServer                                **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: pioneer.h,v 1.4 2001/05/29 22:34:55 blee Exp $ */

#ifndef PIONEER_CONTROLLER_H_
#define PIONEER_CONTROLLER_H_


#include <pthread.h>
#include <string>
#include "robot.h"
#include "sensors.h"
#include "robot_config.h"

#define PIONEER_NUM_SONAR 8
#define PIONEER_NUM_IR 2
#define SONAR_TYPE_A 0 // ENDO
#define SONAR_TYPE_B 1 // ENDO

typedef unsigned char byte_t;

class Pioneer : public Robot
{
  private:
    Sensor* sensorSonar;
    Sensor* sensorIr;
    Sensor* sensorCompass;
    RobotPartSonar* partSonar;
    RobotPartIr* partIr;
    RobotPartXyt* partXyt;
    RobotPartCompass* partCompass;
    char *robotName, *robotClass, *subclass;
    float compass[ 1 ];
    float battery;
    float velocity,avelocity;
    int numIRs,numSonars;
    // ENDO
    int sonar_type;
    int fd;
    float sonar[ PIONEER_NUM_SONAR ];
    float sonar_loc[ PIONEER_NUM_SONAR ][ 3 ];
    float ir[ PIONEER_NUM_IR ];
    pthread_mutex_t serialMutex;
    pthread_t reader,syncer;
    bool syncing,synced,rangeOn;
    void statusbarUpdate();
    void addSensors();
    int sendPack( byte_t pack[] );
    int sendSync( int s );
    int sendString( char str[], byte_t command );
    int sendStringN( char str[], int len, byte_t command );
    int sendInt( int sint, byte_t command );
    void useSip( struct t_sip sip );
    void decodeSync2Pack( byte_t pack[] );
    void decodeStatusPack( byte_t pack[] );
    int decodePack( byte_t pack[] );
    void sync();
    int readPack();
    int testRobotConnection(); 
    void flushSerialLine();
    byte_t nextByte();
    int nextRemainingBytes( byte_t* bytes, int max );
    void getWord( char buf[] );
    int round( float n );
    void set_nibbles( int pos, byte_t buf[] );
    int calc_chksum( byte_t* ptr );
    int shortAngle( int ang1, int ang2 );
  public:
    void syncer_thread();
    void reader_thread();
    void drive( float vel );
    void stop();
    void initXYT();
    void setXYT( float x, float y, float theta ); // m, m, deg
    void sonarStart();
    void sonarStop();
    void reset();
    void control();
    void steer( float avel );
    void steerToward( int angle );
    Pioneer( Robot** a, const string& strPortString );
    ~Pioneer();
};

#endif

/**********************************************************************
# $Log: pioneer.h,v $
# Revision 1.4  2001/05/29 22:34:55  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.3  2001/03/23 21:32:26  blee
# altered to use a config file
#
# Revision 1.2  2000/09/19 03:47:30  endo
# Sonar bug fixed.
#
#
#**********************************************************************/
