/**********************************************************************
 **                                                                  **
 **                              robot.c                             **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  Generic robot class for HServer                                 **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: robot.c,v 1.9 2003/04/02 22:25:20 zkira Exp $ */

#include <math.h>
#include <string>

#include "message.h"
#include "robot.h"
#include "hserver.h"

Robot::Robot( Robot** a, RobotType type, char* name ) : 
    Module( (Module**) a, name )
{
    speed_factor = 10.0;
    angular_speed_factor = 10.0;
    this->type = type;
    m_bposition_set = false; // Ernest
    battery_level = -1;
}

Robot::Robot()
{
    self = NULL;
    speed_factor = 10.0;
    angular_speed_factor = 10.0;
    m_bposition_set = false; // Ernest
    battery_level = -1;
}

Robot::~Robot()
{
}

void Robot::getXYT( float& fX, float& fY, float& fTheta )
{
    pthread_mutex_lock( &m_mutexXytLock );
    fX     = xyt[ 0 ];
    fY     = xyt[ 1 ];
    fTheta = xyt[ 2 ];
    pthread_mutex_unlock( &m_mutexXytLock );
}

void Robot::setXYT( float fX, float fY, float fTheta )
{
    pthread_mutex_lock( &m_mutexXytLock );
    xyt[ 0 ] = fX;
    xyt[ 1 ] = fY;
    xyt[ 2 ] = fTheta;
    // Ernest- m_bposition_set tells us that xyt[] values were initialized at least
    // once to valid values
    m_bposition_set = true; 
    pthread_mutex_unlock( &m_mutexXytLock );
}

/**********************************************************************
# $Log: robot.c,v $
# Revision 1.9  2003/04/02 22:25:20  zkira
# Added battery level variable to class
#
# Revision 1.8  2002/01/16 22:29:42  ernest
# Added flag  m_bposition_set.
#
# Revision 1.7  2001/12/22 16:35:24  endo
# RH 7.1 porting.
#
# $Log: robot.c,v $
# Revision 1.9  2003/04/02 22:25:20  zkira
# Added battery level variable to class
#
# Revision 1.8  2002/01/16 22:29:42  ernest
# Added flag  m_bposition_set.
#
# Revision 1.6  2001/05/29 22:35:15  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.5  2001/03/23 21:32:03  blee
# altered to use a config file
#
# Revision 1.4  2000/12/12 23:29:29  blee
# Changed #defines for the user interface to enumerated types.
#
# Revision 1.3  2000/11/13 20:09:23  endo
# ARTV-Jr class added to HServer.
#
# Revision 1.2  2000/09/19 03:49:46  endo
# RCS log added.
#
#
#**********************************************************************/
