/**********************************************************************
 **                                                                  **
 **                              robot.h                             **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  Generic robot class for HServer                                 **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: robot.h,v 1.7 2003/04/02 22:26:24 zkira Exp $ */

#ifndef ROBOT_H
#define ROBOT_H

#include <pthread.h>
#include "module.h"

enum RobotType { rt_fred, rt_nomad, rt_pioneer, rt_urban, rt_atrvjr, rt_amigobot};

class Robot : public Module
{
protected:
    float speed_factor;
    float angular_speed_factor;
    virtual void statusbarUpdate() = 0;
    RobotType type;
    pthread_mutex_t m_mutexXytLock;

    float battery_level; //trw zk

public:
    float xyt[ 3 ];
    virtual void disconnect() {};
    virtual void getXYT( float& fX, float& fY, float& fTheta );
    virtual void setXYT( float fX, float fY, float fTheta );
    virtual void initXYT() {};
    virtual void adjustXYT( float xinc, float yinc, float tinc ) {};
    virtual void steer( float avel ) {};  
    virtual void drive( float vel ) {};   
    virtual void steerToward( int angle ) {};
    virtual void stop() {};
    virtual void control() {};
    virtual void reset() {};
    virtual void sonarStart() {};
    virtual void sonarStop() {};
    virtual void arm( float avel ) {};
    
    // Returns -1 if no battery information is available
    float getBattery() { return battery_level; } // zk trw

    RobotType getType(){ return type; };
    Robot( Robot** a, RobotType type, char* name );
    // Ernest - flag tells us if position xyt[] has ever been initialized
    // ( for use by PositionCalculator )
    bool m_bposition_set;
    Robot();
    ~Robot();
};

extern Robot* robot;

#endif

/**********************************************************************
# $Log: robot.h,v $
# Revision 1.7  2003/04/02 22:26:24  zkira
# Added battery level variable to class
#
# Revision 1.6  2002/02/18 13:48:21  endo
# AmigoBot added.
#
# Revision 1.5  2002/01/16 22:33:33  ernest
# Added
# m_bposition_set flag.
#
# Revision 1.4  2001/05/29 22:35:28  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.3  2000/11/13 20:09:23  endo
# ARTV-Jr class added to HServer.
#
# Revision 1.2  2000/09/19 03:49:46  endo
# RCS log added.
#
#
#**********************************************************************/
