/**********************************************************************
 **                                                                  **
 **                              urban.c                             **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  RWI Urban Robot control for HServer                             **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: urban.c,v 1.8 2003/04/02 22:28:10 zkira Exp $ */

#define USE_URBAN

#include <pthread.h>
#include <unistd.h>
#include <math.h>
#include "mobilitycomponents_i.h"
#include "mobilitydata_i.h"
#include "mobilitygeometry_i.h"
#include "mobilityactuator_i.h"
#include "mobilityutil.h"
#include "hserver.h"
#include "message.h"
#include "robot_config.h"
#include "urban.h"
#include "sensors.h"
#include "statusbar.h"

float urban_range_angle[] = { 0, 15, 75, 90, 115, -115, -90, -75, -15, 0 };

char defaultRobotName[] = "Urban";
MobilityGeometry::SegmentState_var pSonarSeg;
MobilityGeometry::SegmentData_var pSegData;
//MobilityGeometry::TransformState_var pLocation;
//MobilityGeometry::TransformData      *pLocationData;
mbyClientHelper* pHelper;
CORBA::Object_ptr ptempObj;
MobilityCore::ObjectDescriptor_var pDescriptor;
char pathName[ 255 ];
char* robotName;
MobilityActuator::ActuatorState_var pDriveCommand;
MobilityActuator::ActuatorState_var pArmCommand;
MobilityActuator::ActuatorData      dCommand;
MobilityActuator::ActuatorData      aCommand;
MobilityActuator::ActuatorState_var pLocation;
MobilityActuator::ActuatorData*     pLocationData;
MobilityActuator::ActuatorState_var pArmLocation;
MobilityActuator::ActuatorData*     pArmLocationData;

MobilityData::PowerManagementState_var pPower;
MobilityData::PowerManagementStatus*   pPowerData;


float xOffset = 0;
float yOffset = 0;
float tOffset = 0;

float urban_internal_xyt[ 3 ];
float urban_xyt[ 3 ];
float urban_sonar[ URBAN_NUM_SONAR ];
float urban_arm[ 1 ];
float urban_battery[ 1 ];

bool urban_debug = false;


void Urban::statusbarUpdate()
{
    static int c = 0;
    char buf[ 100 ];
  
    statusStr[ 0 ] = 0;

    strcat( statusStr, "Urban: " );
    switch( c )
    {
    case 0:
        strcat( statusStr, "-" );
        break;
    case 1:
        strcat( statusStr, "\\" );
        break;
    case 2:
        strcat( statusStr, "|" );
        break;
    case 3:
        strcat( statusStr, "/" );
        break;
    }
    c = ( c + 1 ) % 4;
    sprintf( buf, " %.2fV", urban_battery[ 0 ] );
    strcat( statusStr, buf );
    statusbar->update( statusLine );
}


void Urban::adjustXYT( float x, float y, float t )
{
    xOffset += x;
    yOffset += y;
    tOffset += t;
}

int Urban::initializeMobility()
{
    int argc;
    char* argv[ 4 ];

    argv[ 0 ] = "hserver";
    argv[ 1 ] = "-robot";
    argv[ 2 ] = "Urban";
    argv[ 3 ] = 0;
    argc = 3;

    robotName = defaultRobotName;

    pHelper = new mbyClientHelper( argc, argv );

    // Build a pathname to the component we want to use to get sensor data.
    sprintf( pathName, "%s/Sonar/Segment", robotName ); // Use robot name arg.

    // Locate the component we want.
    ptempObj = pHelper->find_object( pathName );

    // Request the interface we want from the object we found
    pSonarSeg = MobilityGeometry::SegmentState::_narrow( ptempObj ); //, env );
    
    // Build pathname to the component we want to use to drive the robot.
    sprintf( pathName, "%s/Drive/Command", robotName ); // Use robot name arg.

    // Locate object within robot.
    ptempObj = pHelper->find_object( pathName );

    // We'll send two axes of command. Axis[ 0 ] == translate, Axis[ 1 ] == rotate.
    dCommand.velocity.length( 2 );
    dCommand.velocity[ 0 ] = 0.0; 
    dCommand.velocity[ 1 ] = 0.0;

    // Request the interface we need from the object we found.
    pDriveCommand = MobilityActuator::ActuatorState::_duplicate( 
		MobilityActuator::ActuatorState::_narrow( ptempObj ) );

    // Build pathname to the component we want to use to drive the robot.
    sprintf( pathName, "%s/Arm/Command", robotName ); // Use robot name arg.

    // Locate object within robot.
    ptempObj = pHelper->find_object( pathName );

    // We'll send two axes of command. Axis[ 0 ] == translate, Axis[ 1 ] == rotate.
    aCommand.velocity.length( 2 );
    aCommand.velocity[ 0 ] = 0.0;
    aCommand.velocity[ 1 ] = 0.0;

    // Request the interface we need from the object we found.
    pArmCommand = MobilityActuator::ActuatorState::_duplicate( 
		MobilityActuator::ActuatorState::_narrow( ptempObj ) );
    // Build pathname to the component we want to use to drive the robot.
    sprintf( pathName, "%s/Drive/State", robotName ); // Use robot name arg.

    // Locate object within robot.
    ptempObj = pHelper->find_object( pathName );

    // Request the interface we need from the object we found.
    pLocation = MobilityActuator::ActuatorState::_duplicate( 
		MobilityActuator::ActuatorState::_narrow( ptempObj ) );

    // Build pathname to the component we want to use to drive the robot.
    sprintf( pathName, "%s/Arm/State", robotName ); // Use robot name arg.

    // Locate object within robot.
    ptempObj = pHelper->find_object( pathName );

    // Request the interface we need from the object we found.
    pArmLocation = MobilityActuator::ActuatorState::_duplicate( 
		MobilityActuator::ActuatorState::_narrow( ptempObj ) );

    // Build pathname to the component we want to use to drive the robot.
    sprintf( pathName, "%s/Power", robotName ); // Use robot name arg.
  
    // Locate object within robot.
    ptempObj = pHelper->find_object( pathName );
  
    // Request the interface we need from the object we found.
    pPower = MobilityData::PowerManagementState::_duplicate( 
        MobilityData::PowerManagementState::_narrow( ptempObj ) );

    return 0;
}  



void Urban::readSonarRanges()
{
    pSegData = pSonarSeg->get_sample( 0 );
  
    for ( unsigned int i = 0; i < pSegData->org.length(); i++ )
    {
        // Compute segment lengths.
        urban_sonar[ i ] = sqrt( 
            ( pSegData->org[ i ].x - pSegData->end[ i ].x ) *
            ( pSegData->org[ i ].x - pSegData->end[ i ].x ) +
            ( pSegData->org[ i ].y - pSegData->end[ i ].y ) *
            ( pSegData->org[ i ].y - pSegData->end[ i ].y ) );
    }
}

void Urban::sendMovement()
{
    dCommand.velocity[ 0 ] = goalvelocity;
    dCommand.velocity[ 1 ] = goalavelocity;
    pDriveCommand->new_sample( dCommand, 0 );
}

void Urban::drive( float vel )
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printfTextWindow( "Urban Drive: %f", vel );
    }
    goalvelocity = vel;
    sendMovement();
}

void Urban::arm( float avel )
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printfTextWindow( "Urban Arm: %f", avel );
    }
    goalarmvelocity = avel;
    aCommand.velocity[ 0 ] = avel;
    pArmCommand->new_sample( aCommand, 0 );  
}

void Urban::initXYT()
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printTextWindow( "Urban InitXY" );
    }
}

void Urban::stop()
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printTextWindow( "Urban Stop" );
    }
    goalvelocity = goalarmvelocity = goalavelocity = 0;
    dCommand.velocity[ 0 ] = 0.0;
    dCommand.velocity[ 1 ] = 0.0;
    pDriveCommand->new_sample( dCommand, 0 );  
    arm( 0.0 );
} 

void Urban::steer( float avel )
{
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printfTextWindow( "Urban Steer: %f", avel );
    }
  
    avel *= M_PI / 180.0; // deg/sec to rad/sec

    goalavelocity = avel;
    sendMovement();
}


void Urban::steerToward( int angle )
{
    float avel; // deg/sec
    int absang;
    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printfTextWindow( "Urban steer towards: %d", angle );
    }
    absang = abs( angle );

    // Works on carpet with fishtailing
    if ( absang > 90 )
    {
        avel = 60;
    }
    else if ( absang > 60 )
    {
        avel = 50;
    }
    else if ( absang > 30 )
    {
        avel = 40;
    }
    else if ( absang > 20 )
    {
        avel = 27;
    }
    else if ( absang > 10 )
    {
        avel = 22;
    }
    else if ( absang > 5 )
    {
        avel = 17;    
    }
    else
    {
        avel = 12;
    }

    //Original
    //if ( absang>90 ) avel = 60;
    //else if ( absang>60 ) avel = 50;
    //else if ( absang>30 ) avel = 40;
    //else avel = 30;

    if ( angle < 0 )
    {
        avel *= -1;
    }
    //avel *= float( turbo_factor )/10.0;
    if ( fabs( angle ) < 4 )
    {
        avel = 0;
    }
    steer( int( avel ) );
}	


void Urban::setXYT( float x, float y, float t )
{
    float xu, yu, tu;
    pLocationData = pLocation->get_sample( 0 );

    xu = pLocationData->position[ 0 ];
    yu = pLocationData->position[ 1 ];
    tu = pLocationData->position[ 2 ];

    tu = tu * 180.0 / M_PI;

    if ( tu < 0 )
    {
        tu += 360;
    }

    xOffset = x - xu;
    yOffset = y - yu;
    tOffset = t - tu;

    updateXytFromInternalXyt();

    printfTextWindow( "setxyt %f %f %f", xOffset, yOffset, tOffset );

    if ( report_level >= HS_REP_LEV_DEBUG )
    {
        printfTextWindow( "Urban Set XYT X:%f Y:%f T:%f", x, y, t );
    }
}


void Urban::updateXytFromInternalXyt()
{
    urban_xyt[ 0 ] = urban_internal_xyt[ 0 ] + xOffset;
    urban_xyt[ 1 ] = urban_internal_xyt[ 1 ] + yOffset;
    urban_xyt[ 2 ] = urban_internal_xyt[ 2 ] * 180 / M_PI;
    if ( urban_xyt[ 2 ] < 0 )
    {
        urban_xyt[ 2 ] += 360;
    }

    Robot::setXYT( urban_xyt[ 0 ], urban_xyt[ 1 ], urban_xyt[ 2 ] );
}

static void start_thread( void* urban_instance )
{
    ( (Urban*) urban_instance )->reader_thread();
}


// Thread to update urban information
void Urban::reader_thread()
{
    while ( true )
    {
        pthread_testcancel();
        usleep( 100000 );		// Need to make this variable
        pPowerData = pPower->get_sample( 0 );
        urban_battery[ 0 ] = pPowerData->SourceVoltage[ 0 ];
        battery_level = urban_battery[ 0 ];
        pLocationData = pLocation->get_sample( 0 );
        pArmLocationData = pArmLocation->get_sample( 0 );
        urban_internal_xyt[ 0 ] = pLocationData->position[ 0 ];
        urban_internal_xyt[ 1 ] = pLocationData->position[ 1 ];
        urban_internal_xyt[ 2 ] = pLocationData->position[ 2 ];
        updateXytFromInternalXyt();
        urban_arm[ 0 ] = pArmLocationData->position[ 0 ];
        readSonarRanges();
        drive( goalvelocity ); // watchdog
        statusbarUpdate();
    }
}

Urban::~Urban()
{
    pthread_cancel( reader );
    pthread_join( reader, NULL );
    delete sensorSonar;
    delete sensorArm;
    printTextWindow( "Urban disconnected" );
}

void Urban::addSensors()
{
    int i;
    
    partXyt = new RobotPartXyt();
    for( i = 0; i < 3; i++ )
    {
        urban_xyt[ i ] = 0.0;
    }

    partSonar = new RobotPartSonar( URBAN_NUM_SONAR, urban_range_angle );
    // ENDO
    //sensorSonar = new Sensor( SENSOR_SONAR, 0, NULL, 0, NULL, URBAN_NUM_SONAR, urban_sonar );
    sensorSonar = new Sensor( SENSOR_SONAR, 0, NULL, URBAN_NUM_SONAR, urban_sonar, 
                              URBAN_NUM_SONAR, urban_sonar );
    for( i = 0; i < URBAN_NUM_SONAR; i++ )
    {
        urban_sonar[ i ] = 0.0;
    }

    partArm = new RobotPartArm();
    sensorArm = new Sensor( SENSOR_ARM, 0, NULL, 1, urban_arm );
    urban_arm[ 0 ] = 0.0;
}

// ENDO
//Urban::Urban( Robot **a ) : Robot( a, rt_urban ){
Urban::Urban( Robot** a ) : Robot( a, rt_urban, "Urban" )
{
    int rtn;
    if ( report_level > HS_REP_LEV_NONE )
    {
        printTextWindow( "Connecting to urban... ", false );
    }
    rtn = initializeMobility();
    refreshScreen();
    if ( rtn )
    {
        if ( report_level > HS_REP_LEV_NONE )
        {
            printTextWindow( "failed", true );
        }    
        //return( -1 ); // fix this
        return;
    }

    // *** initialize xyt needed
    pthread_create( &reader, NULL, (void*(*)(void*)) &start_thread, (void*) this );

    if ( report_level > HS_REP_LEV_NONE )
    {
        printTextWindow( "connected", true );
    }
    statusbarUpdate();
    addSensors();
}

void Urban::control()
{
    bool bDone = false;

    messageDrawWindow( EnMessageType_URBAN_CONTROL, EnMessageErrType_NONE );
    redrawWindows();
    do
    {
        int c = getch();
        switch ( c )
        {
        case 'd':
            messageHide();
            delete this;
            return;
            break;

        case 'i':
            setXYT( 5, 5, 0 );
            break;

        case 'x':
        case 'Q':
        case KEY_ESC:
            bDone = true;
            break;

        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case KEY_UP:
        case KEY_DOWN:
        case KEY_LEFT:
        case KEY_RIGHT:
            messageMovePanelKey( c );
            break;
        }
    } while ( !bDone );
    messageHide();
}

/**********************************************************************
# $Log: urban.c,v $
# Revision 1.8  2003/04/02 22:28:10  zkira
# Added battery level variable to class
#
# Revision 1.7  2001/05/29 22:36:14  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.6  2000/12/12 23:24:45  blee
# Changed #defines for the user interface to enumerated types.
#
# Revision 1.5  2000/11/13 20:12:59  endo
# Minor compiler warning fix.
#
# Revision 1.4  2000/10/16 21:22:12  endo
# steerToward() adjusted. (Modified at Rockville, MD)
#
# Revision 1.3  2000/09/19 03:51:29  endo
# RCS log added.
#
#
#**********************************************************************/
