/**********************************************************************
 **                                                                  **
 **                              urban.h                             **
 **                                                                  **
 **                                                                  **
 **  Written by:  William Halliburton                                **
 **                                                                  **
 **  RWI Urban Robot control for HServer                             **
 **                                                                  **
 **  Copyright 2000, Georgia Tech Research Corporation               **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: urban.h,v 1.4 2001/05/29 22:36:19 blee Exp $ */

#ifndef URBAN_H_
#define URBAN_H_

#define URBAN_NUM_SONAR 10
#include "robot.h"
#include "sensors.h"

class Urban : public Robot
{
private:
    Sensor* sensorSonar;
    Sensor* sensorArm;
    RobotPartXyt* partXyt;
    RobotPartSonar* partSonar;
    RobotPartArm* partArm;
    int initializeMobility();
    void readSonarRanges();
    void sendMovement();
    void updateXytFromInternalXyt();
    void addSensors();
    void statusbarUpdate();
    pthread_t reader;
    float goalvelocity;
    float goalavelocity;
    float goalarmvelocity;

public:
    void reader_thread();
    void drive( float vel );
    void arm( float avel );
    void stop();
    void steer( float avel );
    void setXYT( float x, float y, float t );
    void initXYT();
    void adjustXYT( float x, float y, float t );
    void control();
    void steerToward( int angle );
    Urban( Robot** a );
    ~Urban();
};

#endif

/**********************************************************************
# $Log: urban.h,v $
# Revision 1.4  2001/05/29 22:36:19  blee
# Made changes to incorporate gps data w/ other position info.
#
# Revision 1.3  2000/09/19 03:51:29  endo
# RCS log added.
#
#
#**********************************************************************/
