/**********************************************************************
 **                                                                  **
 **                              cbrclient.h                         **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  This is a header file for cbrclient.cc.                         **
 **                                                                  **
 **  Copyright 2002 - 2003 Georgia Tech Research Corporation         **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/
#ifndef CBRCLIENT_H
#define CBRCLIENT_H

/* $Id: cbrclient.h,v 1.3 2003/04/06 15:21:25 endo Exp $ */

#include <string>
#include <vector>
#include "mission_expert_types.h"
#include "cbrplanner_protocol.h"
#include "configuration.hpp"

using std::string;
using std::vector;

class CBRClient {

protected:
    int socketFd_;
    CBRPlanner_ClientInfo_t clientInfo_;
    static const string STRING_TASK_;
    static const string STRING_START_X_;
    static const string STRING_START_Y_;
    static const string RCTABLE_SERVERSOCKET_STRING_;
    static const string EMPTY_STRING_;

    bool connectToServer_(void);
    bool readNBytes_(const int fd, unsigned char *buf, const int n);
    void sendCommandToServer_(const unsigned char cmd);
    void sendDataToServer_(void *data, int dataSize);
    vector<int> sendMissionPlanToServer_(
	CBRPlanner_MissionPlan_t missionplan,
	vector<CBRPlanner_Feature_t> features);
    vector<CBRPlanner_MissionPlan_t> retrieveMissionPlansFromServer_(
	vector<CBRPlanner_Feature_t> features,
	int maxRating);
    void sendFSANamesToServer_(config_fsa_lists_t fsaLists);
    void sendFeedbackToServer_(const unsigned char cmd, vector<int> dataIndexList);

public:
    CBRClient(void);
    ~CBRClient(void);
    bool initialize(void);
    vector<int> saveMissionPlan(
	vector<MExpFeature_t> globalFeatures_,
	MExpMissionTask_t task,
	double taskWeight,
	string solutionFileName);
    vector<MExpRetrievedMission_t> retrieveMissionPlan(
	string featureFileName,
	vector<MExpMissionTask_t> defaultTaskList,
	vector<MExpFeature_t> defaultGlobalFeatures,
	vector<MExpFeature_t> defaultLocalFeatures,
	double taskWeight,
	int maxRating,
	string *overlayFileName);
    void saveFSANames(config_fsa_lists_t fsaLists);
    void sendSelectedMissionIndexToServer(int index);
    void sendPositiveFeedbackToServer(vector<int> dataIndexList);
    void sendNegativeFeedbackToServer(vector<int> dataIndexList);
};

extern CBRClient *gCBRClient;

#endif
/**********************************************************************
 * $Log: cbrclient.h,v $
 * Revision 1.3  2003/04/06 15:21:25  endo
 * gcc 3.1.1
 *
 * Revision 1.2  2003/04/06 08:48:14  endo
 * Updated for CBR Wizard Prototype II.
 *
 * Revision 1.1  2002/01/13 01:08:36  endo
 * Initial revision
 *
 **********************************************************************/
