/****************************************************************************
*                                                                           *
*                      gt_ipt_msg_def.h                                     *
*                                                                           *
*                                                                           *
* Declare MSG_TYPE array for ipt communication between console & robots     *
*                                                                           *
****************************************************************************/

/* $Id: gt_ipt_msg_def.h,v 1.20 2003/04/02 23:07:32 zkira Exp $ */

#ifndef GT_IPT_MSG_DEF_H
#define GT_IPT_MSG_DEF_H

#include "ipt/ipt.h"
#include "gt_message_types.h"

char* DefaultConsoleName = "Console";
char* ComVersion  = "$Revision: 1.20 $";

/************************************/
/* name some formatters we will use */
/************************************/

char* TYP_Point = "Point";
#define FRM_Point "{float, float}"
/*
struct REC_Point
{
    double x;
    double y;
    double z;
};
*/

char* TYP_ObjectReading = "ObjectReading";
#define FRM_ObjectReading "{string, int, int, float, float, float, float, float, boolean}"
struct REC_ObjectReading
{
    char* object_color;
    int   id;
    int   object_shape;
    float x1;
    float y1;
    float r;
    float x2;
    float y2;
    bool  ismovable;
};

char* TYP_SensorReading = "SensorReading";
#define FRM_SensorReading "{int,int,float,float}"
struct REC_SensorReading
{
    int type;
    int color;
    float angle;
    float reading;
};

/* Resets the console world */
/* For simulation only */
char *MSG_ResetWorld = "ResetWorld";
#define FRM_ResetWorld "{string,int}"
typedef struct REC_ResetWorld {
    char* new_overlay;
    int  load_new_map;
} REC_ResetWorld;

char* MSG_UpdateLMParams = "UpdateLMParams";
#define FRM_UpdateLMParams "{int}"
struct REC_UpdateLMParams
{
    int iRobotID;
};

char* MSG_StepTaken = "StepTaken";
#define FRM_StepTaken "{int,int}"
struct REC_StepTaken
{
    int iRobotID;
    int iStep;
};

/************************************************/
/* ipt messages sent from robots to the console */
/************************************************/

char* MSG_RobotBirth = "RobotBirth";
#define FRM_RobotBirth "{int, float, float, float, string, string}"
struct REC_RobotBirth
{
    int   robot_id;
    float start_x;
    float start_y;
    float start_heading;
    char* port_name;
    char* com_version;
};

char* MSG_NotifyConfig = "NotifyConfig";
#define FRM_NotifyConfig "{int, int, int}"
struct REC_NotifyConfig
{
    int robot_id;
    int robot_type;
    int run_type;
};

char* MSG_RobotDeath = "RobotDeath";
#define FRM_RobotDeath "{int, string}"
struct REC_RobotDeath
{
    int   robot_id;
    char* message;
};

char* MSG_ReportedState = "ReportedState";
#define FRM_ReportedState "{string}"
struct REC_ReportedState
{
    char* message;
};

char* MSG_Pickup = "Pickup";
#define FRM_Pickup "{int, int}"
struct REC_Pickup
{
    int robot_id;
    int object_id;
};

char* MSG_Probe = "Probe";
#define FRM_Probe "{int, int}"
struct REC_Probe
{
    int robot_id;
    int object_id;
};

char* MSG_Alert = "Alert";
#define FRM_Alert "{string}"
struct REC_Alert
{
    char* message;
};

char* MSG_PopUpTelop = "PopUpTelop";
#define FRM_PopUpTelop "{int, int}"
struct REC_PopUpTelop
{
    int popup;
    int robot_id;
};

char* MSG_AskToProceed = "AskToProceed";
#define FRM_AskToProceed "{int, int}"
struct REC_AskToProceed
{
    int ask;
    int robot_id;
};

char* MSG_ChangeColor = "ChangeColor";
#define FRM_ChangeColor "{int, int, string}"
struct REC_ChangeColor
{
    int   robot_id;
    int   object_id;
    char* new_color;
};

char *MSG_ChangeRobotColor = "ChangeRobotColor";
#define FRM_ChangeRobotColor "{int, string}"
typedef struct {
   int    robot_id;
   char  *new_color;
} REC_ChangeRobotColor;

char* MSG_DropInBasket = "DropInBasket";
#define FRM_DropInBasket "{int, int}"
struct REC_DropInBasket
{
    int robot_id;
    int object_id;
    int basket_id;
};

char* MSG_MoveRobot = "MoveRobot";
#define FRM_MoveRobot "{int, float, float}"
struct REC_MoveRobot
{
    int   robot_id;
    float x;
    float y;
};

char* MSG_SpinRobot = "SpinRobot";
#define FRM_SpinRobot "{int, float}"
struct REC_SpinRobot
{
    int   robot_id;
    float w;
};

char* MSG_RobotLocation = "RobotLocation";
#define FRM_RobotLocation "{int, float, float, float}"
struct REC_RobotLocation
{
    int   robot_id;
    float x;
    float y;
    float heading;
};

//trw zk
char* MSG_BatteryInfo = "BatteryInfo";
#define FRM_BatteryInfo "{int, int}"
struct REC_BatteryInfo
{
    int id;
    int level;
};


//trw lilia
char* MSG_GpsPosition = "GpsPosition";
#define FRM_GpsPosition "{int, double, double, double, double, int}"                      //jh
struct REC_GpsPosition
{
    int   robot_id;
    float latitude;
    float longitude;
    float direction;
    float pingtime; //jh
    int    pingloss;
};

char* MSG_InquireMapLocation = "InquireMapLocation";
#define FRM_InquireMapLocation "{int, string}"
struct REC_InquireMapLocation 
{
    int   robot_id;
    char* name;
};


// Queries from robot to the console
// For simulation only

char* MSG_GetPosition = "GetPosition";
#define FRM_GetPosition "{int}"
struct REC_GetPosition
{
    int robot_id;
};

char* MSG_GetCompassHeading = "GetCompassHeading";
#define FRM_GetCompassHeading "{int}"
struct REC_GetCompassHeading
{
    int heading;
};

char* MSG_MarkDoorway = "MarkDoorway";
#define FRM_MarkDoorway "{int, int}"
struct REC_MarkDoorway
{
    int robot_id;
    int mark_status;
};

char* MSG_DetectDoorway = "DetectDoorway";
#define FRM_DetectDoorway "{int, double, double, double}"
struct REC_DetectDoorway
{
    int    robot_id;
    double sensor_dir;
    double sensor_fov;
    double sensor_range;
};

char* MSG_DetectDoorwayReply = "DetectDoorwayReply";
#define FRM_DetectDoorwayReply "{int, <{float, float, float, float, double, int}: 1>}" 

char* MSG_DetectHallway = "DetectHallway";
#define FRM_DetectHallway "{int, {{double, double, double}, double}}"
struct REC_DetectHallway
{
    int            robot_id;
    robot_position robot_loc;
};

char* MSG_DetectHallwayReply = "DetectHallwayReply";
#define FRM_DetectHallwayReply "{int, <{{double, double, double}, {double, double, double}, double}: 1>}"

char* MSG_DetectRoom = "DetectRoom";
#define FRM_DetectRoom "{int, {{double, double, double}, double}}"
struct REC_DetectRoom
{
    int            robot_id;
    robot_position robot_loc;
};

char* MSG_DetectRoomReply = "DetectRoomReply";
#define FRM_DetectRoomReply "{boolean}"

char* MSG_DetectObstacles = "DetectObstacles";
#define FRM_DetectObstacles "{int, float}"
struct REC_DetectObstacles
{
    int   robot_id;
    float max_dist;
};

char* MSG_DetectObjects = "DetectObjects";
#define FRM_DetectObjects "{int, float}"
struct REC_DetectObjects
{
    int   robot_id;
    float max_dist;
};

char* MSG_DetectRobots = "DetectRobots";
#define FRM_DetectRobots "{int, float}"
struct REC_DetectRobots
{
    int   robot_id;
    float max_dist;
};


// reports from robot to console

char* MSG_PutState = "PutState";
#define FRM_PutState "{int, string, string}"
struct REC_PutState
{
    int   robot_id;
    char* key;
    char* value;
};


char* MSG_ObjectReadings = "ObjectReadings";
#define FRM_ObjectReadings "{int, <ObjectReading: 1>}"
struct REC_ObjectReadings
{
    int num_readings;
    REC_ObjectReading* readings;
};

// why can't use RobotLocation?
char* MSG_RobotPos = "RobotPos";
#define FRM_RobotPos "{float, float, float}"
struct REC_RobotPos
{
    float x;
    float y;
    float heading;
};

char* MSG_LaserReadings = "LaserReadings";
#define FRM_LaserReadings "{int, int, <float: 2>, <float: 2>}"
struct REC_LaserReadings
{
    int robot_id;
    int num_readings;
    float* readings;
    float* angles;
};

char* MSG_SensorReadings = "SensorReadings";
#define FRM_SensorReadings "{int, int, <SensorReading: 2>}"
struct REC_SensorReadings
{
    int robot_id;
    int num_readings;
    REC_SensorReading* reading;
};

char* MSG_PingSend = "PingSend";
#define FRM_PingSend "{int}"
struct REC_PingSend
{
    int check;
};

/****************************************************/
/* IPT messages sent from the console to a robot    */
/****************************************************/

char* MSG_Ack = "Ack";
#define FRM_Ack "{boolean}"
struct REC_Ack
{
    bool junk;
};

char* MSG_StepOneCycle = "StepOneCycle";
#define FRM_StepOneCycle "boolean"
typedef short REC_StepOneCycle;

char* MSG_RobotPause = "RobotPause";
#define FRM_RobotPause "boolean"
typedef short REC_RobotPause;

char* MSG_RobotDebug = "RobotDebug";
#define FRM_RobotDebug "boolean"
typedef short REC_RobotDebug;

char* MSG_ReportState = "ReportState";
#define FRM_ReportState "boolean"
typedef short REC_ReportState;

char* MSG_RobotSuicide = "RobotSuicide";
#define FRM_RobotSuicide "int"
typedef int REC_RobotSuicide;

char* MSG_MapLocation = "MapLocation";
#define FRM_MapLocation "{boolean, float, float}"

struct REC_MapLocation 
{
    bool valid;
    gt_Point pos;
};

char* MSG_CurPosition = "CurXY";
#define FRM_CurPosition "{float, float, float}"

char* MSG_ObstacleList = "ObstacleList";
#define FRM_ObstacleList "{int, <Point: 1>, <float: 1>}"
struct REC_ObstacleList
{
    int num_obs;
    gt_Point* obs;
    float* radius;
};
  
char* MSG_ObjectList = "ObjectList";
#define FRM_ObjectList "{int, <ObjectReading: 1>}"
struct REC_ObjectList
{
    int num_objects;
    REC_ObjectReading* object;
};

char* MSG_CurRobots = "CurRobots";
#define FRM_CurRobots "{int, <{int, {float, float}}: 1>}"
struct REC_CurRobots
{
    int num_robots;
    gt_Id_and_v* robots;
};


char* MSG_NewCommand = "NewCommand";
#define FRM_NewCommand "{int, <{float, float}: 1>, int, <int: 3>, int, <{float, float}: 5>, {float, float}, int, int, int, float, float, int, int, string, int, int, string, int, int, int, int}"
struct REC_NewCommand 
{
    int           num_path_points;
    gt_Point*     path;
  
    int           unit_size;
    int*          unit;
  
    int           num_pl_points;
    gt_Point*     phase_line;
  
    gt_Point 	  goal;
    gt_Behavior   behavior;
    gt_Formation  formation;
    gt_Technique  technique;
    float         speed;
    float         width;
    int           phase_line_given;
    int           time;   // Seconds since epoch (see 'man 3 time').
                          // 0 means time is not a factor (no phase line).
                          // Negative means seconds for timeout.
    char*         phase_line_ack_msg;
    int           wait_at_phase_line;
    int           completion_given;
    char*         completion_msg;
    int           freeze_on_completion;
    int           until_timeout_given;
    int           robot_id;
    int           seq_num;    // unique command sequence number
  
};
// Note the above is a duplicate of gt_Command defined in gt_command.h


char* MSG_NewState = "NewState";
#define FRM_NewState "{string, string}"
struct REC_NewState
{
    char* key;
    char* value;
};


char* MSG_Declare_iptClient = "Declare_iptClient";
#define FRM_Declare_iptClient "{int, string}"
struct REC_Declare_iptClient
{
    int server_id;
    char* server_name;
};

char* MSG_AddObject = "AddObject";
#define FRM_AddObject "{int, string, float, float, float, int}"
struct REC_AddObject
{
    int   robot_id;
    char* color;
    float x;
    float y;
    float diameter;
    int   style;
};

char* MSG_PingBack = "PingBack";
#define FRM_PingBack "{int}"
struct REC_PingBack
{
    int check;
};

char* MSG_DrawingVectorField = "DrawingVectorField";
#define FRM_DrawingVectorField "int"
typedef int REC_DrawingVectorField;

char* MSG_DoneDrawingVectorField = "DoneDrawingVectorField";
#define FRM_DoneDrawingVectorField "int"
typedef int REC_DoneDrawingVectorField;

char* MSG_ReportSensorReadings = "ReportSensorReadings";
#define FRM_ReportSensorReadings "int"
typedef int REC_ReportSensorReadings;

/***********************************/
/* array declaring all ipt formats */

IPFormatSpec formatArray[] =
{
    { TYP_Point,                FRM_Point },
    { TYP_ObjectReading,        FRM_ObjectReading },         
    { TYP_SensorReading,        FRM_SensorReading },
    { NULL,                     NULL }
};


/*******************************************************************/
// array declaring all ipt messages
IPMessageSpec messageArray[] = {

    // ipt messages sent from robots to the console
    { MSG_RobotBirth,             FRM_RobotBirth },
    { MSG_NotifyConfig,           FRM_NotifyConfig },
    { MSG_RobotDeath,             FRM_RobotDeath },
    { MSG_Pickup,                 FRM_Pickup },
    { MSG_Probe,                  FRM_Probe },
    { MSG_Alert,                  FRM_Alert },
    { MSG_PopUpTelop,             FRM_PopUpTelop },
    { MSG_AskToProceed,           FRM_AskToProceed },
    { MSG_ChangeColor,            FRM_ChangeColor },
    { MSG_ChangeRobotColor,       FRM_ChangeRobotColor },
    { MSG_DropInBasket,           FRM_DropInBasket },
    { MSG_MoveRobot,              FRM_MoveRobot },
    { MSG_SpinRobot,              FRM_SpinRobot },
    { MSG_RobotLocation,          FRM_RobotLocation },
    { MSG_GpsPosition,            FRM_GpsPosition },
    { MSG_BatteryInfo,            FRM_BatteryInfo },          // trw zk
    { MSG_InquireMapLocation,     FRM_InquireMapLocation },
    { MSG_DetectDoorway,          FRM_DetectDoorway},
    { MSG_DetectHallway,          FRM_DetectHallway},
    { MSG_DetectRoom,             FRM_DetectRoom},
    { MSG_MarkDoorway,            FRM_MarkDoorway},
    { MSG_GetPosition,            FRM_GetPosition }, 
    { MSG_GetCompassHeading,      FRM_GetCompassHeading },
    { MSG_DetectObstacles,        FRM_DetectObstacles },
    { MSG_DetectObjects,          FRM_DetectObjects },
    { MSG_DetectRobots,           FRM_DetectRobots },
    { MSG_PutState,               FRM_PutState },
    { MSG_ObjectReadings,         FRM_ObjectReadings },
    { MSG_RobotPos,               FRM_RobotPos },
    { MSG_LaserReadings,          FRM_LaserReadings },
    { MSG_ReportedState,          FRM_ReportedState }, // robot reports status.
    { MSG_PingSend,               FRM_PingSend},
    { MSG_Ack,                    FRM_Ack },
    { MSG_StepOneCycle,           FRM_StepOneCycle },
    { MSG_RobotPause,             FRM_RobotPause },
    { MSG_RobotSuicide,           FRM_RobotSuicide },
    { MSG_RobotDebug,             FRM_RobotDebug },
    { MSG_ReportState,            FRM_ReportState }, // mlab asks robot to report status.
    { MSG_MapLocation,            FRM_MapLocation },
    { MSG_CurPosition,            FRM_CurPosition },
    { MSG_ObstacleList,           FRM_ObstacleList },
    { MSG_ObjectList,             FRM_ObjectList },
    { MSG_CurRobots,              FRM_CurRobots },
    { MSG_NewCommand,             FRM_NewCommand },
    { MSG_NewState,               FRM_NewState },
    { MSG_Declare_iptClient,      FRM_Declare_iptClient },
    { MSG_AddObject,              FRM_AddObject },
    { MSG_DetectDoorwayReply,     FRM_DetectDoorwayReply},
    { MSG_DetectHallwayReply,     FRM_DetectHallwayReply},
    { MSG_DetectRoomReply,        FRM_DetectRoomReply},
    { MSG_SensorReadings,         FRM_SensorReadings},
    { MSG_PingBack,               FRM_PingBack},
    { MSG_UpdateLMParams,         FRM_UpdateLMParams},
    { MSG_StepTaken,              FRM_StepTaken},
    { MSG_DrawingVectorField,     FRM_DrawingVectorField },
    { MSG_DoneDrawingVectorField, FRM_DoneDrawingVectorField },
    { MSG_ReportSensorReadings,   FRM_ReportSensorReadings },
    { MSG_ResetWorld,             FRM_ResetWorld },
    { NULL,                       NULL }
};

#endif  // GT_IPT_MSG_DEF_H

/**********************************************************************
 * $Log: gt_ipt_msg_def.h,v $
 * Revision 1.20  2003/04/02 23:07:32  zkira
 * Added GPS and battery messages
 *
 * Revision 1.19  2002/10/31 21:28:00  ebeowulf
 * Added messaging and header for Reset World code.
 *
 * Revision 1.18  2002/10/31 20:56:31  ebeowulf
 * Fixed typo in ChangeRobotColor code.
 *
 * Revision 1.17  2002/10/31 20:00:00  ebeowulf
 * added change robot color function and message
 *
 * Revision 1.16  2002/07/02 20:36:27  blee
 * added messages so mlab could draw vector fields
 *
 * Revision 1.15  2002/04/04 18:35:28  blee
 * Added MSG_StepTaken stuff.
 *
 * Revision 1.14  2000/12/12 22:44:15  blee
 * altered MSG_LaserReadings stuff
 *
 * Revision 1.13  2000/11/10 23:29:06  blee
 * added UpdateLMParams message info (and in the message table)
 *
 * Revision 1.12  2000/08/19 00:57:26  endo
 * MSG_PingSend, MSG_PingBack added.
 *
 * Revision 1.11  2000/08/15 20:52:34  endo
 * MSG_AskToProceed added.
 *
 * Revision 1.10  2000/07/02 01:18:24  conrad
 * added sensor messages
 *
 * Revision 1.9  2000/06/13 16:44:35  endo
 * MSG_Declare_iptClient added.
 *
 * Revision 1.8  2000/05/19 21:39:11  endo
 * Ipt message handling for PopUpTelop added.
 *
 * Revision 1.7  2000/04/25 08:51:28  jdiaz
 * added start_heading field to robot_birth message
 *
 * Revision 1.6  2000/03/30 01:42:17  endo
 * MSG_SpinRobot FRM_SpinRobot REC_SpinRobot added.
 *
 * Revision 1.5  2000/02/28 23:54:30  jdiaz
 * messages to mark and unmark a doorway
 *
 * Revision 1.4  2000/02/18 02:02:00  endo
 * REC_Alert, Msg_Alert, FRM_Alert added for the
 * Alert state.
 *
 * Revision 1.3  2000/02/07 05:20:30  jdiaz
 * doorways walls hallways rooms
 *
 * Revision 1.2  1999/12/18 09:18:55  endo
 * ReportedState (robot -> mlab)
 * ReportState (mlab -> robot)
 *
 **********************************************************************/
