/**********************************************************************
 **                                                                  **
 **                           mission_expert.h                       **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2002 - 2003 Georgia Tech Research Corporation         **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: mission_expert.h,v 1.3 2003/04/06 12:25:34 endo Exp $ */

#ifndef MISSION_EXPERT_H
#define MISSION_EXPERT_H

#include <stdio.h>
#include <string>
#include <vector>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <Xm/Xm.h>

#include "symbol.hpp"
#include "load_rc.h"
#include "mission_expert_types.h"
#include "cbrclient.h"
#include "configuration.hpp"

using std::string;
using std::vector;

// Extentions of some of the output files
const string MEXP_EXTENTION_FEATUREFILENAME = "data";
const string MEXP_EXTENTION_FEATURESTATUSFILENAME = "status";
const string MEXP_EXTENTION_CDL = "cdl";
const string MEXP_EXTENTION_ACDL = "acdl";

// Range of the slider bar
const string MEXP_DEFAULT_SLIDER1_RANAGE_TITLE = "[0.01 - 1.00]";
const string MEXP_DEFAULT_SLIDER10_RANAGE_TITLE = "[1 - 10]";

// Some strings
const string MEXP_STRING_OVERLAY = "overlay";
const string MEXP_STRING_TASK = "task";
const string MEXP_STRING_COORDINATE = "coordinate";
const string MEXP_STRING_FEATURE = "feature";
const string MEXP_STRING_OPTION = "option";
const string MEXP_STRING_CONFIGURE = "configure";
const string MEXP_STRING_YES = "Yes";
const string MEXP_STRING_NO = "No";
const string MEXP_STRING_RETRIEVE = "retrieve";
const string MEXP_STRING_ADD = "add";
const string MEXP_STRING_GLOBALFEATURE = "global-feature";
const string MEXP_STRING_ROBOT = "Robot";

// RC file name.
const string MEXP_RC_FILENAME = ".mission_expertrc";

// RC file vocabulary
const string MEXP_RC_STRING_FEATURE_KEY = "<FEATURE";
const string MEXP_RC_STRING_FEATURE_BEGIN = "BEGIN>";
const string MEXP_RC_STRING_FEATURE_END = "END>";
const string MEXP_RC_STRING_FEATURE_TYPE = "FEATURE-TYPE";
const string MEXP_RC_STRING_FEATURE_NAME = "FEATURE-NAME";
const string MEXP_RC_STRING_FEATURE_NON_INDEX = "FEATURE-NON-INDEX";
const string MEXP_RC_STRING_FEATURE_WEIGHT = "FEATURE-WEIGHT";
const string MEXP_RC_STRING_FEATURE_OPTION_TYPE = "FEATURE-OPTION-TYPE";
const string MEXP_RC_STRING_FEATURE_SELECTED_OPTION = "FEATURE-SELECTED-OPTION";
const string MEXP_RC_STRING_FEATURE_OPTION = "FEATURE-OPTION";
const string MEXP_RC_STRING_TASK_KEY = "<TASK";
const string MEXP_RC_STRING_TASK_BEGIN = "BEGIN>";
const string MEXP_RC_STRING_TASK_END = "END>";
const string MEXP_RC_STRING_TASK_NAME = "TASK-NAME";
const string MEXP_RC_STRING_TASK_BUTTON_NAME = "TASK-BUTTON-NAME";
const string MEXP_RC_STRING_TASK_SPECIFIC_SELECTED_OPTION = "TASK-SPECIFIC-SELECTED-OPTION";
const string MEXP_RC_STRING_DEFAULT_TASK = "DEFAULT-TASK";
const string MEXP_RC_STRING_DEFAULT_TASK_WEIGHT = "DEFAULT-WEIGHT";
const string MEXP_RC_STRING_GLOBAL = "GLOBAL";
const string MEXP_RC_STRING_LOCAL = "LOCAL";
const string MEXP_RC_STRING_TOGGLE = "TOGGLE";
const string MEXP_RC_STRING_SLIDER = "SLIDER";

class MissionExpert {

    typedef struct GeneralYesNoWindowCallbackData_t {
	bool result;
	MissionExpert *missionExpertInstance;
    };

    typedef struct FeatureMenuCallbackData_t {
	int featureId;
	int optionType;
	float optionValue;
	MissionExpert *missionExpertInstance;
    };

    typedef struct GlobalFeatureMenuCallbackData_t {
	int globalFeaturesId;
	int optionType;
	float optionValue;
	MissionExpert *missionExpertInstance;
    };

    typedef struct FeatureMenuTaskCallbackData_t {
	int taskId;
	MissionExpert *missionExpertInstance;
    };

    typedef struct StatusWindows_t {
	Widget popupShell;
	Widget titleBox;
	Widget textBox;
	Widget prevBtn;
	Widget nextBtn;
	Widget loadBtn;
	Widget cancelBtn;
	Widget *starBox;
	int textPos;
	bool isUp;
	int currentMissionId;
	int selectedMissionId;
    };

    typedef enum StatusWindowNavigation_t {
	STATUS_WINDOW_PREV,
	STATUS_WINDOW_NEXT
    };

protected:
    Display *MExpDisplay_;
    XtAppContext MExpAppContext_;
    CBRClient *cbrclient_;
    FILE *featureFile_;
    FILE *featureStatusFile_;
    FILE *resultFile_;
    Pixel MExpParentFg_;
    Pixel MExpParentBg_;
    Widget MExpParent_;
    Widget featureMenu_w_;
    Widget **featureMenuOption_w_;
    Widget *featureMenuTask_w_;
    Widget **globalFeaturesMenuOption_w_;
    Widget *globalFeaturesMenuTask_w_;
    StatusWindows_t statusWindows_;
    Pixmap starPixmap_;
    Pixmap emptyStarPixmap_;
    MExpMissionTask_t featureMenuUpTask_;
    MExpRetrievedMissionSummary_t retrievedMissionSummary_;
    vector<MExpFeature_t> defaultGlobalFeatures_;
    vector<MExpFeature_t> featureMenuUpGlobalFeatures_;
    vector<MExpFeature_t> fixedGlobalFeatures_;
    vector<MExpFeature_t> defaultLocalFeatures_;
    vector<MExpMissionTask_t> defaultTaskList_;
    vector<string> createdDataFileList_;
    vector<int> currentCBRLibraryDataIndexList_;
    string featureFileString_;
    string featureFileName_;
    string featureStatusFileName_;
    string resultFileName_;
    string missionPlanFileName_;
    string robotName_;
    string overlayFileName_;
    string dataFilesDirectory_;
    string defaultTaskName_;
    int numTaskTypes_;
    bool disableMExp_;
    bool dataFilesNamed_;
    bool CBRClientInstantiated_;
    bool generalYesNoWindowResult_;
    bool generalYesNoWindowDone_;
    bool keepDataFiles_;
    bool featuresLoaded_;
    bool featureMenuIsUp_;
    bool shouldSaveMissionToCBRLibrary_;
    bool missionSavedToCBRLibrary_;
    bool missionLoadedFromCBRLibrary_;
    double defaultTaskWeight_;

    static const int DEFAULT_FEATUREMENU_WIDTH_;
    static const int DEFAULT_FEATUREMENU_LINEHEIGHT_;
    static const int DEFAULT_STATUSWINDOW_WIDTH_;
    static const int DEFAULT_STATUSWINDOW_LINEHEIGHT_;
    static const int DEFAULT_STATUSWINDOW_TOPTITLE_HEIGHT_;
    static const int DEFAULT_STATUSWINDOW_TEXT_HEIGHT_;
    static const int DEFAULT_STATUSWINDOW_TEXT_NUMROWS_;
    static const int MAX_SUITABILITY_RATING_;
    static const int EVENT_WAITER_SLEEP_TIME_USEC_;
    static const int MAX_NOWAIT_EVENTCOUNT_;
    static const string EMPTY_STRING_;
    static const string RCTABLE_DISABLE_STRING_;
    static const string RCTABLE_DATAFILESDIR_STRING_;
    static const string DEFAULT_FEATUREMENU_TITLE_;
    static const string DEFAULT_FEATUREMENU_TASK_LABEL_;
    static const string DEFAULT_FEATUREMENU_SAVE_BUTTON_LABEL_;
    static const string DEFAULT_FEATUREMENU_CANCEL_BUTTON_LABEL_;
    static const string DEFAULT_STATUSWINDOW_TITLE_;
    static const string DEFAULT_STATUSWINDOW_RATING_LABEL_;
    static const string DEFAULT_STATUSWINDOW_NAVIGATION_LABEL_;
    static const string DEFAULT_STATUSWINDOW_PREV_BUTTON_LABEL_;
    static const string DEFAULT_STATUSWINDOW_NEXT_BUTTON_LABEL_;
    static const string DEFAULT_STATUSWINDOW_LOAD_BUTTON_LABEL_;
    static const string DEFAULT_STATUSWINDOW_CANCEL_BUTTON_LABEL_;
    static const string ACDL_STRING_AGENT_NAME_;
    static const string ACDL_STRING_TASK_INDEX_;
    static const string ACDL_STRING_START_;
    static const string CDL_STRING_MAXVELOCITY_;
    static const string CDL_STRING_BASEVELOCITY_;
    static const string CDL_STRING_START_SUB_MISSION_;
    static const string FEATURE_NAME_NUMROBOTS_;
    static const string FEATURE_NAME_MAXVELOCITY_;
    static const string FEATURE_NAME_AGGRESSIVENESS_;
    
    Widget createGeneralYesNoWindow_(char *question);
    void popupGeneralYesNoWindow_(Widget w);
    void sendGeneralYesNoWindowResult_(bool result);
    bool confirmNewMission_();
    void nameDataFiles_(void);
    bool initCBRClient_(void);
    void runMlabMissionDesign_(void);
    void runCBRMExp_(void);
    void readCBRMExpResult_(void);
    bool buildMission_(void);
    MExpRetrievedMissionSummary_t summarizeRetrievedMission_(
	const vector<MExpRetrievedMission_t> retrievedMissions);
    void createTemporaryTaskAndGlobalFeatures_(void);
    void popupFeatureMenu_(bool askTaskType);
    void popupStatusWindow_(
	int rating,
	string missionSummary,
	bool prevBtnIsSensitive,
	bool nextBtnIsSensitive,
	bool loadBtnIsSensitive,
	bool cancelBtnIsSensitive,
	int currentMissionId);
    bool updateStatusWindow_(
	int rating,
	string missionSummary,
	bool prevBtnIsSensitive,
	bool nextBtnIsSensitive,
	bool loadBtnIsSensitive,
	bool cancelBtnIsSensitive,
	int currentMissionId);
    void selectMissionForStatusWindow_(void);
    void closeStatusWindow_(void);
    void changeStatusWindow_(int direction);
    bool loadRetrievedMissionChosenByUser_(
	vector<MExpRetrievedMission_t> retrievedMissions,
	string overlayFileName);
    void changeFeatureMenuValue_(XtPointer client_data);
    void changeGlobalFeatureMenuValue_(XtPointer client_data);
    void changeFeatureMenuTaskValue_(int taskId);
    void applyFeatureMenu_(void);
    void closeFeatureMenu_(void);
    bool checkFeatureStatusFile_(void);
    void deleteCreatedDataFiles_(void);
    void addCreatedDataFileList_(string filename);
    string createPrefix_(void);
    config_fsa_lists_t getFSALists_(void);
    bool loadFeaturesFromRCFile_(void);
    void createPixmaps_(Pixel fg, Pixel bg, Widget parent);
    bool hasAgentInMissionPlan_(string agentName);

    // Callbacks
    static void cbFeatureMenuTaskValue_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbChangeGlobalFeatureMenuValue_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbFeatureMenuValue_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbApplyFeatureMenu_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbCloseFeatureMenu_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbGeneralYesNoWindow_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbPrevBtnStatusWindow_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbNextBtnStatusWindow_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbLoadBtnStatusWindow_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbCancelBtnStatusWindow_(Widget w, XtPointer client_data, XtPointer callback_data);

public:
    MissionExpert(
	Display *display,
	Widget parent,
	XtAppContext app,
	const symbol_table<rc_chain> &rc_table,
	Widget menuMExpCascade);
    ~MissionExpert(void);
    void initialize(void);
    bool askUseMissionExpert(void);
    void confirmSuccessfulMission(void);
    void runMissionExpert(int mode);
    string getOverlayFileName(void);
    string getFeatureFileString(void);
    void saveOverlayFileName(string overlayFileName);
    void saveSelectionToCBRLibrary(Symbol * agent);
    bool isDisabled(void);
    bool isEnabled(void);
    bool shouldSaveMissionToCBRLibrary(void);
    void madeChange(void);
    bool shouldAskFeedback(void);
};

inline bool MissionExpert::isDisabled(void)
{
    return disableMExp_;
};

inline bool MissionExpert::isEnabled(void)
{
    return (!disableMExp_);
};

inline bool MissionExpert::shouldSaveMissionToCBRLibrary(void)
{
    return shouldSaveMissionToCBRLibrary_;
};

inline void MissionExpert::selectMissionForStatusWindow_(void)
{
    statusWindows_.selectedMissionId =  statusWindows_.currentMissionId;
};

extern MissionExpert *gMExp;

#endif

/**********************************************************************
 * $Log: mission_expert.h,v $
 * Revision 1.3  2003/04/06 12:25:34  endo
 * gcc 3.1.1
 *
 * Revision 1.2  2003/04/06 08:50:37  endo
 * Updated for CBR Wizard Prototype II.
 *
 * Revision 1.1  2002/01/12 23:09:58  endo
 * Initial revision
 *
 **********************************************************************/
