/**************************************************
*
* This file ALERTED_ONCE.cc was created with the command
* "/net/hr1/robot/mission-RCS-rh-6.0/bin/cnl -c -I../include -I/net/hr1/robot/mission-RCS-rh-6.0/src/ipt/include ALERTED_ONCE.cnl"
* using the CNL compiler, version 4.0.
*
**************************************************/


extern "C"
{
   #include <cthread.h>
   #include "../include/ipt/ipt.h"
}
   #include <string.h>

#ifndef NULL
#define NULL 0
#endif
#define CNL_FALSE (0)
#define CNL_TRUE (!CNL_FALSE)
#define _USE(var) static void * use_##var = (void *)&var
#define SRC_TYPE_LOCAL_NODE 	(0)
#define SRC_TYPE_REMOTE 	(1)
#define SRC_TYPE_CONSTANT 	(2)
#define require_input(name) parms->##name##require = true
#define require_input_list_entry(name,index) parms->##name##_require[##index##] = true

extern int 		_num_nodes;
extern int 		_done_count;
extern mutex_t 		_done_count_lock;
extern condition_t	_new_cycle;
extern char  	       *_prefix;
extern char  	       *_ipt_home;
extern IPCommunicator  *_communicator;
extern IPConnection   **_ipt_modules;
extern int   	        _ipt_is_active;
extern double 		_zero_time;






/* Execution philosophy:
*
* Start of cycle
*      o Each node checks all of its users to see if its output will be used
*        if no user wants the output, then the node is blocked.  Void output
*        nodes never block.
*
*      o Nodes which are not blocked then execute, begining with the sensors.
*
* End of cycle
*/

/*************************************************************************/
// Task Control Block class externs
template<class T> class _list
{
   struct SLink
   {
      struct SLink *next;
      T             data;

      // constructor: create objects
      SLink(const T val)
      {
         data = val;
      }

      // destructor: needed so don't delete the data objects
      ~SLink()
      {
	 // Nothing to do, here to keep from deleting the data
      }
   };
   SLink *last_ptr;


public:
   // constructor: create objects
   _list()
   {
      last_ptr = NULL;
   }

   // Is list empty?
   int isempty(void) const
   {
      return last_ptr == NULL;
   }

   // Append to end of list
   // Note: last_ptr->next is first record
   void append(const T data)
   {
      SLink *rec = new SLink(data);
      if( last_ptr )
      {
	 rec->next = last_ptr->next;
	 last_ptr->next = rec;
      }
      else
      {
	 rec->next = rec;
      }
      last_ptr = rec;
   }

   // Return first element without removing it
   // Note: last_ptr->next is first record
   // Returns NULL if list is empty
   void *first(T *rec) const
   {
      if( this == NULL || last_ptr == NULL )
	 return NULL;
      
      *rec = last_ptr->next->data;

      return (void *)last_ptr->next;
   }

   // Return next element without removing it
   // Note: must call first to setup the iterator
   // Returns NULL if list is empty
   void *next(T *rec, void *cur) const
   {
      if( (SLink *)cur == last_ptr )
	 return NULL;
      
      *rec = ((SLink *)cur)->next->data;
      return (void *)((SLink *)cur)->next;
   }
};

enum _STAT_TYPE {STAT_UNKNOWN, STAT_BLOCKED, STAT_RUNNING};
class tcb
{ 
public:
   enum _STAT_TYPE status_;
   _list<tcb *>    users_;
   const char     *task_name;
   bool            output_valid_;
   bool            done_;

   void update_status(void *port);
   virtual bool will_use_port(void *port) = 0;
   virtual void update_run_status() = 0;
   void reset_status();
   bool blocked();
   bool done();
   void block_me();
   void set_name(const char *name);
};




void reset_all_tasks();
void update_all_tasks();
extern _list<tcb *> list_of_tasks;

#line 1 "cnl.inc"
#line 9 "cnl.inc"

extern "C" {     
   #include <stdio.h>
   #include <stdlib.h>
   #include <math.h>
}

#include "gt_simulation.h"
#include "cnl.h"
#include "gt_std_types.h"
#include "gt_std.h"
#include "FSA_status.h"
#line 21 "cnl.inc"
#line 180 "ALERTED_ONCE.cc"

struct T_IS_LOOKING_AT_DEGREE : public tcb
{
   char *node_name;
   double *delta_psi;
   int delta_psi_type;
   int delta_psi_module;
   char *delta_psi_src;
   tcb *delta_psi_tcb;

   double *desired_psi;
   int desired_psi_type;
   int desired_psi_module;
   char *desired_psi_src;
   tcb *desired_psi_tcb;

   double *delta_theta;
   int delta_theta_type;
   int delta_theta_module;
   char *delta_theta_src;
   tcb *delta_theta_tcb;

   double *desired_theta;
   int desired_theta_type;
   int desired_theta_module;
   char *desired_theta_src;
   tcb *desired_theta_tcb;

   int *lookfor;
   int lookfor_type;
   int lookfor_module;
   char *lookfor_src;
   tcb *lookfor_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_IS_LOOKING_AT_DEGREE();
};

struct T_LOOK_TO : public tcb
{
   char *node_name;
   double *look_psi;
   int look_psi_type;
   int look_psi_module;
   char *look_psi_src;
   tcb *look_psi_tcb;

   double *look_theta;
   int look_theta_type;
   int look_theta_module;
   char *look_theta_src;
   tcb *look_theta_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_LOOK_TO();
};

struct T_PANTILT : public tcb
{
   char *node_name;
   Vector *look_loc;
   int look_loc_type;
   int look_loc_module;
   char *look_loc_src;
   tcb *look_loc_tcb;

   void update_run_status()
   {
      update_status(NULL);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(NULL);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_PANTILT();
};

struct T_SUM : public tcb
{
   char *node_name;
   int **values;
   int *values_type;
   int *values_module;
   char **values_src;
   tcb **values_tcb;
   int values_num; /* number of entries in list */

   int output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SUM();
};

struct T_COUNT_OBJECTS : public tcb
{
   char *node_name;
   ObjectList *objlist;
   int objlist_type;
   int objlist_module;
   char *objlist_src;
   tcb *objlist_tcb;

   int output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_COUNT_OBJECTS();
};

struct T_DEBUG_SHOW_INPUT_PRY : public tcb
{
   char *node_name;
   type_pry *input;
   int input_type;
   int input_module;
   char *input_src;
   tcb *input_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DEBUG_SHOW_INPUT_PRY();
};

struct T_SCALE_VECTOR : public tcb
{
   char *node_name;
   double *multiplier;
   int multiplier_type;
   int multiplier_module;
   char *multiplier_src;
   tcb *multiplier_tcb;

   Vector *v;
   int v_type;
   int v_module;
   char *v_src;
   tcb *v_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SCALE_VECTOR();
};

struct T_GET_UPHILL : public tcb
{
   char *node_name;
   type_pry *pry;
   int pry_type;
   int pry_module;
   char *pry_src;
   tcb *pry_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_UPHILL();
};

struct T_GET_YAW : public tcb
{
   char *node_name;
   type_pry *pry;
   int pry_type;
   int pry_module;
   char *pry_src;
   tcb *pry_tcb;

   double output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_YAW();
};

struct T_GET_PRY : public tcb
{
   char *node_name;
   type_pry output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_PRY();
};

struct T_DROP_OBJECT : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DROP_OBJECT();
};

struct T_EQUAL_INT : public tcb
{
   char *node_name;
   int *b;
   int b_type;
   int b_module;
   char *b_src;
   tcb *b_tcb;

   int *a;
   int a_type;
   int a_module;
   char *a_src;
   tcb *a_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_EQUAL_INT();
};

struct T_MAP_LOCATION : public tcb
{
   char *node_name;
   string *feature_name;
   int feature_name_type;
   int feature_name_module;
   char *feature_name_src;
   tcb *feature_name_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MAP_LOCATION();
};

struct T_IS_VALID_OBJECT : public tcb
{
   char *node_name;
   ObjectReading *object;
   int object_type;
   int object_module;
   char *object_src;
   tcb *object_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_IS_VALID_OBJECT();
};

struct T_ADD_OBJECT : public tcb
{
   char *node_name;
   double *diameter;
   int diameter_type;
   int diameter_module;
   char *diameter_src;
   tcb *diameter_tcb;

   Vector *place;
   int place_type;
   int place_module;
   char *place_src;
   tcb *place_tcb;

   int *style;
   int style_type;
   int style_module;
   char *style_src;
   tcb *style_tcb;

   string *color;
   int color_type;
   int color_module;
   char *color_src;
   tcb *color_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_ADD_OBJECT();
};

struct T_PROBE_OBJECT : public tcb
{
   char *node_name;
   ObjectReading *object;
   int object_type;
   int object_module;
   char *object_src;
   tcb *object_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_PROBE_OBJECT();
};

struct T_PROBE_MOVABLE : public tcb
{
   char *node_name;
   ObjectReading *object;
   int object_type;
   int object_module;
   char *object_src;
   tcb *object_tcb;

   boolean output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_PROBE_MOVABLE();
};

struct T_COMPUTE_UNIT_CENTER : public tcb
{
   char *node_name;
   ObjectList *robots;
   int robots_type;
   int robots_module;
   char *robots_src;
   tcb *robots_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_COMPUTE_UNIT_CENTER();
};

struct T_ECHO_OBJECTS : public tcb
{
   char *node_name;
   ObjectList *full_list;
   int full_list_type;
   int full_list_module;
   char *full_list_src;
   tcb *full_list_tcb;

   ObjectList output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_ECHO_OBJECTS();
};

struct T_FILTER_OBJECTS_BY_CLASS : public tcb
{
   char *node_name;
   ObjectClasses *classes;
   int classes_type;
   int classes_module;
   char *classes_src;
   tcb *classes_tcb;

   bool *remove_these;
   int remove_these_type;
   int remove_these_module;
   char *remove_these_src;
   tcb *remove_these_tcb;

   ObjectList *full_list;
   int full_list_type;
   int full_list_module;
   char *full_list_src;
   tcb *full_list_tcb;

   ObjectList output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_FILTER_OBJECTS_BY_CLASS();
};

struct T_SETXYTHETA : public tcb
{
   char *node_name;
   double *new_theta;
   int new_theta_type;
   int new_theta_module;
   char *new_theta_src;
   tcb *new_theta_tcb;

   Vector *new_robot_location;
   int new_robot_location_type;
   int new_robot_location_module;
   char *new_robot_location_src;
   tcb *new_robot_location_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SETXYTHETA();
};

struct T_SETXY : public tcb
{
   char *node_name;
   double *y;
   int y_type;
   int y_module;
   char *y_src;
   tcb *y_tcb;

   double *x;
   int x_type;
   int x_module;
   char *x_src;
   tcb *x_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SETXY();
};

struct T_SPIN : public tcb
{
   char *node_name;
   double *slippageFactor;
   int slippageFactor_type;
   int slippageFactor_module;
   char *slippageFactor_src;
   tcb *slippageFactor_tcb;

   double *angular_velocity;
   int angular_velocity_type;
   int angular_velocity_module;
   char *angular_velocity_src;
   tcb *angular_velocity_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SPIN();
};

struct T_MOVED_DISTANCE : public tcb
{
   char *node_name;
   robot_position *position;
   int position_type;
   int position_module;
   char *position_src;
   tcb *position_tcb;

   double *desired_distance;
   int desired_distance_type;
   int desired_distance_module;
   char *desired_distance_src;
   tcb *desired_distance_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MOVED_DISTANCE();
};

struct T_HAS_TURNED : public tcb
{
   char *node_name;
   double *off_set;
   int off_set_type;
   int off_set_module;
   char *off_set_src;
   tcb *off_set_tcb;

   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   double *desired_turn_angle;
   int desired_turn_angle_type;
   int desired_turn_angle_module;
   char *desired_turn_angle_src;
   tcb *desired_turn_angle_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_HAS_TURNED();
};

struct T_ISFACING : public tcb
{
   char *node_name;
   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   double *accuracy;
   int accuracy_type;
   int accuracy_module;
   char *accuracy_src;
   tcb *accuracy_tcb;

   double *desired_heading;
   int desired_heading_type;
   int desired_heading_module;
   char *desired_heading_src;
   tcb *desired_heading_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_ISFACING();
};

struct T_WAIT : public tcb
{
   char *node_name;
   double *delay_time;
   int delay_time_type;
   int delay_time_module;
   char *delay_time_src;
   tcb *delay_time_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_WAIT();
};

struct T_MARK_OBJECT : public tcb
{
   char *node_name;
   OneObjectClass *new_class;
   int new_class_type;
   int new_class_module;
   char *new_class_src;
   tcb *new_class_tcb;

   ObjectReading *object;
   int object_type;
   int object_module;
   char *object_src;
   tcb *object_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MARK_OBJECT();
};

struct T_MOVE_TO_FREE_SPACE : public tcb
{
   char *node_name;
   obs_array *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   double *horizon;
   int horizon_type;
   int horizon_module;
   char *horizon_src;
   tcb *horizon_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MOVE_TO_FREE_SPACE();
};

struct T_FIRST_TIME : public tcb
{
   char *node_name;
   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_FIRST_TIME();
};

struct T_DROP_IN_BASKET : public tcb
{
   char *node_name;
   ObjectReading *basket;
   int basket_type;
   int basket_module;
   char *basket_src;
   tcb *basket_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DROP_IN_BASKET();
};

struct T_NOOP : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_NOOP();
};

struct T_NULL_MS : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_NULL_MS();
};

struct T_HALT : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_HALT();
};

struct T_DEBUG_SHOW_INPUT : public tcb
{
   char *node_name;
   Vector *input;
   int input_type;
   int input_module;
   char *input_src;
   tcb *input_tcb;

   string *msg;
   int msg_type;
   int msg_module;
   char *msg_src;
   tcb *msg_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DEBUG_SHOW_INPUT();
};

struct T_DEBUG_PROC : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DEBUG_PROC();
};

struct T_CONSTANT : public tcb
{
   char *node_name;
   bool *value;
   int value_type;
   int value_module;
   char *value_src;
   tcb *value_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_CONSTANT();
};

struct T_AVOID_OBSTACLES_MEM : public tcb
{
   char *node_name;
   robot_position *cur_pos;
   int cur_pos_type;
   int cur_pos_module;
   char *cur_pos_src;
   tcb *cur_pos_tcb;

   obs_array *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   double *decay_rate;
   int decay_rate_type;
   int decay_rate_module;
   char *decay_rate_src;
   tcb *decay_rate_tcb;

   double *safety_margin;
   int safety_margin_type;
   int safety_margin_module;
   char *safety_margin_src;
   tcb *safety_margin_tcb;

   double *sphere;
   int sphere_type;
   int sphere_module;
   char *sphere_src;
   tcb *sphere_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_AVOID_OBSTACLES_MEM();
};

struct T_AVOID_OBSTACLES : public tcb
{
   char *node_name;
   obs_array *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   double *safety_margin;
   int safety_margin_type;
   int safety_margin_module;
   char *safety_margin_src;
   tcb *safety_margin_tcb;

   double *sphere;
   int sphere_type;
   int sphere_module;
   char *sphere_src;
   tcb *sphere_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_AVOID_OBSTACLES();
};

struct T_AVOID_OBJECTS : public tcb
{
   char *node_name;
   ObjectList *objlist;
   int objlist_type;
   int objlist_module;
   char *objlist_src;
   tcb *objlist_tcb;

   double *safety_margin;
   int safety_margin_type;
   int safety_margin_module;
   char *safety_margin_src;
   tcb *safety_margin_tcb;

   double *sphere;
   int sphere_type;
   int sphere_module;
   char *sphere_src;
   tcb *sphere_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_AVOID_OBJECTS();
};

struct T_OBJECT_LOCATION : public tcb
{
   char *node_name;
   ObjectReading *object;
   int object_type;
   int object_module;
   char *object_src;
   tcb *object_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_OBJECT_LOCATION();
};

struct T_PICKUP_OBJECT : public tcb
{
   char *node_name;
   ObjectReading *object;
   int object_type;
   int object_module;
   char *object_src;
   tcb *object_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_PICKUP_OBJECT();
};

struct T_IS_AT_GOAL : public tcb
{
   char *node_name;
   double *success_radius;
   int success_radius_type;
   int success_radius_module;
   char *success_radius_src;
   tcb *success_radius_tcb;

   Vector *goal_rel_loc;
   int goal_rel_loc_type;
   int goal_rel_loc_module;
   char *goal_rel_loc_src;
   tcb *goal_rel_loc_tcb;

   bool *have_a_goal;
   int have_a_goal_type;
   int have_a_goal_module;
   char *have_a_goal_src;
   tcb *have_a_goal_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_IS_AT_GOAL();
};

struct T_CLOSEST_OBSTACLE : public tcb
{
   char *node_name;
   obs_array *obstacle_list;
   int obstacle_list_type;
   int obstacle_list_module;
   char *obstacle_list_src;
   tcb *obstacle_list_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_CLOSEST_OBSTACLE();
};

struct T_CLOSEST_OBJECT : public tcb
{
   char *node_name;
   ObjectList *object_list;
   int object_list_type;
   int object_list_module;
   char *object_list_src;
   tcb *object_list_tcb;

   ObjectReading output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_CLOSEST_OBJECT();
};

struct T_IS_AN_OBJECT_MOTIVATED : public tcb
{
   char *node_name;
   double *curiousity_level;
   int curiousity_level_type;
   int curiousity_level_module;
   char *curiousity_level_src;
   tcb *curiousity_level_tcb;

   double *hunger_level;
   int hunger_level_type;
   int hunger_level_module;
   char *hunger_level_src;
   tcb *hunger_level_tcb;

   double *fear_level;
   int fear_level_type;
   int fear_level_module;
   char *fear_level_src;
   tcb *fear_level_tcb;

   double *anger_level;
   int anger_level_type;
   int anger_level_module;
   char *anger_level_src;
   tcb *anger_level_tcb;

   double *curiousity_upper;
   int curiousity_upper_type;
   int curiousity_upper_module;
   char *curiousity_upper_src;
   tcb *curiousity_upper_tcb;

   double *curiousity_lower;
   int curiousity_lower_type;
   int curiousity_lower_module;
   char *curiousity_lower_src;
   tcb *curiousity_lower_tcb;

   double *hunger_upper;
   int hunger_upper_type;
   int hunger_upper_module;
   char *hunger_upper_src;
   tcb *hunger_upper_tcb;

   double *hunger_lower;
   int hunger_lower_type;
   int hunger_lower_module;
   char *hunger_lower_src;
   tcb *hunger_lower_tcb;

   double *fear_upper;
   int fear_upper_type;
   int fear_upper_module;
   char *fear_upper_src;
   tcb *fear_upper_tcb;

   double *fear_lower;
   int fear_lower_type;
   int fear_lower_module;
   char *fear_lower_src;
   tcb *fear_lower_tcb;

   double *anger_upper;
   int anger_upper_type;
   int anger_upper_module;
   char *anger_upper_src;
   tcb *anger_upper_tcb;

   double *anger_lower;
   int anger_lower_type;
   int anger_lower_module;
   char *anger_lower_src;
   tcb *anger_lower_tcb;

   ObjectList *object_list;
   int object_list_type;
   int object_list_module;
   char *object_list_src;
   tcb *object_list_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_IS_AN_OBJECT_MOTIVATED();
};

struct T_IS_AN_OBJECT : public tcb
{
   char *node_name;
   ObjectList *object_list;
   int object_list_type;
   int object_list_module;
   char *object_list_src;
   tcb *object_list_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_IS_AN_OBJECT();
};

struct T_FILTER_OBJECTS : public tcb
{
   char *node_name;
   ObjectList *full_list;
   int full_list_type;
   int full_list_module;
   char *full_list_src;
   tcb *full_list_tcb;

   string *desired_color;
   int desired_color_type;
   int desired_color_module;
   char *desired_color_src;
   tcb *desired_color_tcb;

   ObjectList output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_FILTER_OBJECTS();
};

struct T_FILTER_OBJECT_CLASSES : public tcb
{
   char *node_name;
   ObjectList *full_list;
   int full_list_type;
   int full_list_module;
   char *full_list_src;
   tcb *full_list_tcb;

   string *desired_color;
   int desired_color_type;
   int desired_color_module;
   char *desired_color_src;
   tcb *desired_color_tcb;

   ObjectList output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_FILTER_OBJECT_CLASSES();
};

struct T_DETECT_OBJECTS : public tcb
{
   char *node_name;
   double *max_sensor_range;
   int max_sensor_range_type;
   int max_sensor_range_module;
   char *max_sensor_range_src;
   tcb *max_sensor_range_tcb;

   ObjectList output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_OBJECTS();
};

struct T_EXTRACT_GOAL : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_EXTRACT_GOAL();
};

struct T_TALK : public tcb
{
   char *node_name;
   string *talkmessage;
   int talkmessage_type;
   int talkmessage_module;
   char *talkmessage_src;
   tcb *talkmessage_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_TALK();
};

struct T_DETECT_SOUND_MOTIVATED : public tcb
{
   char *node_name;
   double *curiousity_level;
   int curiousity_level_type;
   int curiousity_level_module;
   char *curiousity_level_src;
   tcb *curiousity_level_tcb;

   double *curiousity_threshold;
   int curiousity_threshold_type;
   int curiousity_threshold_module;
   char *curiousity_threshold_src;
   tcb *curiousity_threshold_tcb;

   double *volume_threshold;
   int volume_threshold_type;
   int volume_threshold_module;
   char *volume_threshold_src;
   tcb *volume_threshold_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_SOUND_MOTIVATED();
};

struct T_DETECT_SOUND : public tcb
{
   char *node_name;
   double *volume_threshold;
   int volume_threshold_type;
   int volume_threshold_module;
   char *volume_threshold_src;
   tcb *volume_threshold_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_SOUND();
};

struct T_SOUND : public tcb
{
   char *node_name;
   double *sound_direction_y;
   int sound_direction_y_type;
   int sound_direction_y_module;
   char *sound_direction_y_src;
   tcb *sound_direction_y_tcb;

   double *sound_direction_x;
   int sound_direction_x_type;
   int sound_direction_x_module;
   char *sound_direction_x_src;
   tcb *sound_direction_x_tcb;

   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SOUND();
};

struct T_TELOP : public tcb
{
   char *node_name;
   double *joystick_y;
   int joystick_y_type;
   int joystick_y_module;
   char *joystick_y_src;
   tcb *joystick_y_tcb;

   double *joystick_x;
   int joystick_x_type;
   int joystick_x_module;
   char *joystick_x_src;
   tcb *joystick_x_tcb;

   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_TELOP();
};

struct T_NOT : public tcb
{
   char *node_name;
   bool *a;
   int a_type;
   int a_module;
   char *a_src;
   tcb *a_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_NOT();
};

struct T_AND : public tcb
{
   char *node_name;
   bool *b;
   int b_type;
   int b_module;
   char *b_src;
   tcb *b_tcb;

   bool *a;
   int a_type;
   int a_module;
   char *a_src;
   tcb *a_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_AND();
};

struct T_MDL_coordinator : public tcb
{
   char *node_name;
   bool *occupy_done;
   int occupy_done_type;
   int occupy_done_module;
   char *occupy_done_src;
   tcb *occupy_done_tcb;

   bool *teleop_done;
   int teleop_done_type;
   int teleop_done_module;
   char *teleop_done_src;
   tcb *teleop_done_tcb;

   bool *moveto_done;
   int moveto_done_type;
   int moveto_done_module;
   char *moveto_done_src;
   tcb *moveto_done_tcb;

   Vector *occupy;
   int occupy_type;
   int occupy_module;
   char *occupy_src;
   tcb *occupy_tcb;

   Vector *teleop;
   int teleop_type;
   int teleop_module;
   char *teleop_src;
   tcb *teleop_tcb;

   Vector *moveto;
   int moveto_type;
   int moveto_module;
   char *moveto_src;
   tcb *moveto_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MDL_coordinator();
};

struct T_SUBTRACT_VECTORS : public tcb
{
   char *node_name;
   Vector *B;
   int B_type;
   int B_module;
   char *B_src;
   tcb *B_tcb;

   Vector *A;
   int A_type;
   int A_module;
   char *A_src;
   tcb *A_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SUBTRACT_VECTORS();
};

struct T_MAINTAIN_FORM : public tcb
{
   char *node_name;
   double *saturation_length;
   int saturation_length_type;
   int saturation_length_module;
   char *saturation_length_src;
   tcb *saturation_length_tcb;

   double *dead_zone_radius;
   int dead_zone_radius_type;
   int dead_zone_radius_module;
   char *dead_zone_radius_src;
   tcb *dead_zone_radius_tcb;

   Vector *offset;
   int offset_type;
   int offset_module;
   char *offset_src;
   tcb *offset_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MAINTAIN_FORM();
};

struct T_REMOVE_ROBOTS : public tcb
{
   char *node_name;
   Robots *robots;
   int robots_type;
   int robots_module;
   char *robots_src;
   tcb *robots_tcb;

   obs_array *mixed;
   int mixed_type;
   int mixed_module;
   char *mixed_src;
   tcb *mixed_tcb;

   obs_array output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_REMOVE_ROBOTS();
};

struct T_SWIRL_STATIC_OBSTACLES : public tcb
{
   char *node_name;
   Vector *goal_rel_loc;
   int goal_rel_loc_type;
   int goal_rel_loc_module;
   char *goal_rel_loc_src;
   tcb *goal_rel_loc_tcb;

   obs_array *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   double *open_sphere;
   int open_sphere_type;
   int open_sphere_module;
   char *open_sphere_src;
   tcb *open_sphere_tcb;

   double *open_space;
   int open_space_type;
   int open_space_module;
   char *open_space_src;
   tcb *open_space_tcb;

   double *safety_margin;
   int safety_margin_type;
   int safety_margin_module;
   char *safety_margin_src;
   tcb *safety_margin_tcb;

   double *sphere;
   int sphere_type;
   int sphere_module;
   char *sphere_src;
   tcb *sphere_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SWIRL_STATIC_OBSTACLES();
};

struct T_UPDATE_DISTRIBUTED_FLAG : public tcb
{
   char *node_name;
   bool *value;
   int value_type;
   int value_module;
   char *value_src;
   tcb *value_tcb;

   string *key;
   int key_type;
   int key_module;
   char *key_src;
   tcb *key_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_UPDATE_DISTRIBUTED_FLAG();
};

struct T_TEST_DISTRIBUTED_FLAG : public tcb
{
   char *node_name;
   bool *default_value;
   int default_value_type;
   int default_value_module;
   char *default_value_src;
   tcb *default_value_tcb;

   string *key;
   int key_type;
   int key_module;
   char *key_src;
   tcb *key_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_TEST_DISTRIBUTED_FLAG();
};

struct T_DATABASE_BOOLEAN : public tcb
{
   char *node_name;
   bool *initial;
   int initial_type;
   int initial_module;
   char *initial_src;
   tcb *initial_tcb;

   string *key;
   int key_type;
   int key_module;
   char *key_src;
   tcb *key_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DATABASE_BOOLEAN();
};

struct T_DATABASE_INT : public tcb
{
   char *node_name;
   int *initial;
   int initial_type;
   int initial_module;
   char *initial_src;
   tcb *initial_tcb;

   string *key;
   int key_type;
   int key_module;
   char *key_src;
   tcb *key_tcb;

   int output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DATABASE_INT();
};

struct T_DATABASE_DOUBLE : public tcb
{
   char *node_name;
   double *initial;
   int initial_type;
   int initial_module;
   char *initial_src;
   tcb *initial_tcb;

   string *key;
   int key_type;
   int key_module;
   char *key_src;
   tcb *key_tcb;

   double output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DATABASE_DOUBLE();
};

struct T_DETECT_UNIT_CENTER_GLOBAL : public tcb
{
   char *node_name;
   Vector *own_pos;
   int own_pos_type;
   int own_pos_module;
   char *own_pos_src;
   tcb *own_pos_tcb;

   Robots *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_UNIT_CENTER_GLOBAL();
};

struct T_DETECT_UNIT_CENTER : public tcb
{
   char *node_name;
   gt_Unit_Members *members;
   int members_type;
   int members_module;
   char *members_src;
   tcb *members_tcb;

   Robots *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_UNIT_CENTER();
};

struct T_DETECT_LEADER_CENTER : public tcb
{
   char *node_name;
   gt_Unit_Members *members;
   int members_type;
   int members_module;
   char *members_src;
   tcb *members_tcb;

   Robots *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   gt_Formation *form_type;
   int form_type_type;
   int form_type_module;
   char *form_type_src;
   tcb *form_type_tcb;

   double *form_heading;
   int form_heading_type;
   int form_heading_module;
   char *form_heading_src;
   tcb *form_heading_tcb;

   double *spacing;
   int spacing_type;
   int spacing_module;
   char *spacing_src;
   tcb *spacing_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_LEADER_CENTER();
};

struct T_DETECT_IN_POSITION : public tcb
{
   char *node_name;
   double *slop;
   int slop_type;
   int slop_module;
   char *slop_src;
   tcb *slop_tcb;

   Vector *offset;
   int offset_type;
   int offset_module;
   char *offset_src;
   tcb *offset_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_IN_POSITION();
};

struct T_GET_HALLWAY_INFO_DOUBLE : public tcb
{
   char *node_name;
   string *key;
   int key_type;
   int key_module;
   char *key_src;
   tcb *key_tcb;

   double output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_HALLWAY_INFO_DOUBLE();
};

struct T_GET_HALLWAY_INFO_VECTOR : public tcb
{
   char *node_name;
   string *key;
   int key_type;
   int key_module;
   char *key_src;
   tcb *key_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_HALLWAY_INFO_VECTOR();
};

struct T_COMPUTE_FORM_POSITION : public tcb
{
   char *node_name;
   string *formation_name;
   int formation_name_type;
   int formation_name_module;
   char *formation_name_src;
   tcb *formation_name_tcb;

   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   Vector *robot_location;
   int robot_location_type;
   int robot_location_module;
   char *robot_location_src;
   tcb *robot_location_tcb;

   double *formation_heading;
   int formation_heading_type;
   int formation_heading_module;
   char *formation_heading_src;
   tcb *formation_heading_tcb;

   Vector *unit_center_relative;
   int unit_center_relative_type;
   int unit_center_relative_module;
   char *unit_center_relative_src;
   tcb *unit_center_relative_tcb;

   int *unit_size;
   int unit_size_type;
   int unit_size_module;
   char *unit_size_src;
   tcb *unit_size_tcb;

   double *spacing;
   int spacing_type;
   int spacing_module;
   char *spacing_src;
   tcb *spacing_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_COMPUTE_FORM_POSITION();
};

struct T_DETECT_FORM_POSITION : public tcb
{
   char *node_name;
   int *formation_ref;
   int formation_ref_type;
   int formation_ref_module;
   char *formation_ref_src;
   tcb *formation_ref_tcb;

   gt_Formation *formation_type;
   int formation_type_type;
   int formation_type_module;
   char *formation_type_src;
   tcb *formation_type_tcb;

   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   Vector *robot_pos;
   int robot_pos_type;
   int robot_pos_module;
   char *robot_pos_src;
   tcb *robot_pos_tcb;

   double *formation_heading;
   int formation_heading_type;
   int formation_heading_module;
   char *formation_heading_src;
   tcb *formation_heading_tcb;

   Vector *leader_center_relative;
   int leader_center_relative_type;
   int leader_center_relative_module;
   char *leader_center_relative_src;
   tcb *leader_center_relative_tcb;

   Vector *unit_center_relative;
   int unit_center_relative_type;
   int unit_center_relative_module;
   char *unit_center_relative_src;
   tcb *unit_center_relative_tcb;

   gt_Unit_Members *members;
   int members_type;
   int members_module;
   char *members_src;
   tcb *members_tcb;

   double *spacing;
   int spacing_type;
   int spacing_module;
   char *spacing_src;
   tcb *spacing_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_FORM_POSITION();
};

struct T_DETECT_FORM_HEADING : public tcb
{
   char *node_name;
   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   double *default_trigger;
   int default_trigger_type;
   int default_trigger_module;
   char *default_trigger_src;
   tcb *default_trigger_tcb;

   double *telop_mag;
   int telop_mag_type;
   int telop_mag_module;
   char *telop_mag_src;
   tcb *telop_mag_tcb;

   Vector *telop_input;
   int telop_input_type;
   int telop_input_module;
   char *telop_input_src;
   tcb *telop_input_tcb;

   Vector *relative_goal;
   int relative_goal_type;
   int relative_goal_module;
   char *relative_goal_src;
   tcb *relative_goal_tcb;

   double output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_FORM_HEADING();
};

struct T_DETECT_CENTER_OF_MASS : public tcb
{
   char *node_name;
   Robots *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_CENTER_OF_MASS();
};

struct T_AVOID_ROBOT : public tcb
{
   char *node_name;
   Robots *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   double *min_range;
   int min_range_type;
   int min_range_module;
   char *min_range_src;
   tcb *min_range_tcb;

   double *sphere;
   int sphere_type;
   int sphere_module;
   char *sphere_src;
   tcb *sphere_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_AVOID_ROBOT();
};

struct T_AVOID_STATIC_OBSTACLES : public tcb
{
   char *node_name;
   obs_array *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   double *safety_margin;
   int safety_margin_type;
   int safety_margin_module;
   char *safety_margin_src;
   tcb *safety_margin_tcb;

   double *sphere;
   int sphere_type;
   int sphere_module;
   char *sphere_src;
   tcb *sphere_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_AVOID_STATIC_OBSTACLES();
};

struct T_ULTRASONICS : public tcb
{
   char *node_name;
   double *heading;
   int heading_type;
   int heading_module;
   char *heading_src;
   tcb *heading_tcb;

   double *max_sensor_range;
   int max_sensor_range_type;
   int max_sensor_range_module;
   char *max_sensor_range_src;
   tcb *max_sensor_range_tcb;

   obs_array output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_ULTRASONICS();
};

struct T_DETECT_OBSTACLES : public tcb
{
   char *node_name;
   double *max_sensor_range;
   int max_sensor_range_type;
   int max_sensor_range_module;
   char *max_sensor_range_src;
   tcb *max_sensor_range_tcb;

   obs_array output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_OBSTACLES();
};

struct T_ALERTED_ONCE : public tcb
{
   char *node_name;
   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_ALERTED_ONCE();
};

struct T_ALERT : public tcb
{
   char *node_name;
   bool *attach_image;
   int attach_image_type;
   int attach_image_module;
   char *attach_image_src;
   tcb *attach_image_tcb;

   string *mail_recipient;
   int mail_recipient_type;
   int mail_recipient_module;
   char *mail_recipient_src;
   tcb *mail_recipient_tcb;

   bool *use_email;
   int use_email_type;
   int use_email_module;
   char *use_email_src;
   tcb *use_email_tcb;

   string *alertmessage;
   int alertmessage_type;
   int alertmessage_module;
   char *alertmessage_src;
   tcb *alertmessage_tcb;

   string *alertsubject;
   int alertsubject_type;
   int alertsubject_module;
   char *alertsubject_src;
   tcb *alertsubject_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_ALERT();
};

struct T_PROBE : public tcb
{
   char *node_name;
   obs_array *readings;
   int readings_type;
   int readings_module;
   char *readings_src;
   tcb *readings_tcb;

   double *safety_margin;
   int safety_margin_type;
   int safety_margin_module;
   char *safety_margin_src;
   tcb *safety_margin_tcb;

   double *sphere;
   int sphere_type;
   int sphere_module;
   char *sphere_src;
   tcb *sphere_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_PROBE();
};

struct T_LAST_KEY_PRESS : public tcb
{
   char *node_name;
   int output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_LAST_KEY_PRESS();
};

struct T_NOISE : public tcb
{
   char *node_name;
   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   int *persistence;
   int persistence_type;
   int persistence_module;
   char *persistence_src;
   tcb *persistence_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_NOISE();
};

struct T_DETECT_ROBOTS : public tcb
{
   char *node_name;
   double *max_dist;
   int max_dist_type;
   int max_dist_module;
   char *max_dist_src;
   tcb *max_dist_tcb;

   Robots output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_ROBOTS();
};

struct T_GET_LOCATION : public tcb
{
   char *node_name;
   robot_position output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_LOCATION();
};

struct T_GET_HEADING : public tcb
{
   char *node_name;
   robot_position *cur_pos;
   int cur_pos_type;
   int cur_pos_module;
   char *cur_pos_src;
   tcb *cur_pos_tcb;

   double output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_HEADING();
};

struct T_GET_XY : public tcb
{
   char *node_name;
   robot_position *cur_pos;
   int cur_pos_type;
   int cur_pos_module;
   char *cur_pos_src;
   tcb *cur_pos_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_XY();
};

struct T_COOP : public tcb
{
   char *node_name;
   double **weight;
   int *weight_type;
   int *weight_module;
   char **weight_src;
   tcb **weight_tcb;
   int weight_num; /* number of entries in list */

   Vector **members;
   int *members_type;
   int *members_module;
   char **members_src;
   tcb **members_tcb;
   int members_num; /* number of entries in list */

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_COOP();
};

struct T_SUM_WEIGHTED_VECTORS : public tcb
{
   char *node_name;
   string *name;
   int name_type;
   int name_module;
   char *name_src;
   tcb *name_tcb;

   double **gain;
   int *gain_type;
   int *gain_module;
   char **gain_src;
   tcb **gain_tcb;
   int gain_num; /* number of entries in list */

   Vector **v;
   int *v_type;
   int *v_module;
   char **v_src;
   tcb **v_tcb;
   int v_num; /* number of entries in list */

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SUM_WEIGHTED_VECTORS();
};

struct T_DOCK : public tcb
{
   char *node_name;
   double *controlled_gain;
   int controlled_gain_type;
   int controlled_gain_module;
   char *controlled_gain_src;
   tcb *controlled_gain_tcb;

   double *ballistic_gain;
   int ballistic_gain_type;
   int ballistic_gain_module;
   char *ballistic_gain_src;
   tcb *ballistic_gain_tcb;

   double *cone_angle;
   int cone_angle_type;
   int cone_angle_module;
   char *cone_angle_src;
   tcb *cone_angle_tcb;

   double *controlled_radius;
   int controlled_radius_type;
   int controlled_radius_module;
   char *controlled_radius_src;
   tcb *controlled_radius_tcb;

   double *dockxy;
   int dockxy_type;
   int dockxy_module;
   char *dockxy_src;
   tcb *dockxy_tcb;

   Vector *dockpos;
   int dockpos_type;
   int dockpos_module;
   char *dockpos_src;
   tcb *dockpos_tcb;

   robot_position *robotloc;
   int robotloc_type;
   int robotloc_module;
   char *robotloc_src;
   tcb *robotloc_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DOCK();
};

struct T_DETECT_ALTERNATE_HALLWAY : public tcb
{
   char *node_name;
   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_ALTERNATE_HALLWAY();
};

struct T_ENTER_ALTERNATE_HALLWAY : public tcb
{
   char *node_name;
   double *distance;
   int distance_type;
   int distance_module;
   char *distance_src;
   tcb *distance_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_ENTER_ALTERNATE_HALLWAY();
};

struct T_LEAVE_ROOM : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_LEAVE_ROOM();
};

struct T_ENTER_ROOM : public tcb
{
   char *node_name;
   int *unmarked;
   int unmarked_type;
   int unmarked_module;
   char *unmarked_src;
   tcb *unmarked_tcb;

   double *distance;
   int distance_type;
   int distance_module;
   char *distance_src;
   tcb *distance_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_ENTER_ROOM();
};

struct T_IN_HALLWAY : public tcb
{
   char *node_name;
   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_IN_HALLWAY();
};

struct T_IN_ROOM : public tcb
{
   char *node_name;
   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_IN_ROOM();
};

struct T_GLOBAL_TO_EGOCENTRIC : public tcb
{
   char *node_name;
   double *robot_heading;
   int robot_heading_type;
   int robot_heading_module;
   char *robot_heading_src;
   tcb *robot_heading_tcb;

   Vector *robot_location;
   int robot_location_type;
   int robot_location_module;
   char *robot_location_src;
   tcb *robot_location_tcb;

   Vector *global;
   int global_type;
   int global_module;
   char *global_src;
   tcb *global_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GLOBAL_TO_EGOCENTRIC();
};

struct T_PUT_ROBOT_STATE : public tcb
{
   char *node_name;
   int *state;
   int state_type;
   int state_module;
   char *state_src;
   tcb *state_tcb;

   void update_run_status()
   {
      update_status(NULL);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(NULL);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_PUT_ROBOT_STATE();
};

struct T_LAST_BUTTON_PRESS : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_LAST_BUTTON_PRESS();
};

struct T_V_IS_ZERO : public tcb
{
   char *node_name;
   Vector *v;
   int v_type;
   int v_module;
   char *v_src;
   tcb *v_tcb;

   int output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_V_IS_ZERO();
};

struct T_GET_ROBOT_STATE : public tcb
{
   char *node_name;
   int *robot_id;
   int robot_id_type;
   int robot_id_module;
   char *robot_id_src;
   tcb *robot_id_tcb;

   int output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_ROBOT_STATE();
};

struct T_DETECT_GOAL_WITH_SHAFT_ENCODERS : public tcb
{
   char *node_name;
   Vector *robot_loc;
   int robot_loc_type;
   int robot_loc_module;
   char *robot_loc_src;
   tcb *robot_loc_tcb;

   Vector *goal;
   int goal_type;
   int goal_module;
   char *goal_src;
   tcb *goal_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DETECT_GOAL_WITH_SHAFT_ENCODERS();
};

struct T_SHAFTENCODERS : public tcb
{
   char *node_name;
   robot_position output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_SHAFTENCODERS();
};

struct T_MOVE_TO : public tcb
{
   char *node_name;
   bool *have_a_goal;
   int have_a_goal_type;
   int have_a_goal_module;
   char *have_a_goal_src;
   tcb *have_a_goal_tcb;

   Vector *goal_rel_loc;
   int goal_rel_loc_type;
   int goal_rel_loc_module;
   char *goal_rel_loc_src;
   tcb *goal_rel_loc_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MOVE_TO();
};

struct T_MOVE_TO_GOAL : public tcb
{
   char *node_name;
   double *success_radius;
   int success_radius_type;
   int success_radius_module;
   char *success_radius_src;
   tcb *success_radius_tcb;

   Vector *goal_rel_loc;
   int goal_rel_loc_type;
   int goal_rel_loc_module;
   char *goal_rel_loc_src;
   tcb *goal_rel_loc_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MOVE_TO_GOAL();
};

struct T_MOVE_AHEAD : public tcb
{
   char *node_name;
   double *heading;
   int heading_type;
   int heading_module;
   char *heading_src;
   tcb *heading_tcb;

   double *amount;
   int amount_type;
   int amount_module;
   char *amount_src;
   tcb *amount_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MOVE_AHEAD();
};

struct T_MOVE_ROBOT : public tcb
{
   char *node_name;
   bool *cautious_mode;
   int cautious_mode_type;
   int cautious_mode_module;
   char *cautious_mode_src;
   tcb *cautious_mode_tcb;

   double *cautious_vel;
   int cautious_vel_type;
   int cautious_vel_module;
   char *cautious_vel_src;
   tcb *cautious_vel_tcb;

   double *base_vel;
   int base_vel_type;
   int base_vel_module;
   char *base_vel_src;
   tcb *base_vel_tcb;

   double *max_vel;
   int max_vel_type;
   int max_vel_module;
   char *max_vel_src;
   tcb *max_vel_tcb;

   Vector *v;
   int v_type;
   int v_module;
   char *v_src;
   tcb *v_tcb;

   void update_run_status()
   {
      update_status(NULL);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(NULL);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MOVE_ROBOT();
};

struct T_DRIVE_W_SPIN : public tcb
{
   char *node_name;
   bool *cautious_mode;
   int cautious_mode_type;
   int cautious_mode_module;
   char *cautious_mode_src;
   tcb *cautious_mode_tcb;

   double *cautious_vel;
   int cautious_vel_type;
   int cautious_vel_module;
   char *cautious_vel_src;
   tcb *cautious_vel_tcb;

   double *base_vel;
   int base_vel_type;
   int base_vel_module;
   char *base_vel_src;
   tcb *base_vel_tcb;

   double *max_vel;
   int max_vel_type;
   int max_vel_module;
   char *max_vel_src;
   tcb *max_vel_tcb;

   Vector *v;
   int v_type;
   int v_module;
   char *v_src;
   tcb *v_tcb;

   void update_run_status()
   {
      update_status(NULL);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(NULL);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DRIVE_W_SPIN();
};

struct T_STAY_ON_PATH : public tcb
{
   char *node_name;
   double *Path_Width;
   int Path_Width_type;
   int Path_Width_module;
   char *Path_Width_src;
   tcb *Path_Width_tcb;

   Vector *End;
   int End_type;
   int End_module;
   char *End_src;
   tcb *End_tcb;

   Vector *Start;
   int Start_type;
   int Start_module;
   char *Start_src;
   tcb *Start_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_STAY_ON_PATH();
};

struct T_VECTOR_COMPASS_DIFFERENTIAL : public tcb
{
   char *node_name;
   double *Tolerance;
   int Tolerance_type;
   int Tolerance_module;
   char *Tolerance_src;
   tcb *Tolerance_tcb;

   int *Compass_heading;
   int Compass_heading_type;
   int Compass_heading_module;
   char *Compass_heading_src;
   tcb *Compass_heading_tcb;

   double *Desired_heading;
   int Desired_heading_type;
   int Desired_heading_module;
   char *Desired_heading_src;
   tcb *Desired_heading_tcb;

   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_VECTOR_COMPASS_DIFFERENTIAL();
};

struct T_UNMARKED_DOORWAY : public tcb
{
   char *node_name;
   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_UNMARKED_DOORWAY();
};

struct T_UNMARK_DOORWAY : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_UNMARK_DOORWAY();
};

struct T_MARKED_DOORWAY : public tcb
{
   char *node_name;
   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MARKED_DOORWAY();
};

struct T_MARK_DOORWAY : public tcb
{
   char *node_name;
   Vector output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_MARK_DOORWAY();
};

struct T_DOORWAY_ULTRASOUND : public tcb
{
   char *node_name;
   int *unmarked;
   int unmarked_type;
   int unmarked_module;
   char *unmarked_src;
   tcb *unmarked_tcb;

   int *look_for;
   int look_for_type;
   int look_for_module;
   char *look_for_src;
   tcb *look_for_tcb;

   double *hallway_width;
   int hallway_width_type;
   int hallway_width_module;
   char *hallway_width_src;
   tcb *hallway_width_tcb;

   double *robot_length;
   int robot_length_type;
   int robot_length_module;
   char *robot_length_src;
   tcb *robot_length_tcb;

   bool output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DOORWAY_ULTRASOUND();
};

struct T_RAW_ULTRASONICS : public tcb
{
   char *node_name;
   raw_sonar_array output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_RAW_ULTRASONICS();
};

struct T_GET_COMPASS_HEADING : public tcb
{
   char *node_name;
   int output;
   void update_run_status()
   {
      update_status(&output);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(&output);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_GET_COMPASS_HEADING();
};

struct T_DRIVE : public tcb
{
   char *node_name;
   bool *cautious_mode;
   int cautious_mode_type;
   int cautious_mode_module;
   char *cautious_mode_src;
   tcb *cautious_mode_tcb;

   double *cautious_vel;
   int cautious_vel_type;
   int cautious_vel_module;
   char *cautious_vel_src;
   tcb *cautious_vel_tcb;

   double *base_vel;
   int base_vel_type;
   int base_vel_module;
   char *base_vel_src;
   tcb *base_vel_tcb;

   double *max_vel;
   int max_vel_type;
   int max_vel_module;
   char *max_vel_src;
   tcb *max_vel_tcb;

   Vector *v;
   int v_type;
   int v_module;
   char *v_src;
   tcb *v_tcb;

   void update_run_status()
   {
      update_status(NULL);
   }
   bool will_use_port(void *port)
   {
      _USE(port); // Get rid of unused parm warnings
      if( status_ == STAT_UNKNOWN )
         update_status(NULL);
      return status_ == STAT_RUNNING;
   }
   virtual ~T_DRIVE();
};
extern char *FRM_Query;
typedef struct PCK_type_pry {int valid; type_pry val;} PCK_type_pry;
extern char *FRM_type_pry;
extern char *MESG_type_pry;
typedef struct PCK_OneObjectClass {int valid; OneObjectClass val;} PCK_OneObjectClass;
extern char *FRM_OneObjectClass;
extern char *MESG_OneObjectClass;
typedef struct PCK_ObjectClasses {int valid; ObjectClasses val;} PCK_ObjectClasses;
extern char *FRM_ObjectClasses;
extern char *MESG_ObjectClasses;
typedef struct PCK_ObjectReading {int valid; ObjectReading val;} PCK_ObjectReading;
extern char *FRM_ObjectReading;
extern char *MESG_ObjectReading;
typedef struct PCK_ObjectList {int valid; ObjectList val;} PCK_ObjectList;
extern char *FRM_ObjectList;
extern char *MESG_ObjectList;
typedef struct PCK_gt_Command {int valid; gt_Command val;} PCK_gt_Command;
extern char *FRM_gt_Command;
extern char *MESG_gt_Command;
typedef struct PCK_boolean {int valid; boolean val;} PCK_boolean;
extern char *FRM_boolean;
extern char *MESG_boolean;
typedef struct PCK_bool {int valid; bool val;} PCK_bool;
extern char *FRM_bool;
extern char *MESG_bool;
typedef struct PCK_string {int valid; string val;} PCK_string;
extern char *FRM_string;
extern char *MESG_string;
typedef struct PCK_gt_Unit_Members {int valid; gt_Unit_Members val;} PCK_gt_Unit_Members;
extern char *FRM_gt_Unit_Members;
extern char *MESG_gt_Unit_Members;
typedef struct PCK_gt_Formation {int valid; gt_Formation val;} PCK_gt_Formation;
extern char *FRM_gt_Formation;
extern char *MESG_gt_Formation;
typedef struct PCK_robot_position {int valid; robot_position val;} PCK_robot_position;
extern char *FRM_robot_position;
extern char *MESG_robot_position;
typedef struct PCK_Robots {int valid; Robots val;} PCK_Robots;
extern char *FRM_Robots;
extern char *MESG_Robots;
typedef struct PCK_Vector {int valid; Vector val;} PCK_Vector;
extern char *FRM_Vector;
extern char *MESG_Vector;
typedef struct PCK_raw_sonar_array {int valid; raw_sonar_array val;} PCK_raw_sonar_array;
extern char *FRM_raw_sonar_array;
extern char *MESG_raw_sonar_array;
typedef struct PCK_obs_array {int valid; obs_array val;} PCK_obs_array;
extern char *FRM_obs_array;
extern char *MESG_obs_array;
typedef struct PCK_vector_array {int valid; vector_array val;} PCK_vector_array;
extern char *FRM_vector_array;
extern char *MESG_vector_array;
typedef struct PCK_signed {int valid; signed val;} PCK_signed;
extern char *FRM_signed;
extern char *MESG_signed;
typedef struct PCK_unsigned {int valid; unsigned val;} PCK_unsigned;
extern char *FRM_unsigned;
extern char *MESG_unsigned;
typedef struct PCK_double {int valid; double val;} PCK_double;
extern char *FRM_double;
extern char *MESG_double;
typedef struct PCK_float {int valid; float val;} PCK_float;
extern char *FRM_float;
extern char *MESG_float;
typedef struct PCK_long {int valid; long val;} PCK_long;
extern char *FRM_long;
extern char *MESG_long;
typedef struct PCK_short {int valid; short val;} PCK_short;
extern char *FRM_short;
extern char *MESG_short;
typedef struct PCK_char {int valid; char val;} PCK_char;
extern char *FRM_char;
extern char *MESG_char;
typedef struct PCK_int {int valid; int val;} PCK_int;
extern char *FRM_int;
extern char *MESG_int;

extern "C" {
   void *malloc(size_t);
   void exit(int);
}

extern void DRIVE(int parm);
extern void GET_COMPASS_HEADING(int parm);
extern void RAW_ULTRASONICS(int parm);
extern void DOORWAY_ULTRASOUND(int parm);
extern void MARK_DOORWAY(int parm);
extern void MARKED_DOORWAY(int parm);
extern void UNMARK_DOORWAY(int parm);
extern void UNMARKED_DOORWAY(int parm);
extern void VECTOR_COMPASS_DIFFERENTIAL(int parm);
extern void STAY_ON_PATH(int parm);
extern void DRIVE_W_SPIN(int parm);
extern void MOVE_ROBOT(int parm);
extern void MOVE_AHEAD(int parm);
extern void MOVE_TO_GOAL(int parm);
extern void MOVE_TO(int parm);
extern void SHAFTENCODERS(int parm);
extern void DETECT_GOAL_WITH_SHAFT_ENCODERS(int parm);
extern void GET_ROBOT_STATE(int parm);
extern void V_IS_ZERO(int parm);
extern void LAST_BUTTON_PRESS(int parm);
extern void PUT_ROBOT_STATE(int parm);
extern void GLOBAL_TO_EGOCENTRIC(int parm);
extern void IN_ROOM(int parm);
extern void IN_HALLWAY(int parm);
extern void ENTER_ROOM(int parm);
extern void LEAVE_ROOM(int parm);
extern void ENTER_ALTERNATE_HALLWAY(int parm);
extern void DETECT_ALTERNATE_HALLWAY(int parm);
extern void DOCK(int parm);
extern void SUM_WEIGHTED_VECTORS(int parm);
extern void COOP(int parm);
extern void GET_XY(int parm);
extern void GET_HEADING(int parm);
extern void GET_LOCATION(int parm);
extern void DETECT_ROBOTS(int parm);
extern void NOISE(int parm);
extern void LAST_KEY_PRESS(int parm);
extern void PROBE(int parm);
extern void ALERT(int parm);
extern void ALERTED_ONCE(int parm);
extern void DETECT_OBSTACLES(int parm);
extern void ULTRASONICS(int parm);
extern void AVOID_STATIC_OBSTACLES(int parm);
extern void AVOID_ROBOT(int parm);
extern void DETECT_CENTER_OF_MASS(int parm);
extern void DETECT_FORM_HEADING(int parm);
extern void DETECT_FORM_POSITION(int parm);
extern void COMPUTE_FORM_POSITION(int parm);
extern void GET_HALLWAY_INFO_VECTOR(int parm);
extern void GET_HALLWAY_INFO_DOUBLE(int parm);
extern void DETECT_IN_POSITION(int parm);
extern void DETECT_LEADER_CENTER(int parm);
extern void DETECT_UNIT_CENTER(int parm);
extern void DETECT_UNIT_CENTER_GLOBAL(int parm);
extern void DATABASE_DOUBLE(int parm);
extern void DATABASE_INT(int parm);
extern void DATABASE_BOOLEAN(int parm);
extern void TEST_DISTRIBUTED_FLAG(int parm);
extern void UPDATE_DISTRIBUTED_FLAG(int parm);
extern void SWIRL_STATIC_OBSTACLES(int parm);
extern void REMOVE_ROBOTS(int parm);
extern void MAINTAIN_FORM(int parm);
extern void SUBTRACT_VECTORS(int parm);
extern void MDL_coordinator(int parm);
extern void AND(int parm);
extern void NOT(int parm);
extern void TELOP(int parm);
extern void SOUND(int parm);
extern void DETECT_SOUND(int parm);
extern void DETECT_SOUND_MOTIVATED(int parm);
extern void TALK(int parm);
extern void EXTRACT_GOAL(int parm);
extern void DETECT_OBJECTS(int parm);
extern void FILTER_OBJECT_CLASSES(int parm);
extern void FILTER_OBJECTS(int parm);
extern void IS_AN_OBJECT(int parm);
extern void IS_AN_OBJECT_MOTIVATED(int parm);
extern void CLOSEST_OBJECT(int parm);
extern void CLOSEST_OBSTACLE(int parm);
extern void IS_AT_GOAL(int parm);
extern void PICKUP_OBJECT(int parm);
extern void OBJECT_LOCATION(int parm);
extern void AVOID_OBJECTS(int parm);
extern void AVOID_OBSTACLES(int parm);
extern void AVOID_OBSTACLES_MEM(int parm);
extern void CONSTANT(int parm);
extern void DEBUG_PROC(int parm);
extern void DEBUG_SHOW_INPUT(int parm);
extern void HALT(int parm);
extern void NULL_MS(int parm);
extern void NOOP(int parm);
extern void DROP_IN_BASKET(int parm);
extern void FIRST_TIME(int parm);
extern void MOVE_TO_FREE_SPACE(int parm);
extern void MARK_OBJECT(int parm);
extern void WAIT(int parm);
extern void ISFACING(int parm);
extern void HAS_TURNED(int parm);
extern void MOVED_DISTANCE(int parm);
extern void SPIN(int parm);
extern void SETXY(int parm);
extern void SETXYTHETA(int parm);
extern void FILTER_OBJECTS_BY_CLASS(int parm);
extern void ECHO_OBJECTS(int parm);
extern void COMPUTE_UNIT_CENTER(int parm);
extern void PROBE_MOVABLE(int parm);
extern void PROBE_OBJECT(int parm);
extern void ADD_OBJECT(int parm);
extern void IS_VALID_OBJECT(int parm);
extern void MAP_LOCATION(int parm);
extern void EQUAL_INT(int parm);
extern void DROP_OBJECT(int parm);
extern void GET_PRY(int parm);
extern void GET_YAW(int parm);
extern void GET_UPHILL(int parm);
extern void SCALE_VECTOR(int parm);
extern void DEBUG_SHOW_INPUT_PRY(int parm);
extern void COUNT_OBJECTS(int parm);
extern void SUM(int parm);
extern void PANTILT(int parm);
extern void LOOK_TO(int parm);
extern void IS_LOOKING_AT_DEGREE(int parm);

   T_ALERTED_ONCE::~T_ALERTED_ONCE() {};

void _cb_ALERTED_ONCE(IPCommunicator *comm, IPMessage *msg, void *hndData)
{
   struct T_ALERTED_ONCE *parms = (struct T_ALERTED_ONCE *)hndData;
   PCK_bool out;

   out.valid = parms->output_valid_;
   out.val = parms->output;

   comm->Reply( msg, MESG_bool, &out);
}


void ALERTED_ONCE(int parm)
{
   bool output;
   struct T_ALERTED_ONCE *parms = (struct T_ALERTED_ONCE *)parm;

   // node_name is the name of this node instance (AN_xxx) 
   const char *node_name = parms->node_name;
   USE(node_name);


   /******** load any constants ********/
   /******** start of once code ********/
#line 10 "ALERTED_ONCE.cnl"

      /******** End of once code ********/
   while(1)
   {
      // Loop here while we are blocked
      do {
         parms->done_ = true;
         mutex_lock(_done_count_lock);
         _done_count++;
         if(_done_count >= _num_nodes) 
         {
            extern void end_of_cycle(void);
            _done_count = 0;
            end_of_cycle();	/* user function hook */
            reset_all_tasks();	/* clear the status flags */
            update_all_tasks();	/* reset the status flags */
            condition_broadcast(_new_cycle);
            mutex_unlock(_done_count_lock);
            continue; /* skip the condition wait */
         }
         condition_wait(_new_cycle,_done_count_lock);
         mutex_unlock(_done_count_lock);
      } while( parms->blocked() ); 
      /******** start of user header ********/
#line 10 "ALERTED_ONCE.cnl"


#line 4100 "ALERTED_ONCE.cc"
      /******** end of user header ********/
      while(!parms->blocked())
      {
      //   memset(&output,0,sizeof(output));
         bool ok_to_run = true;

         /* if using ipt, run any waiting message handlers */
         if( _ipt_is_active )
            _communicator->Idle( _zero_time);


         if( ok_to_run )
         {
            if( debug )
               printf("%s: Running\n",parms->task_name);
            {/******** start of user body ********/
#line 11 "ALERTED_ONCE.cnl"
       // Toggles output depends on whether robot has
       // sent an alert message or not.
       if (alerted_once)
       {
          output = true;
          alerted_once = FALSE;
       }
       else
          output = false;
          
#line 4128 "ALERTED_ONCE.cc"

            }/******** end of user code ********/

            parms->output = output;
            parms->output_valid_ = true;
         }

         parms->done_ = true;
         mutex_lock(_done_count_lock);
         _done_count++;
         if(_done_count >= _num_nodes) 
         {
            extern void end_of_cycle(void);
            _done_count = 0;
            end_of_cycle();	/* user function hook */
            reset_all_tasks();	/* clear the status flags */
            update_all_tasks();	/* reset the status flags */
            condition_broadcast(_new_cycle);
            mutex_unlock(_done_count_lock);
            continue; /* skip the condition wait */
         }
         condition_wait(_new_cycle,_done_count_lock);
         mutex_unlock(_done_count_lock);
      } /* while not blocked */
   } /* end of main while loop */
};

/* End of procedures */
