/*********************************************************************
 **                                                                  **
 **                           gt_3d_display.c                        **
 **                                                                  **
 **                                                                  **
 **  Written by:  Jonathan M. Cameron                                **
 **                                                                  **
 **  Copyright 1995, 1996, 1997 Georgia Tech Research Corporation    **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: gt_3d_display.c,v 1.6 2003/04/06 12:44:19 endo Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <iostream>

#include "gt_sim.h"

#include "gt_3d_display.h"
#include "3d_sphigs_display.h"
#include "gt_simulation.h"


/**********************************************************************
 **           initialize_3d            **
 ****************************************/

void initialize_3d()
{

// printf("initialize_3d  called\n");

  /* init the display */

  init_display( 0 /*bb.display.for_overlay*/, 
                0 /*debug*/, 
		1 /*bb.display.solid*/, 
                1 /*bb.display.color*/);

}

/**********************************************************************
 **           close_3d            **
 ****************************************/

void close_3d()
{
//  printf("close_3d  called\n");
  exit_display();
}

void restart_3d()
{
//  printf("restart_3d  called\n");
  restart_display();
}

/**********************************************************************
 **           gt_add_boundary            **
 ****************************************/

void add_3d_boundary(char *name, gt_Point_list *pt)
{
  make_outline_of_room (name, pt); 
//  printf(" add_3d_boundary  called\n");
}

/**********************************************************************
 **           render_3d            **
 ****************************************/

void render_3d()
{
//  printf(" render_3d\n");
  draw_room();
}
/**********************************************************************
 **           add_3d_passage_point            **
 ****************************************/

void add_3d_passage_point(char *name, float x, float y, float diameter)
{
  add_3d_passage_point_sphigs (name, x, y, diameter); 
//  printf("add_3d_passage_point\n");
}


/**********************************************************************
 **           add_3d_obstacle            **
 ****************************************/

void add_3d_obstacle(double x, double y, double radius)
{
  add_3d_obstacle_sphigs(x,y,radius); 
//  printf("add_3d_obstacle\n");
}

/**********************************************************************
 **           add_3d_object            **
 ****************************************/

void add_3d_object(double x, double y, double radius, char *color)
{
  add_3d_object_sphigs(x,y,radius,color); 
//  printf("add_3d_object\n");
}

/**********************************************************************
 **           move_3d_robot            **
 ****************************************/

void move_3d_robot(double x, double y, double heading, int color, int is_up)
{
  moveto(x,y,heading,color, is_up);
} 

void remove_3d_object(double x, double y)
{
  remove_3d_object_sphigs(x,y);
//  printf("remove_3d_object\n");
}

/**********************************************************************
 * $Log: gt_3d_display.c,v $
 * Revision 1.6  2003/04/06 12:44:19  endo
 * gcc 3.1.1
 *
 * Revision 1.5  1997/02/12  05:42:50  zchen
 * *** empty log message ***
 *
 * Revision 1.4  1997/01/15  21:31:06  zchen
 * add real 3d stuff
 *
 * Revision 1.2  1995/10/23  18:21:57  doug
 * Extend object stuff to support obstacles and also add 3d code
 *
 * Revision 1.1  1995/10/23  16:42:19  doug
 * Initial revision
 *
 **********************************************************************/




