/**********************************************************************
 **                                                                  **
 **                          mission_design.h                        **
 **                                                                  **
 **  Written by:  Yoichiro Endo                                      **
 **                                                                  **
 **  Copyright 2002 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: mission_design.h,v 1.3 2003/04/06 12:44:19 endo Exp $ */

#ifndef MISSION_DESIGN_H
#define MISSION_DESIGN_H

#include <vector>
#include <string>
#include "gt_simulation.h"
#include "mission_expert.h"

using std::vector;
using std::string;

class MlabMissionDesign {

    typedef enum MMDMouseButtonNum_t {
	NO_BTN,
	LEFT_BTN,
	MIDDLE_BTN,
	RIGHT_BTN
    };

    typedef enum MMDExitType_t {
	MMD_EXIT_CANCEL_RETURN,
	MMD_EXIT_CONFIGURE
    };

    typedef struct MMDPixmap_t {
	string name;
	Pixmap pixmap;
	Pixmap insensitivePixmap;
    };

    typedef struct MMDGlobalFeatureMenuCallbackData_t {
	int globalFeatureId;
	int optionType;
	float optionValue;
	MlabMissionDesign *mlabMissionDesignInstance;
    };

    typedef struct MMDRightBtnMenuCallbackData_t {
	int featureId;
	int optionType;
	float optionValue;
	MlabMissionDesign *mlabMissionDesignInstance;
    };

    typedef struct MMDToolBoxSetModeCallbackData_t {
	int mode;
	MlabMissionDesign *mlabMissionDesignInstance;
    };

    typedef struct GeneralYesNoWindowCallbackData_t {
	bool result;
	MlabMissionDesign *mlabMissionDesignInstance;
    };

protected:
    Display *MMDDisplay_;
    XtAppContext MMDAppContext_;
    FILE *featureFile_;
    FILE *featureStatusFile_;
    Pixel MMDParentFg_;
    Pixel MMDParentBg_;
    Pixel MMDToolboxBtnBgColorSet_;
    Pixel MMDToolboxBtnBgColorUnset_;
    Widget MMDParent_;
    Widget MMDDrawingArea_;
    Widget Biohazard_btn_;
    Widget Waypoints_btn_;
    Widget Mine_btn_;
    Widget Other_btn_;
    Widget *task_btn_;
    Widget toolbox_w_;
    Widget globalFeatureMenu_w_;
    Widget **globalFeatureMenuOption_w_;
    Widget rightBtnMenu_w_;
    Widget **rightBtnMenuOption_w_;
    MExpMissionTask_t rightBtnMenuUpTask_;
    vector<MExpFeature_t> globalFeatures_;
    vector<MExpFeature_t> defaultLocalFeatures_;
    vector<MExpMissionTask_t> defaultTaskList_;
    vector<MExpMissionTask_t> specifiedTaskList_;
    vector<MMDPixmap_t> pixmapList_;
    string featureFilename_;
    string featureStatusFilename_;
    string overlayFilename_;
    string defaultTaskName_;
    double drawingUnitLength_;
    double taskPointRadius_;
    double defaultTaskWeight_;
    int MMDMode_;
    int numTaskTypes_;
    int numFeatureTypes_;
    int lastMouseXRoot_, lastMouseYRoot_;
    int numTasks_;
    int rightBtnMenuLineHeight_;
    int returnConfigureMission_;
    unsigned int MMDMissionAreaHeightPixels_;
    bool windowsCreated_, rightBtnMenuIsUp_;
    bool generalYesNoWindowResult_;
    bool generalYesNoWindowDone_;

    static const int DEFAULT_TASKPOINT_RADIUS_;
    static const int DEFAULT_TOOLBOX_WIDTH_;
    static const int DEFAULT_TOOLBOX_XPOS_;
    static const int DEFAULT_TOOLBOX_YPOS_;
    static const int DEFAULT_TOOLBOX_ACTIONAREA_HEIGHT_;
    static const int DEFAULT_TOOLBOX_EXTRASPACE_HEIGHT_;
    static const int DEFAULT_TOOLBOX_BUTTON_HEIGHT_;
    static const int DEFAULT_TOOLBOX_BUTTON_WIDTH_;
    static const int DEFAULT_TOOLBOX_BUTTON_EXTRASPACE_HEIGHT_;
    static const int DEFAULT_TOOLBOX_NUM_COLUMNS_;
    static const int DEFAULT_RIGHTBTN_WIDTH_;
    static const int DEFAULT_RIGHTBTN_LINEHEIGHT_;
    static const int DEFAULT_RIGHTBTN_EDGEOFFSET_;
    static const int DEFAULT_GLOBALFEATURE_WIDTH_;
    static const int DEFAULT_GLOBALFEATURE_LINEHEIGHT_;
    static const int DEFAULT_GLOBALFEATURE_XEDGEOFFSET_;
    static const int DEFAULT_GLOBALFEATURE_YPOS_;
    static const string TOOLBOX_OK_BUTTON_LABEL_;
    static const string TOOLBOX_CANCEL_BUTTON_LABEL_;
    static const string TOOLBOX_GLOBALSETTING_BUTTON_LABEL_;
    static const string EMPTY_STRING_;

    void saveData_(void);
    int loadData_(void);
    void createToolbox_(void);
    void updateToolboxBtn_(void);
    Pixel toolboxBtnBgColor_(int button);
    void addNewTask_(double taskPosX, double taskPosY, int taskType, MExpMissionTask_t *addedTask);
    void placeTask_(double taskPosX, double taskPoxY, int taskType);
    void removeTask_(double taskPosX, double taskPoxY);
    void createGlobalFeatureMenu_(void);
    void popUpRightBtnMenu_(double taskPosX, double taskPoxY);
    void createRightBtnMenu_(MExpMissionTask_t task);
    bool isInsideTask_(double x, double y, double unitLength, MExpMissionTask_t *task);
    void changeGlobalFeatureMenuValue_(XtPointer client_data);
    void applyRightBtnMenu_(void);
    void cancelRightBtnMenu_(void);
    void closeRightBtnMenu_(void);
    void changeRightBtnMenuValue_(XtPointer client_data);
    void setMode_(int mode);
    void setFeatureFileStatus_(void);
    Widget createGeneralYesNoWindow_(char *question);
    void popupGeneralYesNoWindow_(Widget w);
    void sendGeneralYesNoWindowResult_(bool result);
    bool confirmEmptyMission_(void);
    void confirmNoRC_(void);
    bool loadFeaturesFromRCFile_(void);
    void createPixmapList_(Pixel fg, Pixel bg, Widget parent);
    int getDefaultMode_(void);
    Pixmap getPixmapFromList_(string name, bool insensitive);

    // Callbacks
    static void cbSetmode_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbChangeGlobalFeatureMenuValue_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbApplyRightBtnMenu_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbCancelRightBtnMenu_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbChangeRightBtnMenuValue_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbExit_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbGeneralYesNoWindow_(Widget w, XtPointer client_data, XtPointer callback_data);
    static void cbToolboxGlobalSettingBtn_(Widget w, XtPointer client_data, XtPointer callback_data);

public:
    MlabMissionDesign(
	Display *display,
	Widget parent,
	XtAppContext app,
	Widget drawing_area,
	char *featureFileString,
	char **defaultOverlayName);
    
    ~MlabMissionDesign(void);
    void createWindows(void);
    void mouseClick(double x, double y, XEvent *event, double unitLength);
    void refreshTaskEntry(double unitLength);
    void saveOverlayName(char *ovlFilename);
    bool windowsAlreadyCreated(void);
};

inline bool MlabMissionDesign::windowsAlreadyCreated(void){return windowsCreated_;};

extern MlabMissionDesign *gMMD;
extern bool gUseMlabMissionDesign;

#endif

/**********************************************************************
 * $Log: mission_design.h,v $
 * Revision 1.3  2003/04/06 12:44:19  endo
 * gcc 3.1.1
 *
 * Revision 1.2  2003/04/06 09:03:48  endo
 * Updated for CBR Wizard Prototype II.
 *
 * Revision 1.1  2002/01/12 22:54:39  endo
 * Initial revision
 *
 **********************************************************************/
