/**********************************************************************
 **                                                                  **
 **                            obstacle.hpp                          **
 **                                                                  **
 **  class of obstacle shapes                                        **
 **                                                                  **
 **                                                                  **
 **  Written by:  Douglas C. MacKenzie                               **
 **                                                                  **
 **  Copyright 1995, 1996, 1997 Georgia Tech Research Corporation    **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **********************************************************************/

/* $Id: obstacle.hpp,v 1.10 2000/10/16 19:39:40 endo Exp $ */


#ifndef OBSTACLE_HPP
#define OBSTACLE_HPP

#include "draw.h"
#include "circle.hpp"

class obstacle : public circle {
public:
   obstacle(double x, double y, double r, const char *color)
   {
      x_ = x;
      y_ = y;
      r_ = r;
      color_ = strdup(color);
      gc_ = lookup_color(color);
      assert( gc_ );
   }

   ~obstacle()
   {  
      free((void *)color_);
   }

   void draw()
   { 
      DrawFilledCircle(x_, y_, r_, gc_);
   }

   void erase()
   { 
      DrawFilledCircle(x_, y_, r_, eraseGC);
   }

   ObjectReading to_reading(double x, double y, double heading)
   { 
      ObjectReading rtn;
      Vector  v;

      // Make position egocentric
      v.x = x_ - x;
      v.y = y_ - y;
      rotate_z(v, -1 * heading);

      rtn.setColor((char *)color_);
      rtn.id = id_;
      rtn.objshape = CircleObject;
      rtn.x1 = v.x;
      rtn.y1 = v.y;
      rtn.r = r_;
      rtn.ismovable = ismovable();

      // Unused
      rtn.x2 = 0;
      rtn.y2 = 0;

      return rtn;
   }

   bool ismovable() { return false; }

   void bind(shape *owner)
   { 
      warn_once("This object can not be moved");         
   }

   void unbind(shape *owner)
   { 
      warn_once("Hmm, how is the robot carrying an unmovable object?");         
   }

   gt_Point whereis()
   { 
      gt_Point rtn;

      rtn.x = x_;
      rtn.y = y_;

      return rtn;
   }

   bool is_up()
   { 
      return false;
   }
};

#endif


///////////////////////////////////////////////////////////////////////
// $Log: obstacle.hpp,v $
// Revision 1.10  2000/10/16 19:39:40  endo
// Modified due to the compiler upgrade.
//
// Revision 1.9  1999/12/16 22:50:05  mjcramer
// rh6 port
//
// Revision 1.8  1997/02/12  05:41:37  zchen
// *** empty log message ***
//
// Revision 1.7  1996/03/04  00:04:30  doug
// *** empty log message ***
//
// Revision 1.6  1996/02/28  03:54:51  doug
// *** empty log message ***
//
// Revision 1.5  1996/02/27  05:00:04  doug
// *** empty log message ***
//
// Revision 1.4  1996/02/16  00:06:44  doug
// *** empty log message ***
//
// Revision 1.3  1996/02/08  19:24:07  doug
// handle objects of any color
//
// Revision 1.2  1995/11/07  14:31:57  doug
// added baskets and command to drop oranges into them
//
// Revision 1.1  1995/11/06  18:41:44  doug
// Initial revision
//
///////////////////////////////////////////////////////////////////////
