
/**********************************************************************
 **                                                                  **
 **                            robot.hpp                             **
 **                                                                  **
 **  class of robot shapes                                           **
 **                                                                  **
 **                                                                  **
 **  Written by:  Douglas C. MacKenzie                               **
 **                                                                  **
 **  Copyright 1995, 1996, 1997 Georgia Tech Research Corporation    **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **********************************************************************/

/* $Id: robot.hpp,v 1.8 2002/10/31 20:05:29 ebeowulf Exp $ */


#ifndef ROBOT_HPP
#define ROBOT_HPP

#include "circle.hpp"

class robot : public circle {
public:

//this is needed for using a different color for robots which are 
//in 'line-of-sight'

GC LOSgc;
bool LOS;

   robot(double x, double y, double r, const char *color)
   {
      x_ = x;
      y_ = y;
      r_ = r;
      color_ = strdup(color);
      gc_ = lookup_color(color);
      assert( gc_ );

      // Make it xor style
      XSetFunction(XtDisplay(drawing_area), gc_, GXxor);

//this is needed for using a different color for robots which are 
//in 'line-of-sight'

      LOS = FALSE;
      LOSgc = lookup_color("yellow");
      assert( LOSgc );

      // Make it xor style
      XSetFunction(XtDisplay(drawing_area), LOSgc, GXxor);

   }

   ~robot()
   {
      free((void *)color_);
   }

   void change_color(const char *color)
   {
      color_ = strdup(color);
      gc_ = lookup_color(color);
      assert( gc_ );
      XSetFunction(XtDisplay(drawing_area), gc_, GXxor);
   }

   void draw()
   { 
//      DrawFilledCircle(x_, y_, r_, theRobotGC);
   }

   void erase()
   { 
//      DrawFilledCircle(x_, y_, r_, eraseGC);
   }

   ObjectReading to_reading(double x, double y, double heading)
   { 
      ObjectReading rtn;
      Vector  v;

      // Make position egocentric
      v.x = x_ - x;
      v.y = y_ - y;
      rotate_z(v, -1 * heading);

      rtn.setColor((char *)color_);
      rtn.id = id_;
      rtn.objshape = CircleObject;
      rtn.x1 = v.x;
      rtn.y1 = v.y;
      rtn.r = r_;
      rtn.ismovable = ismovable();

      // Unused
      rtn.x2 = 0;
      rtn.y2 = 0;

      return rtn;
   }

   bool ismovable() { return true; }

   void bind(shape *owner)
   { 
      cerr << "Can't pickup robots\n";
   }

   void unbind(shape *owner)
   { 
      cerr << "Can't pickup robots\n";
   }

   gt_Point whereis()
   { 
      gt_Point rtn;

      rtn.x = x_;
      rtn.y = y_;

      return rtn;
   }

   bool is_up()
   { 
      return false;
   }

   virtual bool is_robot() const    {return true;} // is this a robot?

};

#endif


///////////////////////////////////////////////////////////////////////
// $Log: robot.hpp,v $
// Revision 1.8  2002/10/31 20:05:29  ebeowulf
// Added the ability to change the color of a robot.
//
// Revision 1.7  2000/11/20 19:33:54  sgorbiss
// Add visualization of line-of-sight communication
//
// Revision 1.6  2000/10/16 19:39:40  endo
// Modified due to the compiler upgrade.
//
// Revision 1.5  1999/12/16 22:50:21  mjcramer
// rh6 port
//
// Revision 1.4  1997/02/12  05:41:37  zchen
// *** empty log message ***
//
// Revision 1.3  1996/03/04  22:51:58  doug
// *** empty log message ***
//
// Revision 1.2  1996/02/27  05:00:04  doug
// *** empty log message ***
//
// Revision 1.1  1996/02/26  16:04:47  doug
// Initial revision
//
///////////////////////////////////////////////////////////////////////
