
/**********************************************************************
 **                                                                  **
 **                            telop_hw_joystick.c                   **
 **                                                                  **
 **  Written by: Michael Langford                                    **                                     
 **                                                                  **
 **  Copyright 2003 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: telop_hw_joystick.c,v 1.1 2003/04/02 21:24:55 zkira Exp $ */

#define _RENTRANT
#include <pthread.h>

#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <linux/joystick.h>
#include "telop_hw_joystick.h"
#include "telop_joystick.h"
#include "status_window.h" // zk trw

#define WAIT_INTERVAL_US 0L
#define MODNUM 2

#define NAME_LENGTH 128
#define STICK_PORT "/dev/js0"
#define XMIN -27917L
#define XMAX  27917L
#define YMIN -24916L
#define YMAX  24916L

int axial_report_counter=0;

int thread_state=0;

int axis[6];
int button[7];
pthread_t the_thread=0;
int bKeepGoing =0;

void (*axis_callback)(int axis[6]) = 0;

static void *jq_connect_thread(void *unused);

void jq_wait_us(unsigned long time_in_us);


long jq_get_xmax(void)
{
	return XMAX;
}

long jq_get_xmin(void)
{
	return XMIN;
}

long jq_get_ymax(void)
{
	return YMAX;
}

long jq_get_ymin(void)
{
	return YMIN;
}

int jq_connect(void)
{
	return pthread_create(&the_thread,0,jq_connect_thread,0);
}


void *jq_get_axis_callback(void)
{
	return (void *)axis_callback;
}


int jq_register_axis_callback(void (*cb)(int axis[6]))
{
	int bGo;
	
	bGo = (0==axis_callback);

	axis_callback = cb;
	if(cb)
	{
		bKeepGoing = 1;
		if(bGo)jq_connect();
	}
	else
	{
		jq_kill_js_driver();
	}

	while (thread_state == 0) {
	}
	return (thread_state==2);
	
}

static void *jq_connect_thread(void *unused)
{
	int argc = 1;
	char ** argv = 0;

	int fd;
	unsigned char axes = 2;
	unsigned char buttons = 2;
	int version = 0x000800;
	char name[NAME_LENGTH] = "Unknown";

	if ((fd = open(STICK_PORT, O_RDONLY)) < 0) 
	{
		perror("libjoyQ");
		axis_callback = 0;
		thread_state = 1; /* unsuccessful joystick link */
		return NULL;
	}

	ioctl(fd, JSIOCGVERSION, &version);
	ioctl(fd, JSIOCGAXES, &axes);
	ioctl(fd, JSIOCGBUTTONS, &buttons);
	ioctl(fd, JSIOCGNAME(NAME_LENGTH), name);

	if (argc < 3 || !strcmp("--normal", STICK_PORT)) {

		int i;
		struct js_event js;


		while (bKeepGoing) {
			jq_wait_us(WAIT_INTERVAL_US);
			if(!bKeepGoing)
			{
				thread_state = 2; 
				return NULL;
			}

			if (read(fd, &js, sizeof(struct js_event)) != sizeof(struct js_event)) {
				perror("\nlibjoyQ: error reading");
				axis_callback = 0;
				thread_state = 1; /* unsuccessful joystick link */
				return NULL;

			}
			thread_state = 2; /* successful joystick link */
			switch(js.type & ~JS_EVENT_INIT) {
			case JS_EVENT_BUTTON:

                                // zk trw - Show whether deadman switch is released or pressed                               
                                if (js.number == 0 && js.value == 1)
                                  arm_deadman();
                                else if (js.number == 0 && js.value == 0)
                                  disarm_deadman();
 
				button[js.number] = js.value;
				break;
			case JS_EVENT_AXIS:
				axis[js.number] = js.value;
				if(axis_callback) 
				{
						axis_callback(axis);
				}		
				else
				{
				  thread_state = 0; /* uninitialize joystick */
				  return NULL;
				}break;
			default:break;
			}
		}
	}

}

int jq_get_info(void)
{
	return axis[0];
}

int jq_kill_js_driver(void)
{
	bKeepGoing = 0;
	axis_callback = 0;
	thread_state = 0; /* uninitialize joystick */
	/* wait for the thread to die */
	return pthread_join(the_thread,0);
	

}



#include <sys/time.h>     /* struct timeval definition           */
#include <unistd.h>       /* declaration of gettimeofday()       */

void jq_wait_us(unsigned long time_in_us)
{
	struct timeval  now;            /* time when we started waiting        */
	struct timespec timeout;        /* timeout value for the wait function */
	pthread_mutex_t a_mutex = PTHREAD_MUTEX_INITIALIZER;
	pthread_mutex_t request_mutex = PTHREAD_MUTEX_INITIALIZER;
	pthread_cond_t got_request = PTHREAD_COND_INITIALIZER;
	int             done;           /* are we done waiting?                */

	/* first, lock the mutex */
	int rc = pthread_mutex_lock(&a_mutex);
	if (rc) { /* an error has occurred */
	    perror("pthread_mutex_lock");
	    pthread_exit(NULL);
	}
	/* mutex is now locked */

	/* get current time */ 
	if(0!=gettimeofday(&now,0)) printf("Failed time aquire!");

	/* prepare timeout value */
	timeout.tv_sec = now.tv_sec;
	timeout.tv_nsec = (now.tv_usec + time_in_us) * 1000L;

	/* wait on the condition variable. */
	/* we use a loop, since a Unix signal might stop the wait before the timeout */
	done = 0;
	while (!done) 
	{
	    /* remember that pthread_cond_timedwait() unlocks the mutex on entrance */
	    rc = pthread_cond_timedwait(&got_request, &request_mutex, &timeout);
	    switch(rc) 
	    {
	        case ETIMEDOUT: /* our time is up */
        	    done = 1;
	            break;
	        default:        /* some error occurred (e.g. we got a Unix signal) */
	            break;      /* break this switch, but re-do the while loop.   */
	    }
	
	}
	
	/* finally, unlock the mutex */
	pthread_mutex_unlock(&request_mutex);
}

/**********************************************************************
 * $Log: telop_hw_joystick.c,v $
 * Revision 1.1  2003/04/02 21:24:55  zkira
 * Initial revision
 *
 **********************************************************************/
